Package: svySE
Type: Package
Title: Sampling Error Estimation for Complex Surveys
Version: 0.2.0
Authors@R: 
    person(
        given = "Luis",
        family = "Burgos",
        email = "lburgoss1996@gmail.com",
        role = c("aut", "cre"),
        comment = c(
            ORCID = "0009-0008-8290-4228"
        )
    )
Description: Estimates sampling errors and produces indicator tables for
    complex survey data. Supports weighted totals, proportions, standard
    errors, confidence intervals, coefficients of variation, design effects,
    unweighted frequencies, grouped estimates, domain estimates, optional
    stratification and clustering variables, and customizable exports to
    '.xlsx' files. Survey estimation is based on design-based inference using
    Taylor series linearization implemented in the 'survey' package (Lumley,
    2004, <doi:10.18637/jss.v009.i08>; Lumley, 2010,
    ISBN:9780470284308). The package provides a reproducible workflow for
    official statistics, household surveys, and applied survey research.
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
Depends: R (>= 4.2.0)
Imports: survey, openxlsx, stats
Suggests: testthat (>= 3.0.0), knitr, rmarkdown
VignetteBuilder: knitr
Config/testthat/edition: 3
URL: https://github.com/lburgoss/svySE
BugReports: https://github.com/lburgoss/svySE/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-13 23:22:39 UTC; encal154
Author: Luis Burgos [aut, cre] (ORCID: <https://orcid.org/0009-0008-8290-4228>)
Maintainer: Luis Burgos <lburgoss1996@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-14 14:40:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(print,svySE_cfg)
S3method(print,svySE_result)
export(svySE_calc)
export(svySE_cfg)
export(svySE_cols_err)
export(svySE_cols_err_all)
export(svySE_cols_tab)
export(svySE_cols_tab_all)
export(svySE_xlsx)
importFrom(stats,coef)
importFrom(stats,confint)
importFrom(stats,reformulate)
importFrom(stats,update)
importFrom(utils,head)

<p align="center">
  <img src="man/figures/logo.png" width="260" alt="svySE logo">
</p>

# svySE

[![R-CMD-check](https://github.com/lburgoss/svySE/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/lburgoss/svySE/actions/workflows/R-CMD-check.yaml)
[![CRAN status](https://www.r-pkg.org/badges/version/svySE)](https://CRAN.R-project.org/package=svySE)
[![CRAN downloads](https://cranlogs.r-pkg.org/badges/grand-total/svySE)](https://cran.r-project.org/package=svySE)
![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)

**Sampling Error Estimation for Complex Surveys**

`svySE` is an R package for estimating sampling errors, producing descriptive indicator tables, and exporting structured results from complex survey data.

The package provides two complementary workflows:

- `svySE_calc()` calculates weighted estimates and sampling errors using the survey design.
- `svySE_simple()` calculates unweighted frequencies and percentages from the observed sample.

Results can be exported individually or consolidated across multiple datasets, survey weights, indicators, or analysis runs using `svySE_xlsx()`.

`svySE` is built on top of the **survey** package and provides a higher-level workflow for the routine production of survey indicators while preserving the principles of design-based estimation.

---

## Why svySE?

Survey indicator production often involves repeating the same technical steps:

1. preparing indicator variables;
2. identifying grouping and domain variables;
3. defining the survey design;
4. calculating weighted estimates;
5. estimating standard errors and confidence intervals;
6. calculating CV and DEFF;
7. preparing descriptive tables;
8. exporting results to structured workbooks.

`svySE` organizes these tasks into a reproducible and configurable workflow.

The package is designed to be:

- reproducible;
- flexible;
- easy to configure;
- suitable for repeated indicator production;
- compatible with complex survey designs;
- useful for official statistics;
- useful for academic and applied research;
- suitable for analyses involving multiple datasets or expansion factors.

Although the package was developed from practical experience in survey sampling and official statistics, it can be used by any researcher, analyst, public institution, national statistical office, or survey practitioner working with indicator-based survey data.

---

## Main Features

- Weighted totals
- Weighted proportions
- Standard errors
- Confidence intervals
- Coefficients of variation
- Design effects
- Unweighted sample sizes
- Grouped estimates
- Domain estimates
- Optional strata variables
- Optional cluster variables
- Unstratified designs
- Unclustered designs
- Unweighted simple indicator tables
- Flexible column selection
- Consolidated export of multiple analyses
- Selective export of chosen results
- Customizable `.xlsx` outputs

---

## Package Workflow

| Step | Function | Purpose |
|------|----------|---------|
| **1** | `svySE_cfg()` | Configure estimation settings, confidence level, target category, CV, DEFF, and other options. |
| **2A** | `svySE_calc()` | Calculate weighted estimates and sampling errors using the survey design. |
| **2B** | `svySE_simple()` | Calculate unweighted frequencies and percentages from the observed sample. |
| **3** | `svySE_xlsx()` | Export one or multiple results to `.xlsx` files. |

The two calculation functions have separate responsibilities:

| Function | Uses weights | Uses survey design | Main output |
|----------|:------------:|:------------------:|-------------|
| `svySE_calc()` | Yes | Yes | Weighted estimates and sampling errors |
| `svySE_simple()` | No | No | Unweighted frequencies and percentages |

This separation avoids redundant calculations and allows users to run only the workflow required for each analysis.

---

## Installation

### Stable version from CRAN

```r
install.packages("svySE")
```

### Development version from GitHub

```r
install.packages("remotes")

remotes::install_github("lburgoss/svySE")
```

The CRAN version is recommended for regular use. The GitHub version may contain features under development before they are submitted to CRAN.

Load the package with:

```r
library(svySE)
```

---

## Example Data

The following simulated dataset contains:

- a geographic grouping variable;
- a stratification variable;
- a cluster variable;
- a sampling weight;
- a division variable;
- two binary indicators.

```r
library(svySE)

set.seed(123)

df <- data.frame(
  dept = rep(c("A", "B", "C"), each = 50),
  strata = rep(c("S1", "S2", "S3"), each = 50),
  cluster = rep(1:30, each = 5),
  service = rep(c("S1", "S2"), length.out = 150),
  weight = runif(150, 10, 50),
  ind_1 = sample(c(0, 1), 150, replace = TRUE),
  ind_2 = sample(c(0, 1), 150, replace = TRUE)
)

head(df)
```

---

## Configure the Analysis

`svySE_cfg()` defines the common settings used during sampling error estimation.

```r
cfg <- svySE_cfg(
  estimator = "prop",
  variance = "taylor",
  lonely_psu = "adjust",
  conf_level = 0.95,
  target = 1,
  valid_values = c(0, 1),
  truncate_lower_ci = TRUE,
  pct_mult = 100,
  deff = TRUE,
  cv = TRUE,
  na_rm = TRUE
)

cfg
```

The most relevant options are:

| Argument | Description |
|----------|-------------|
| `estimator` | Estimator used in the analysis, such as `"prop"` or `"total"`. |
| `variance` | Variance estimation method. |
| `lonely_psu` | Treatment of strata containing a single PSU. |
| `conf_level` | Confidence level used for interval estimation. |
| `target` | Indicator category treated as the target value. |
| `valid_values` | Values considered valid for the indicator. |
| `truncate_lower_ci` | Whether lower confidence limits are truncated at zero. |
| `pct_mult` | Multiplier used to express percentages. |
| `deff` | Whether design effects are calculated. |
| `cv` | Whether coefficients of variation are calculated. |
| `na_rm` | Whether missing values are removed during estimation. |

---

## Calculate Sampling Errors

`svySE_calc()` estimates weighted indicators and sampling errors using the survey design.

```r
res_error <- svySE_calc(
  data = df,
  indicators = c("ind_1", "ind_2"),
  group_vars = "dept",
  group_labels = "Department",
  strata = "strata",
  cluster = "cluster",
  weight = "weight",
  division = NULL,
  div_weight = NULL,
  cfg = cfg,
  verbose = FALSE
)

res_error
```

The result is an object of class:

```r
class(res_error)
```

A specific sampling error table can be inspected with:

```r
res_error$results$ind_1$error$TOTAL
```

The output may contain:

| Column | Description |
|--------|-------------|
| `est_abs` | Weighted absolute estimate |
| `est_pct` | Weighted percentage estimate |
| `se_abs` | Standard error of the absolute estimate |
| `se_pct` | Standard error of the percentage |
| `ci_l_abs` | Lower confidence limit for the absolute estimate |
| `ci_l_pct` | Lower confidence limit for the percentage |
| `ci_u_abs` | Upper confidence limit for the absolute estimate |
| `ci_u_pct` | Upper confidence limit for the percentage |
| `cv` | Coefficient of variation |
| `deff` | Design effect |
| `n_unw` | Unweighted count of target cases |

---

## Supported Survey Designs

`svySE_calc()` supports several survey design structures.

| Design structure | `strata` | `cluster` |
|------------------|----------|-----------|
| Weight only | `NULL` | `NULL` |
| Stratified design | Variable name | `NULL` |
| Clustered design | `NULL` | Variable name |
| Stratified clustered design | Variable name | Variable name |

### Weight only

```r
res_weight <- svySE_calc(
  data = df,
  indicators = "ind_1",
  group_vars = "dept",
  group_labels = "Department",
  strata = NULL,
  cluster = NULL,
  weight = "weight",
  cfg = cfg,
  verbose = FALSE
)
```

### Stratified design

```r
res_strata <- svySE_calc(
  data = df,
  indicators = "ind_1",
  group_vars = "dept",
  group_labels = "Department",
  strata = "strata",
  cluster = NULL,
  weight = "weight",
  cfg = cfg,
  verbose = FALSE
)
```

### Clustered design

```r
res_cluster <- svySE_calc(
  data = df,
  indicators = "ind_1",
  group_vars = "dept",
  group_labels = "Department",
  strata = NULL,
  cluster = "cluster",
  weight = "weight",
  cfg = cfg,
  verbose = FALSE
)
```

### Stratified clustered design

```r
res_complex <- svySE_calc(
  data = df,
  indicators = "ind_1",
  group_vars = "dept",
  group_labels = "Department",
  strata = "strata",
  cluster = "cluster",
  weight = "weight",
  cfg = cfg,
  verbose = FALSE
)
```

---

## Domain Estimation

A division variable can be used to calculate separate results for its categories while retaining the design-based estimation workflow.

```r
res_domain <- svySE_calc(
  data = df,
  indicators = "ind_1",
  group_vars = "dept",
  group_labels = "Department",
  strata = "strata",
  cluster = "cluster",
  weight = "weight",
  division = "service",
  div_weight = NULL,
  cfg = cfg,
  verbose = FALSE
)
```

Available divisions can be inspected with:

```r
names(res_domain$results$ind_1$error)
```

When `div_weight` is supplied, that weight is used for the corresponding division estimates.

---

## Simple Indicator Tables

`svySE_simple()` calculates frequencies and percentages without using sampling weights, strata, or cluster variables.

```r
res_simple <- svySE_simple(
  data = df,
  indicators = c("ind_1", "ind_2"),
  group_vars = "dept",
  group_labels = "Department",
  division = NULL,
  target = 1,
  valid_values = c(0, 1),
  pct_mult = 100,
  verbose = FALSE
)

res_simple
```

A specific table can be inspected with:

```r
res_simple$results$ind_1$simple$TOTAL
```

The output includes:

| Column | Description |
|--------|-------------|
| `freq_0` | Frequency of non-target cases |
| `pct_0` | Percentage of non-target cases |
| `freq_1` | Frequency of target cases |
| `pct_1` | Percentage of target cases |
| `freq_total` | Total number of valid observations |
| `pct_total` | Total percentage |

> The results produced by `svySE_simple()` describe only the observed sample. Because no sampling weights or survey design variables are used, these percentages should not be interpreted as population estimates.

---

## Simple Tables by Division

```r
res_simple_domain <- svySE_simple(
  data = df,
  indicators = "ind_1",
  group_vars = "dept",
  group_labels = "Department",
  division = "service",
  target = 1,
  valid_values = c(0, 1),
  verbose = FALSE
)
```

Available divisions can be reviewed with:

```r
names(res_simple_domain$results$ind_1$simple)
```

---

## Export Results to XLSX

`svySE_xlsx()` exports sampling error results, simple indicator tables, or both.

### Export one sampling error result

```r
file_err <- tempfile(fileext = ".xlsx")

svySE_xlsx(
  x = res_error,
  file_err = file_err,
  file_tab = NULL,
  cols_err = svySE_cols_err("full"),
  overwrite = TRUE
)

file.exists(file_err)
```

### Export one simple result

```r
file_tab <- tempfile(fileext = ".xlsx")

svySE_xlsx(
  x = res_simple,
  file_err = NULL,
  file_tab = file_tab,
  cols_tab = svySE_cols_tab("full"),
  overwrite = TRUE
)

file.exists(file_tab)
```

---

## Export Multiple Analyses

Multiple results generated from different datasets, indicators, survey weights, or function calls can be exported together.

```r
results <- list(
  Main_errors = res_error,
  Domain_errors = res_domain,
  Main_simple = res_simple,
  Domain_simple = res_simple_domain
)
```

Export all available results:

```r
file_err <- tempfile(fileext = ".xlsx")
file_tab <- tempfile(fileext = ".xlsx")

svySE_xlsx(
  x = results,
  file_err = file_err,
  file_tab = file_tab,
  cols_err = svySE_cols_err("full"),
  cols_tab = svySE_cols_tab("full"),
  overwrite = TRUE
)
```

`svySE_xlsx()` automatically identifies:

- objects generated by `svySE_calc()`;
- objects generated by `svySE_simple()`.

Sampling error tables are written to `file_err`, while simple indicator tables are written to `file_tab`.

---

## Export Selected Results

The `select` argument can be used to export only chosen elements from a named list.

```r
svySE_xlsx(
  x = results,
  select = c("Main_errors", "Domain_errors"),
  file_err = tempfile(fileext = ".xlsx"),
  file_tab = NULL,
  overwrite = TRUE
)
```

Export only simple tables:

```r
svySE_xlsx(
  x = results,
  select = c("Main_simple", "Domain_simple"),
  file_err = NULL,
  file_tab = tempfile(fileext = ".xlsx"),
  overwrite = TRUE
)
```

---

## Customize Exported Columns

### Sampling error columns

```r
svySE_cols_err("full")
```

A custom selection can be defined with:

```r
error_columns <- svySE_cols_err(
  type = "custom",
  cols = c(
    "est_pct",
    "se_pct",
    "ci_l_pct",
    "ci_u_pct",
    "cv",
    "deff",
    "n_unw"
  )
)
```

Use the selected columns during export:

```r
svySE_xlsx(
  x = res_error,
  file_err = tempfile(fileext = ".xlsx"),
  file_tab = NULL,
  cols_err = error_columns
)
```

### Simple table columns

```r
svySE_cols_tab("full")
```

A custom selection can be defined with:

```r
simple_columns <- svySE_cols_tab(
  type = "custom",
  cols = c(
    "freq_1",
    "pct_1",
    "freq_total"
  )
)
```

Use the selected columns during export:

```r
svySE_xlsx(
  x = res_simple,
  file_err = NULL,
  file_tab = tempfile(fileext = ".xlsx"),
  cols_tab = simple_columns
)
```

---

## Main Functions

| Function | Description |
|----------|-------------|
| `svySE_cfg()` | Configure sampling error estimation settings |
| `svySE_calc()` | Calculate weighted estimates and sampling errors |
| `svySE_simple()` | Calculate unweighted frequencies and percentages |
| `svySE_xlsx()` | Export one or multiple results to `.xlsx` files |
| `svySE_cols_err()` | Select sampling error columns |
| `svySE_cols_tab()` | Select simple table columns |

---

## Output Types

| Output | Generated by | Weighted | Uses survey design | Typical use |
|--------|--------------|:--------:|:------------------:|-------------|
| Sampling error tables | `svySE_calc()` | Yes | Yes | Official statistics, complex surveys, technical reports |
| Simple indicator tables | `svySE_simple()` | No | No | Descriptive analysis and sample-level reporting |

---

## Technical Ecosystem

`svySE` integrates functionality from established R packages.

| Package | Role |
|---------|------|
| `survey` | Design-based estimation, standard errors, confidence intervals, CV, and DEFF |
| `openxlsx` | Creation and formatting of `.xlsx` workbooks |
| `stats` | Statistical formulas, coefficients, and confidence intervals |
| `svySE` | High-level workflow for survey indicators, errors, tables, and export |

`svySE` does not replace `survey`. It provides a structured interface for repeated indicator production and export workflows built on top of its design-based estimation capabilities.

---

## Documentation

The package includes:

- a reference manual;
- function documentation;
- package vignettes;
- reproducible examples;
- unit tests.

Open the package help:

```r
help(package = "svySE")
```

Browse available vignettes:

```r
browseVignettes("svySE")
```

Open documentation for the principal functions:

```r
?svySE_cfg
?svySE_calc
?svySE_simple
?svySE_xlsx
```

---

## Development Status

Version **0.2.0** introduces:

- optional cluster variables;
- support for unstratified and unclustered designs;
- the new `svySE_simple()` workflow;
- separation between weighted estimation and descriptive simple tables;
- consolidated export of multiple analyses;
- selective export through the `select` argument;
- customizable `.xlsx` workbooks.

Future releases will focus on additional estimators, expanded quality indicators, more export options, and broader support for complex survey workflows.

---

## Author

**Luis Burgos**

Statistician • RENACYT Researcher (Peru)

Sampling Specialist

National Institute of Statistics and Informatics (INEI)

ENCAL — Public Expenditure Quality Monitoring Survey

The package was developed independently based on professional experience in complex survey sampling, official statistics, and statistical programming.

Email: lburgoss1996@gmail.com

Suggestions, bug reports, and feature requests are welcome through the GitHub issue tracker.

---

## License

MIT License

# ==============================================================================
# svySE: seleccion de columnas para exportacion
# svySE: column selection for export
# Archivo / File: R/columnas.R
# ==============================================================================

#' Available sampling error columns
#'
#' Devuelve todas las columnas metricas disponibles para la tabla de errores
#' muestrales.
#' Returns all available metric columns for the sampling error table.
#'
#' @return Vector de nombres de columnas / Character vector of column names.
#'
#' @export
svySE_cols_err_all <- function() {
  
  c(
    "est_abs",
    "est_pct",
    "se_abs",
    "se_pct",
    "ci_l_abs",
    "ci_l_pct",
    "ci_u_abs",
    "ci_u_pct",
    "cv",
    "deff",
    "n_unw"
  )
}


#' Available simple table columns
#'
#' Devuelve todas las columnas disponibles para la tabla simple.
#' Returns all available columns for the simple table.
#'
#' @return Vector de nombres de columnas / Character vector of column names.
#'
#' @export
svySE_cols_tab_all <- function() {
  
  c(
    "freq_0",
    "pct_0",
    "freq_1",
    "pct_1",
    "freq_total",
    "pct_total"
  )
}


#' Select sampling error columns
#'
#' Selecciona columnas para exportar la tabla de errores muestrales.
#' Selects columns to export the sampling error table.
#'
#' @param type Tipo de salida / Output type. Opciones:
#'   `"full"`, `"pct"`, `"abs"`, `"basic"`, `"quality"` o `"custom"`.
#' @param cols Vector de columnas cuando `type = "custom"`.
#'
#' @return Vector de columnas seleccionadas / Selected column vector.
#'
#' @examples
#' svySE_cols_err("full")
#' svySE_cols_err("pct")
#' svySE_cols_err("custom", cols = c("est_pct", "cv"))
#'
#' @export
svySE_cols_err <- function(
    type = "full",
    cols = NULL
) {
  
  # ---------------------------------------------------------------------------
  # Validar tipo de salida
  # Validate output type
  # ---------------------------------------------------------------------------
  
  type <- match.arg(
    type,
    choices = c("full", "pct", "abs", "basic", "quality", "custom")
  )
  
  all_cols <- svySE_cols_err_all()
  
  # ---------------------------------------------------------------------------
  # Seleccionar columnas segun perfil
  # Select columns according to profile
  # ---------------------------------------------------------------------------
  
  out <- switch(
    type,
    
    full = all_cols,
    
    pct = c(
      "est_pct",
      "se_pct",
      "ci_l_pct",
      "ci_u_pct",
      "cv"
    ),
    
    abs = c(
      "est_abs",
      "se_abs",
      "ci_l_abs",
      "ci_u_abs",
      "cv",
      "deff",
      "n_unw"
    ),
    
    basic = c(
      "est_abs",
      "est_pct",
      "se_abs",
      "se_pct",
      "cv"
    ),
    
    quality = c(
      "est_pct",
      "se_pct",
      "cv",
      "deff",
      "n_unw"
    ),
    
    custom = cols
  )
  
  # ---------------------------------------------------------------------------
  # Validar columnas seleccionadas
  # Validate selected columns
  # ---------------------------------------------------------------------------
  
  svySE_chk_cols(
    cols = out,
    all_cols = all_cols,
    arg = "cols"
  )
  
  out
}


#' Select simple table columns
#'
#' Selecciona columnas para exportar la tabla simple.
#' Selects columns to export the simple table.
#'
#' @param type Tipo de salida / Output type. Opciones:
#'   `"full"`, `"target"`, `"freq"`, `"pct"` o `"custom"`.
#' @param cols Vector de columnas cuando `type = "custom"`.
#'
#' @return Vector de columnas seleccionadas / Selected column vector.
#'
#' @examples
#' svySE_cols_tab("full")
#' svySE_cols_tab("target")
#' svySE_cols_tab("custom", cols = c("freq_1", "pct_1"))
#'
#' @export
svySE_cols_tab <- function(
    type = "full",
    cols = NULL
) {
  
  # ---------------------------------------------------------------------------
  # Validar tipo de salida
  # Validate output type
  # ---------------------------------------------------------------------------
  
  type <- match.arg(
    type,
    choices = c("full", "target", "freq", "pct", "custom")
  )
  
  all_cols <- svySE_cols_tab_all()
  
  # ---------------------------------------------------------------------------
  # Seleccionar columnas segun perfil
  # Select columns according to profile
  # ---------------------------------------------------------------------------
  
  out <- switch(
    type,
    
    full = all_cols,
    
    target = c(
      "freq_1",
      "pct_1",
      "freq_total",
      "pct_total"
    ),
    
    freq = c(
      "freq_0",
      "freq_1",
      "freq_total"
    ),
    
    pct = c(
      "pct_0",
      "pct_1",
      "pct_total"
    ),
    
    custom = cols
  )
  
  # ---------------------------------------------------------------------------
  # Validar columnas seleccionadas
  # Validate selected columns
  # ---------------------------------------------------------------------------
  
  svySE_chk_cols(
    cols = out,
    all_cols = all_cols,
    arg = "cols"
  )
  
  out
}


# ==============================================================================
# Funciones internas
# Internal functions
# ==============================================================================

#' Check selected columns
#'
#' Valida que las columnas seleccionadas existan dentro del conjunto permitido.
#' Checks whether selected columns exist in the allowed column set.
#'
#' @param cols Columnas seleccionadas / Selected columns.
#' @param all_cols Columnas disponibles / Available columns.
#' @param arg Nombre del argumento / Argument name.
#'
#' @return Invisiblemente `TRUE`.
#'
#' @keywords internal
svySE_chk_cols <- function(
    cols,
    all_cols,
    arg = "cols"
) {
  
  if (is.null(cols) || length(cols) == 0) {
    stop(
      paste0(
        "`", arg, "` debe contener al menos una columna / ",
        "must contain at least one column."
      ),
      call. = FALSE
    )
  }
  
  if (!is.character(cols)) {
    stop(
      paste0(
        "`", arg, "` debe ser un vector de texto / ",
        "must be a character vector."
      ),
      call. = FALSE
    )
  }
  
  bad <- setdiff(cols, all_cols)
  
  if (length(bad) > 0) {
    stop(
      paste0(
        "Columnas no validas / Invalid columns: ",
        paste(bad, collapse = ", "),
        ". Columnas disponibles / Available columns: ",
        paste(all_cols, collapse = ", "),
        "."
      ),
      call. = FALSE
    )
  }
  
  invisible(TRUE)
}

# ==============================================================================
# Tests: seleccion de columnas
# Tests: column selection
# Archivo / File: tests/testthat/test-columnas.R
# ==============================================================================


test_that("svySE_cols_err_all returns every sampling error column", {

  expected <- c(
    "est_abs",
    "est_pct",
    "se_abs",
    "se_pct",
    "ci_l_abs",
    "ci_l_pct",
    "ci_u_abs",
    "ci_u_pct",
    "cv",
    "deff",
    "n_unw"
  )

  expect_identical(svySE_cols_err_all(), expected)
})


test_that("svySE_cols_tab_all returns every simple table column", {

  expected <- c(
    "freq_0",
    "pct_0",
    "freq_1",
    "pct_1",
    "freq_total",
    "pct_total"
  )

  expect_identical(svySE_cols_tab_all(), expected)
})


test_that("svySE_cols_err returns the expected predefined profiles", {

  expect_identical(
    svySE_cols_err("full"),
    svySE_cols_err_all()
  )

  expect_identical(
    svySE_cols_err("pct"),
    c("est_pct", "se_pct", "ci_l_pct", "ci_u_pct", "cv")
  )

  expect_identical(
    svySE_cols_err("abs"),
    c("est_abs", "se_abs", "ci_l_abs", "ci_u_abs", "cv", "deff", "n_unw")
  )

  expect_identical(
    svySE_cols_err("basic"),
    c("est_abs", "est_pct", "se_abs", "se_pct", "cv")
  )

  expect_identical(
    svySE_cols_err("quality"),
    c("est_pct", "se_pct", "cv", "deff", "n_unw")
  )
})


test_that("svySE_cols_tab returns the expected predefined profiles", {

  expect_identical(
    svySE_cols_tab("full"),
    svySE_cols_tab_all()
  )

  expect_identical(
    svySE_cols_tab("target"),
    c("freq_1", "pct_1", "freq_total", "pct_total")
  )

  expect_identical(
    svySE_cols_tab("freq"),
    c("freq_0", "freq_1", "freq_total")
  )

  expect_identical(
    svySE_cols_tab("pct"),
    c("pct_0", "pct_1", "pct_total")
  )
})


test_that("custom column profiles preserve valid user selections", {

  err_cols <- c("est_pct", "cv", "n_unw")
  tab_cols <- c("freq_1", "pct_1")

  expect_identical(
    svySE_cols_err("custom", cols = err_cols),
    err_cols
  )

  expect_identical(
    svySE_cols_tab("custom", cols = tab_cols),
    tab_cols
  )
})


test_that("column selectors reject invalid types", {

  expect_error(
    svySE_cols_err("invalid"),
    "arg"
  )

  expect_error(
    svySE_cols_tab("invalid"),
    "arg"
  )
})


test_that("custom column selectors require non-empty character vectors", {

  expect_error(
    svySE_cols_err("custom", cols = NULL),
    "al menos una columna"
  )

  expect_error(
    svySE_cols_tab("custom", cols = character(0)),
    "al menos una columna"
  )

  expect_error(
    svySE_cols_err("custom", cols = 1),
    "vector de texto"
  )

  expect_error(
    svySE_cols_tab("custom", cols = TRUE),
    "vector de texto"
  )
})


test_that("custom column selectors reject unknown column names", {

  expect_error(
    svySE_cols_err("custom", cols = c("est_pct", "unknown")),
    "Columnas no validas"
  )

  expect_error(
    svySE_cols_tab("custom", cols = c("freq_1", "unknown")),
    "Columnas no validas"
  )
})

# ==============================================================================
# Tests: tablas simples
# Tests: simple indicator tables
# Archivo / File: tests/testthat/test-simple.R
# ==============================================================================


make_simple_test_data <- function() {

  data.frame(
    dept = rep(c("A", "B", "C"), each = 8),
    area = rep(c("Urban", "Rural"), length.out = 24),
    ind_1 = c(
      1, 1, 0, 0, 1, 0, 1, 0,
      1, 0, 0, 0, 1, 0, 0, 0,
      1, 1, 1, 0, 1, 1, 0, 0
    ),
    ind_2 = rep(c(0, 1, 1), length.out = 24),
    stringsAsFactors = FALSE
  )
}


test_that("svySE_simple returns a valid simple result", {

  result <- svySE_simple(
    data = make_simple_test_data(),
    indicators = "ind_1",
    group_vars = "dept",
    group_labels = "Department",
    verbose = FALSE
  )

  expect_s3_class(result, "svySE_simple_result")
  expect_identical(names(result$results), "ind_1")
  expect_identical(names(result$results$ind_1), "simple")
  expect_identical(names(result$results$ind_1$simple), "TOTAL")
  expect_false("error" %in% names(result$results$ind_1))

  expect_identical(result$meta$indicators, "ind_1")
  expect_identical(result$meta$group_vars, "dept")
  expect_identical(result$meta$group_labels, "Department")
  expect_null(result$meta$strata)
  expect_null(result$meta$cluster)
  expect_null(result$meta$weight)
})


test_that("svySE_simple calculates exact frequencies and percentages", {

  df <- data.frame(
    dept = rep(c("A", "B"), each = 4),
    ind_1 = c(1, 1, 0, 0, 1, 0, 0, 0),
    stringsAsFactors = FALSE
  )

  result <- svySE_simple(
    data = df,
    indicators = "ind_1",
    group_vars = "dept",
    group_labels = "Department",
    target = 1,
    valid_values = c(0, 1),
    pct_mult = 100,
    verbose = FALSE
  )

  tab <- result$results$ind_1$simple$TOTAL

  national <- tab[tab$dept == "NACIONAL", , drop = FALSE]
  group_a <- tab[tab$dept == "A", , drop = FALSE]
  group_b <- tab[tab$dept == "B", , drop = FALSE]

  expect_equal(national$freq_0, 5)
  expect_equal(national$pct_0, 62.5)
  expect_equal(national$freq_1, 3)
  expect_equal(national$pct_1, 37.5)
  expect_equal(national$freq_total, 8)
  expect_equal(national$pct_total, 100)

  expect_equal(group_a$freq_1, 2)
  expect_equal(group_a$pct_1, 50)
  expect_equal(group_b$freq_1, 1)
  expect_equal(group_b$pct_1, 25)
})


test_that("svySE_simple works with multiple indicators", {

  result <- svySE_simple(
    data = make_simple_test_data(),
    indicators = c("ind_1", "ind_2"),
    group_vars = "dept",
    group_labels = "Department",
    verbose = FALSE
  )

  expect_setequal(names(result$results), c("ind_1", "ind_2"))

  for (indicator in c("ind_1", "ind_2")) {
    expect_identical(names(result$results[[indicator]]), "simple")
    expect_identical(names(result$results[[indicator]]$simple), "TOTAL")
    expect_false("error" %in% names(result$results[[indicator]]))
  }
})


test_that("svySE_simple supports division variables", {

  result <- svySE_simple(
    data = make_simple_test_data(),
    indicators = "ind_1",
    group_vars = "dept",
    group_labels = "Department",
    division = "area",
    verbose = FALSE
  )

  expect_identical(
    names(result$results$ind_1$simple),
    c("TOTAL", "Rural", "Urban")
  )

  for (division_name in names(result$results$ind_1$simple)) {
    tab <- result$results$ind_1$simple[[division_name]]

    expect_equal(tab$dept[1], "NACIONAL")
    expect_setequal(tab$dept[-1], c("A", "B", "C"))
  }

  expect_identical(result$meta$division, "area")
})


test_that("svySE_simple respects target and percentage multiplier", {

  df <- data.frame(
    dept = rep("A", 4),
    ind_1 = c(1, 1, 0, 0),
    stringsAsFactors = FALSE
  )

  result <- svySE_simple(
    data = df,
    indicators = "ind_1",
    group_vars = "dept",
    target = 0,
    valid_values = c(0, 1),
    pct_mult = 1,
    verbose = FALSE
  )

  tab <- result$results$ind_1$simple$TOTAL
  national <- tab[tab$dept == "NACIONAL", , drop = FALSE]

  expect_equal(national$freq_1, 2)
  expect_equal(national$pct_1, 0.5)
  expect_equal(national$freq_0, 2)
  expect_equal(national$pct_0, 0.5)
  expect_equal(national$pct_total, 1)
})


test_that("svySE_simple excludes missing and invalid indicator values", {

  df <- data.frame(
    dept = c("A", "A", "B", "B", "B"),
    ind_1 = c(0, 1, 2, NA, 1),
    stringsAsFactors = FALSE
  )

  expect_warning(
    result <- svySE_simple(
      data = df,
      indicators = "ind_1",
      group_vars = "dept",
      valid_values = c(0, 1),
      strict = FALSE,
      verbose = FALSE
    ),
    "fuera de `valid_values`"
  )

  tab <- result$results$ind_1$simple$TOTAL
  national <- tab[tab$dept == "NACIONAL", , drop = FALSE]

  expect_equal(national$freq_total, 3)
  expect_equal(national$freq_1, 2)
  expect_equal(national$freq_0, 1)
})


test_that("svySE_simple stops on invalid values when strict is TRUE", {

  df <- data.frame(
    dept = c("A", "A", "B"),
    ind_1 = c(0, 2, 1),
    stringsAsFactors = FALSE
  )

  expect_error(
    svySE_simple(
      data = df,
      indicators = "ind_1",
      group_vars = "dept",
      valid_values = c(0, 1),
      strict = TRUE,
      verbose = FALSE
    ),
    class = "svySE_error"
  )
})


test_that("svySE_simple returns NA rows for groups without valid records", {

  df <- data.frame(
    dept = c("A", "A", "B", "B"),
    ind_1 = c(0, 1, NA, NA),
    stringsAsFactors = FALSE
  )

  result <- svySE_simple(
    data = df,
    indicators = "ind_1",
    group_vars = "dept",
    verbose = FALSE
  )

  tab <- result$results$ind_1$simple$TOTAL
  group_b <- tab[tab$dept == "B", , drop = FALSE]

  expect_equal(nrow(group_b), 1)
  expect_true(all(is.na(group_b[svySE_cols_tab_all()])))
})


test_that("svySE_simple does not modify the original data", {

  df <- make_simple_test_data()
  original <- df

  invisible(
    svySE_simple(
      data = df,
      indicators = "ind_1",
      group_vars = "dept",
      verbose = FALSE
    )
  )

  expect_identical(df, original)
  expect_false(".__svySE_group_id__" %in% names(df))
  expect_false(".__svySE_cat__" %in% names(df))
})


test_that("svySE_simple validates its main arguments", {

  df <- make_simple_test_data()

  expect_error(
    svySE_simple(
      data = df,
      indicators = "missing",
      group_vars = "dept",
      verbose = FALSE
    ),
    class = "svySE_error"
  )

  expect_error(
    svySE_simple(
      data = df,
      indicators = "ind_1",
      group_vars = "missing",
      verbose = FALSE
    ),
    class = "svySE_error"
  )

  expect_error(
    svySE_simple(
      data = df,
      indicators = "ind_1",
      group_vars = "dept",
      group_labels = c("Department", "Extra"),
      verbose = FALSE
    ),
    class = "svySE_error"
  )

  expect_error(
    svySE_simple(
      data = df,
      indicators = "ind_1",
      group_vars = "dept",
      target = 2,
      valid_values = c(0, 1),
      verbose = FALSE
    ),
    class = "svySE_error"
  )

  expect_error(
    svySE_simple(
      data = df,
      indicators = "ind_1",
      group_vars = "dept",
      pct_mult = 0,
      verbose = FALSE
    ),
    class = "svySE_error"
  )
})


test_that("print.svySE_simple_result returns the object invisibly", {

  result <- svySE_simple(
    data = make_simple_test_data(),
    indicators = "ind_1",
    group_vars = "dept",
    verbose = FALSE
  )

  output <- capture.output(
    returned <- print(result)
  )

  expect_identical(returned, result)
  expect_true(any(grepl("svySE simple result", output, fixed = TRUE)))
  expect_true(any(grepl("Weighted", output, fixed = TRUE)))
  expect_true(any(grepl("observed sample", output, fixed = TRUE)))
})


test_that("verbose mode reports simple-table progress", {

  expect_message(
    svySE_simple(
      data = make_simple_test_data(),
      indicators = "ind_1",
      group_vars = "dept",
      verbose = TRUE
    ),
    "Procesando tabla simple"
  )
})

# ==============================================================================
# svySE: tablas simples sin diseno muestral
# svySE: unweighted simple indicator tables
# Archivo / File: R/simple.R
# ==============================================================================


#' Calculate unweighted simple indicator tables
#'
#' Calcula frecuencias y porcentajes simples para uno o mas indicadores sin
#' utilizar factores de expansion ni variables del diseno muestral.
#'
#' Calculates unweighted frequencies and percentages for one or more indicators
#' without using sampling weights or survey design variables.
#'
#' @param data Base de datos previamente cargada en R.
#'   Data frame previously loaded in R.
#'
#' @param indicators Vector de texto con los nombres de los indicadores.
#'   Character vector containing the indicator names.
#'
#' @param group_vars Variables utilizadas para agrupar los resultados.
#'   Variables used to group the results.
#'
#' @param group_labels Etiquetas de las variables de agrupacion.
#'   Labels for the grouping variables.
#'
#' @param division Variable opcional para generar resultados separados por
#'   categoria. Si es `NULL`, solo se calcula el total.
#'   Optional variable used to generate results by category. If `NULL`, only
#'   the total is calculated.
#'
#' @param target Valor que identifica la categoria de interes.
#'   Value identifying the target category.
#'
#' @param valid_values Valores permitidos en los indicadores.
#'   Values allowed in the indicators.
#'
#' @param pct_mult Multiplicador utilizado para expresar los porcentajes.
#'   Multiplier used to express percentages.
#'
#' @param verbose Si es `TRUE`, muestra el avance del calculo.
#'   If `TRUE`, displays calculation progress.
#'
#' @param strict Si es `TRUE`, detiene el calculo cuando encuentra valores fuera
#'   de `valid_values`. Si es `FALSE`, los excluye con una advertencia.
#'   If `TRUE`, stops when values outside `valid_values` are found. If `FALSE`,
#'   they are excluded with a warning.
#'
#' @details
#' Los resultados generados por esta funcion describen solamente los registros
#' observados en la muestra. Como no se utilizan factores de expansion ni
#' componentes del diseno muestral, los porcentajes no deben interpretarse como
#' estimaciones representativas de la poblacion.
#'
#' The results generated by this function describe only the records observed in
#' the sample. Because sampling weights and survey design components are not
#' used, percentages should not be interpreted as population estimates.
#'
#' @return Objeto de clase `"svySE_simple_result"`.
#'   Object of class `"svySE_simple_result"`.
#'
#' @export
svySE_simple <- function(
    data,
    indicators,
    group_vars,
    group_labels = group_vars,
    division = NULL,
    target = 1,
    valid_values = c(0, 1),
    pct_mult = 100,
    verbose = TRUE,
    strict = FALSE
) {
  
  # ---------------------------------------------------------------------------
  # Validaciones generales
  # General validations
  # ---------------------------------------------------------------------------
  
  svySE_chk_df(data)
  svySE_chk_chr(indicators, "indicators")
  svySE_chk_chr(group_vars, "group_vars")
  svySE_chk_chr(group_labels, "group_labels")
  svySE_chk_bool(verbose, "verbose")
  svySE_chk_bool(strict, "strict")
  
  if (length(target) != 1 || is.na(target)) {
    svySE_abort(
      title = "Valor objetivo invalido / Invalid target value.",
      details = "`target` debe contener un solo valor no perdido.",
      vars = list(target = target)
    )
  }
  
  if (length(valid_values) < 2 || any(is.na(valid_values))) {
    svySE_abort(
      title = "Valores validos incorrectos / Invalid valid values.",
      details = paste(
        "`valid_values` debe contener al menos dos valores no perdidos.",
        "`valid_values` must contain at least two non-missing values."
      ),
      vars = list(valid_values = valid_values)
    )
  }
  
  if (!(target %in% valid_values)) {
    svySE_abort(
      title = "El valor objetivo no es valido / Target is not valid.",
      details = "`target` debe estar incluido dentro de `valid_values`.",
      vars = list(
        target = target,
        valid_values = valid_values
      )
    )
  }
  
  if (
    !is.numeric(pct_mult) ||
    length(pct_mult) != 1 ||
    is.na(pct_mult) ||
    !is.finite(pct_mult) ||
    pct_mult <= 0
  ) {
    svySE_abort(
      title = "Multiplicador de porcentaje invalido / Invalid percentage multiplier.",
      details = "`pct_mult` debe ser un numero positivo y finito.",
      vars = list(pct_mult = pct_mult)
    )
  }
  
  if (!is.null(division)) {
    svySE_chk_chr(division, "division")
    
    if (length(division) != 1) {
      svySE_abort(
        title = "Variable de division invalida / Invalid division variable.",
        details = paste(
          "`division` debe contener una sola variable.",
          "`division` must contain one variable."
        ),
        vars = list(division = division)
      )
    }
  }
  
  # ---------------------------------------------------------------------------
  # Validar existencia de variables
  # Validate variable existence
  # ---------------------------------------------------------------------------
  
  svySE_chk_vars(data, indicators, "indicators")
  svySE_chk_vars(data, group_vars, "group_vars")
  
  if (!is.null(division)) {
    svySE_chk_vars(data, division, "division")
  }
  
  if (length(group_labels) != length(group_vars)) {
    svySE_abort(
      title = "Etiquetas de grupo incompatibles / Incompatible group labels.",
      details = paste(
        "`group_labels` debe tener la misma longitud que `group_vars`.",
        "`group_labels` must have the same length as `group_vars`."
      ),
      vars = list(
        group_vars = group_vars,
        group_labels = group_labels
      )
    )
  }
  
  svySE_chk_required_no_na(
    data = data,
    vars = group_vars,
    context = "variables de agrupacion / grouping variables"
  )
  
  if (!is.null(division)) {
    svySE_warn_na_division(data, division)
  }
  
  # ---------------------------------------------------------------------------
  # Copia interna
  # Internal copy
  # ---------------------------------------------------------------------------
  
  data <- as.data.frame(data)
  
  data$.__svySE_group_id__ <- do.call(
    paste,
    c(data[group_vars], sep = " | ")
  )
  
  groups_master <- sort(unique(data$.__svySE_group_id__))
  
  if (length(groups_master) == 0) {
    svySE_abort(
      title = "No se encontraron grupos validos / No valid groups found.",
      vars = list(group_vars = group_vars)
    )
  }
  
  # ---------------------------------------------------------------------------
  # Calculo por indicador
  # Calculation by indicator
  # ---------------------------------------------------------------------------
  
  out <- list()
  
  for (ind in indicators) {
    
    if (isTRUE(verbose)) {
      message("Procesando tabla simple / Processing simple table: ", ind)
    }
    
    x <- data[[ind]]
    
    invalid_values <- setdiff(
      unique(x[!is.na(x)]),
      valid_values
    )
    
    if (length(invalid_values) > 0) {
      
      msg <- paste0(
        "El indicador `", ind,
        "` contiene valores fuera de `valid_values`: ",
        paste(invalid_values, collapse = ", "),
        ". Estos registros seran excluidos."
      )
      
      if (isTRUE(strict)) {
        svySE_abort(
          title = "Indicador con valores no permitidos / Indicator with invalid values.",
          details = msg,
          vars = list(
            indicator = ind,
            valid_values = valid_values,
            invalid_values = invalid_values,
            observed_values = svySE_observed_values(x)
          )
        )
      } else {
        warning(msg, call. = FALSE)
      }
    }
    
    data_ind <- data[
      !is.na(data[[ind]]) &
        data[[ind]] %in% valid_values,
      ,
      drop = FALSE
    ]
    
    if (nrow(data_ind) == 0) {
      svySE_abort(
        title = "Indicador sin registros validos / Indicator without valid records.",
        details = paste0(
          "El indicador `", ind,
          "` no tiene registros dentro de `valid_values`."
        ),
        vars = list(
          indicator = ind,
          valid_values = valid_values
        )
      )
    }
    
    data_ind$.__svySE_cat__ <- ifelse(
      data_ind[[ind]] == target,
      1,
      0
    )
    
    # -------------------------------------------------------------------------
    # Crear divisiones
    # Create divisions
    # -------------------------------------------------------------------------
    
    filters <- list(
      TOTAL = list(
        name = "TOTAL",
        data = data_ind
      )
    )
    
    if (!is.null(division)) {
      
      categories <- sort(unique(data_ind[[division]]))
      categories <- categories[!is.na(categories)]
      
      if (length(categories) == 0) {
        svySE_abort(
          title = paste(
            "Variable de division sin categorias validas /",
            "Division variable without valid categories."
          ),
          vars = list(division = division)
        )
      }
      
      for (category in categories) {
        
        filters[[as.character(category)]] <- list(
          name = as.character(category),
          data = data_ind[
            data_ind[[division]] == category,
            ,
            drop = FALSE
          ]
        )
      }
    }
    
    simple_list <- list()
    
    for (i in seq_along(filters)) {
      
      f <- filters[[i]]
      
      simple_list[[f$name]] <- svySE_simple_one(
        data = f$data,
        group_vars = group_vars,
        groups_master = groups_master,
        pct_mult = pct_mult
      )
    }
    
    out[[ind]] <- list(
      simple = simple_list
    )
  }
  
  # ---------------------------------------------------------------------------
  # Resultado
  # Result
  # ---------------------------------------------------------------------------
  
  result <- list(
    results = out,
    meta = list(
      indicators = indicators,
      group_vars = group_vars,
      group_labels = group_labels,
      strata = NULL,
      cluster = NULL,
      weight = NULL,
      division = division,
      div_weight = NULL,
      target = target,
      valid_values = valid_values,
      pct_mult = pct_mult,
      strict = strict
    )
  )
  
  class(result) <- c(
    "svySE_simple_result",
    "list"
  )
  
  result
}


# ==============================================================================
# Calculo de una tabla simple
# Calculate one simple table
# ==============================================================================

#' @keywords internal
svySE_simple_one <- function(
    data,
    group_vars,
    groups_master,
    pct_mult = 100
) {
  
  metric_names <- svySE_cols_tab_all()
  
  out <- data.frame()
  
  groups_in_data <- sort(unique(data$.__svySE_group_id__))
  
  for (g in groups_master) {
    
    if (!(g %in% groups_in_data)) {
      
      row_na <- svySE_na_row(
        group_id = g,
        group_vars = group_vars,
        metric_names = metric_names
      )
      
      out <- rbind(out, row_na)
      next
    }
    
    group_data <- data[
      data$.__svySE_group_id__ == g,
      ,
      drop = FALSE
    ]
    
    group_row <- group_data[
      1,
      group_vars,
      drop = FALSE
    ]
    
    n_total <- nrow(group_data)
    n_target <- sum(group_data$.__svySE_cat__ == 1, na.rm = TRUE)
    n_other <- sum(group_data$.__svySE_cat__ == 0, na.rm = TRUE)
    
    metrics <- data.frame(
      freq_0 = n_other,
      pct_0 = if (n_total > 0) {
        n_other / n_total * pct_mult
      } else {
        NA_real_
      },
      freq_1 = n_target,
      pct_1 = if (n_total > 0) {
        n_target / n_total * pct_mult
      } else {
        NA_real_
      },
      freq_total = n_total,
      pct_total = pct_mult
    )
    
    out <- rbind(
      out,
      cbind(group_row, metrics)
    )
  }
  
  # ---------------------------------------------------------------------------
  # Fila total
  # Total row
  # ---------------------------------------------------------------------------
  
  total_group <- as.data.frame(
    as.list(rep("NACIONAL", length(group_vars))),
    stringsAsFactors = FALSE
  )
  
  names(total_group) <- group_vars
  
  n_total <- nrow(data)
  n_target <- sum(data$.__svySE_cat__ == 1, na.rm = TRUE)
  n_other <- sum(data$.__svySE_cat__ == 0, na.rm = TRUE)
  
  total_metrics <- data.frame(
    freq_0 = n_other,
    pct_0 = if (n_total > 0) {
      n_other / n_total * pct_mult
    } else {
      NA_real_
    },
    freq_1 = n_target,
    pct_1 = if (n_total > 0) {
      n_target / n_total * pct_mult
    } else {
      NA_real_
    },
    freq_total = n_total,
    pct_total = pct_mult
  )
  
  out <- rbind(
    cbind(total_group, total_metrics),
    out
  )
  
  rownames(out) <- NULL
  
  out
}


# ==============================================================================
# Impresion del resultado simple
# Print simple result
# ==============================================================================

#' Print an svySE simple result
#'
#' Imprime un resumen del resultado generado por `svySE_simple()`.
#'
#' Prints a summary of the result generated by `svySE_simple()`.
#'
#' @param x Objeto de clase `"svySE_simple_result"`.
#' @param ... Argumentos adicionales.
#'
#' @return Invisiblemente `x`.
#'
#' @export
print.svySE_simple_result <- function(x, ...) {
  
  cat("svySE simple result\n")
  cat("--------------------------------------------------\n")
  cat(
    "Indicators :",
    paste(x$meta$indicators, collapse = ", "),
    "\n"
  )
  cat(
    "Groups     :",
    paste(x$meta$group_vars, collapse = ", "),
    "\n"
  )
  cat(
    "Division   :",
    if (is.null(x$meta$division)) {
      "NULL"
    } else {
      x$meta$division
    },
    "\n"
  )
  cat("Target     :", x$meta$target, "\n")
  cat(
    "Weighted   : No\n"
  )
  cat(
    "Warning    : Results describe the observed sample only.\n"
  )
  
  invisible(x)
}

# ==============================================================================
# svySE: configuracion general para errores muestrales
# svySE: general configuration for sampling errors
# Archivo / File: R/config.R
# ==============================================================================

#' Configure sampling error parameters
#'
#' Crea una configuracion general para el calculo de errores muestrales.
#' Creates a general configuration object for sampling error estimation.
#'
#' @param estimator Tipo de estimador. Opciones: `"prop"`, `"total"`,
#'   `"mean"` o `"ratio"`.
#' @param variance Metodo de estimacion de varianza. Actualmente solo `"taylor"`.
#' @param lonely_psu Tratamiento de estratos con una sola UPM. Opciones:
#'   `"adjust"`, `"average"`, `"certainty"`, `"remove"` o `"fail"`.
#' @param conf_level Nivel de confianza. Por defecto `0.95`.
#' @param target Valor objetivo del indicador. Por defecto `1`.
#' @param valid_values Valores validos del indicador. Por defecto `c(0, 1)`.
#' @param truncate_lower_ci Si es `TRUE`, trunca el limite inferior del IC en cero.
#' @param pct_mult Multiplicador para expresar proporciones como porcentajes.
#'   Por defecto `100`.
#' @param deff Si es `TRUE`, calcula efecto de diseno cuando sea posible.
#' @param cv Si es `TRUE`, calcula coeficiente de variacion.
#' @param na_rm Si es `TRUE`, remueve valores perdidos en operaciones auxiliares.
#'
#' @return Lista de clase `"svySE_cfg"`.
#'
#' @examples
#' cfg <- svySE_cfg(
#'   estimator = "prop",
#'   variance = "taylor",
#'   target = 1
#' )
#'
#' @export
svySE_cfg <- function(
    estimator = "prop",
    variance = "taylor",
    lonely_psu = "adjust",
    conf_level = 0.95,
    target = 1,
    valid_values = c(0, 1),
    truncate_lower_ci = TRUE,
    pct_mult = 100,
    deff = TRUE,
    cv = TRUE,
    na_rm = TRUE
) {
  
  # ---------------------------------------------------------------------------
  # Validar tipo de estimador
  # Validate estimator type
  # ---------------------------------------------------------------------------
  
  estimator <- match.arg(
    estimator,
    choices = c("prop", "total", "mean", "ratio")
  )
  
  # ---------------------------------------------------------------------------
  # Validar metodo de varianza
  # Validate variance method
  # ---------------------------------------------------------------------------
  
  variance <- match.arg(
    variance,
    choices = c("taylor")
  )
  
  # ---------------------------------------------------------------------------
  # Validar tratamiento de lonely PSU
  # Validate lonely PSU treatment
  # ---------------------------------------------------------------------------
  
  lonely_psu <- match.arg(
    lonely_psu,
    choices = c("adjust", "average", "certainty", "remove", "fail")
  )
  
  # ---------------------------------------------------------------------------
  # Validar nivel de confianza
  # Validate confidence level
  # ---------------------------------------------------------------------------
  
  if (!is.numeric(conf_level) || length(conf_level) != 1) {
    stop("`conf_level` debe ser un valor numerico unico / must be a single numeric value.",
         call. = FALSE)
  }
  
  if (is.na(conf_level) || conf_level <= 0 || conf_level >= 1) {
    stop("`conf_level` debe estar entre 0 y 1 / must be between 0 and 1.",
         call. = FALSE)
  }
  
  # ---------------------------------------------------------------------------
  # Validar valor objetivo
  # Validate target value
  # ---------------------------------------------------------------------------
  
  if (length(target) != 1) {
    stop("`target` debe contener un solo valor / must contain one single value.",
         call. = FALSE)
  }
  
  if (is.null(valid_values) || length(valid_values) == 0) {
    stop("`valid_values` debe contener al menos un valor / must contain at least one value.",
         call. = FALSE)
  }
  
  if (!(target %in% valid_values)) {
    stop("`target` debe estar incluido en `valid_values` / must be included in `valid_values`.",
         call. = FALSE)
  }
  
  # ---------------------------------------------------------------------------
  # Validar multiplicador porcentual
  # Validate percentage multiplier
  # ---------------------------------------------------------------------------
  
  if (!is.numeric(pct_mult) || length(pct_mult) != 1) {
    stop("`pct_mult` debe ser un valor numerico unico / must be a single numeric value.",
         call. = FALSE)
  }
  
  if (is.na(pct_mult) || pct_mult <= 0) {
    stop("`pct_mult` debe ser mayor que cero / must be greater than zero.",
         call. = FALSE)
  }
  
  # ---------------------------------------------------------------------------
  # Validar argumentos logicos
  # Validate logical arguments
  # ---------------------------------------------------------------------------
  
  svySE_chk_bool(truncate_lower_ci, "truncate_lower_ci")
  svySE_chk_bool(deff, "deff")
  svySE_chk_bool(cv, "cv")
  svySE_chk_bool(na_rm, "na_rm")
  
  # ---------------------------------------------------------------------------
  # Aplicar configuracion global del paquete survey
  # Apply global survey package option
  # ---------------------------------------------------------------------------
  
  options(survey.lonely.psu = lonely_psu)
  
  # ---------------------------------------------------------------------------
  # Crear objeto de configuracion
  # Create configuration object
  # ---------------------------------------------------------------------------
  
  out <- list(
    estimator = estimator,
    variance = variance,
    lonely_psu = lonely_psu,
    conf_level = conf_level,
    target = target,
    valid_values = valid_values,
    truncate_lower_ci = truncate_lower_ci,
    pct_mult = pct_mult,
    deff = deff,
    cv = cv,
    na_rm = na_rm
  )
  
  class(out) <- c("svySE_cfg", "list")
  
  out
}


# ==============================================================================
# Funciones internas
# Internal functions
# ==============================================================================

#' Check logical scalar
#'
#' Valida que un argumento sea logico de longitud uno.
#' Checks whether an argument is a single logical value.
#'
#' @param x Objeto a validar / Object to validate.
#' @param arg Nombre del argumento / Argument name.
#'
#' @return Invisiblemente `TRUE`.
#'
#' @keywords internal
svySE_chk_bool <- function(x, arg) {
  
  if (!is.logical(x) || length(x) != 1 || is.na(x)) {
    stop(
      paste0(
        "`", arg, "` debe ser TRUE o FALSE / must be TRUE or FALSE."
      ),
      call. = FALSE
    )
  }
  
  invisible(TRUE)
}


#' Check svySE configuration object
#'
#' Verifica si un objeto pertenece a la clase `svySE_cfg`.
#' Checks whether an object belongs to class `svySE_cfg`.
#'
#' @param x Objeto a evaluar / Object to check.
#'
#' @return `TRUE` o `FALSE`.
#'
#' @keywords internal
svySE_is_cfg <- function(x) {
  inherits(x, "svySE_cfg")
}


#' Print svySE configuration
#'
#' Imprime la configuracion de `svySE`.
#' Prints the `svySE` configuration.
#'
#' @param x Objeto de clase `"svySE_cfg"`.
#' @param ... Argumentos adicionales.
#'
#' @return Invisiblemente el objeto `x`.
#'
#' @export
print.svySE_cfg <- function(x, ...) {
  
  cat("svySE configuration\n")
  cat("--------------------------------------------------\n")
  cat("Estimator          :", x$estimator, "\n")
  cat("Variance           :", x$variance, "\n")
  cat("Lonely PSU         :", x$lonely_psu, "\n")
  cat("Confidence level   :", x$conf_level, "\n")
  cat("Target value       :", x$target, "\n")
  cat("Valid values       :", paste(x$valid_values, collapse = ", "), "\n")
  cat("Truncate lower CI  :", x$truncate_lower_ci, "\n")
  cat("Percentage mult.   :", x$pct_mult, "\n")
  cat("Include DEFF       :", x$deff, "\n")
  cat("Include CV         :", x$cv, "\n")
  cat("Remove NA          :", x$na_rm, "\n")
  
  invisible(x)
}

# ==============================================================================
# Tests: configuracion
# Tests: configuration
# Archivo / File: tests/testthat/test-config.R
# ==============================================================================


test_that("svySE_cfg creates a valid configuration object", {

  old_option <- getOption("survey.lonely.psu")
  on.exit(options(survey.lonely.psu = old_option), add = TRUE)

  cfg <- svySE_cfg(
    estimator = "prop",
    variance = "taylor",
    lonely_psu = "adjust",
    conf_level = 0.95,
    target = 1,
    valid_values = c(0, 1),
    truncate_lower_ci = TRUE,
    pct_mult = 100,
    deff = TRUE,
    cv = TRUE,
    na_rm = TRUE
  )

  expect_s3_class(cfg, "svySE_cfg")
  expect_true(svySE_is_cfg(cfg))

  expect_equal(cfg$estimator, "prop")
  expect_equal(cfg$variance, "taylor")
  expect_equal(cfg$lonely_psu, "adjust")
  expect_equal(cfg$conf_level, 0.95)
  expect_equal(cfg$target, 1)
  expect_equal(cfg$valid_values, c(0, 1))
  expect_true(cfg$truncate_lower_ci)
  expect_equal(cfg$pct_mult, 100)
  expect_true(cfg$deff)
  expect_true(cfg$cv)
  expect_true(cfg$na_rm)

  expect_equal(getOption("survey.lonely.psu"), "adjust")
})


test_that("svySE_cfg accepts the supported estimators and lonely PSU options", {

  old_option <- getOption("survey.lonely.psu")
  on.exit(options(survey.lonely.psu = old_option), add = TRUE)

  estimators <- c("prop", "total", "mean", "ratio")
  lonely_options <- c("adjust", "average", "certainty", "remove", "fail")

  for (estimator in estimators) {
    cfg <- svySE_cfg(estimator = estimator)
    expect_equal(cfg$estimator, estimator)
  }

  for (option in lonely_options) {
    cfg <- svySE_cfg(lonely_psu = option)
    expect_equal(cfg$lonely_psu, option)
    expect_equal(getOption("survey.lonely.psu"), option)
  }
})


test_that("svySE_cfg validates estimator, variance, and lonely PSU values", {

  expect_error(
    svySE_cfg(estimator = "invalid"),
    "arg"
  )

  expect_error(
    svySE_cfg(variance = "bootstrap"),
    "arg"
  )

  expect_error(
    svySE_cfg(lonely_psu = "invalid"),
    "arg"
  )
})


test_that("svySE_cfg validates confidence level and percentage multiplier", {

  expect_error(
    svySE_cfg(conf_level = "0.95"),
    "conf_level"
  )

  expect_error(
    svySE_cfg(conf_level = c(0.90, 0.95)),
    "conf_level"
  )

  expect_error(
    svySE_cfg(conf_level = 0),
    "conf_level"
  )

  expect_error(
    svySE_cfg(conf_level = 1),
    "conf_level"
  )

  expect_error(
    svySE_cfg(conf_level = NA_real_),
    "conf_level"
  )

  expect_error(
    svySE_cfg(pct_mult = "100"),
    "pct_mult"
  )

  expect_error(
    svySE_cfg(pct_mult = c(100, 1)),
    "pct_mult"
  )

  expect_error(
    svySE_cfg(pct_mult = 0),
    "pct_mult"
  )

  expect_error(
    svySE_cfg(pct_mult = NA_real_),
    "pct_mult"
  )
})


test_that("svySE_cfg validates target and valid values", {

  expect_error(
    svySE_cfg(target = c(0, 1)),
    "target"
  )

  expect_error(
    svySE_cfg(valid_values = NULL),
    "valid_values"
  )

  expect_error(
    svySE_cfg(valid_values = numeric(0)),
    "valid_values"
  )

  expect_error(
    svySE_cfg(target = 2, valid_values = c(0, 1)),
    "target"
  )
})


test_that("svySE_cfg validates logical arguments", {

  expect_error(
    svySE_cfg(truncate_lower_ci = 1),
    "truncate_lower_ci"
  )

  expect_error(
    svySE_cfg(deff = NA),
    "deff"
  )

  expect_error(
    svySE_cfg(cv = c(TRUE, FALSE)),
    "cv"
  )

  expect_error(
    svySE_cfg(na_rm = "TRUE"),
    "na_rm"
  )
})


test_that("print.svySE_cfg returns the configuration invisibly", {

  cfg <- svySE_cfg()

  output <- capture.output(
    returned <- print(cfg)
  )

  expect_identical(returned, cfg)
  expect_true(any(grepl("svySE configuration", output, fixed = TRUE)))
  expect_true(any(grepl("Estimator", output, fixed = TRUE)))
})

# ==============================================================================
# Tests: disenos muestrales de svySE_calc
# Tests: svySE_calc survey designs
# Archivo / File: tests/testthat/test-calculo-diseno.R
# ==============================================================================


make_design_data <- function() {

  set.seed(123)

  data.frame(
    dept = rep(c("A", "B", "C"), each = 40),
    strata = rep(c("S1", "S2", "S3"), each = 40),
    cluster = rep(seq_len(30), each = 4),
    weight = runif(120, 10, 50),
    ind_1 = rep(c(0, 1, 1, 0, 1), length.out = 120),
    stringsAsFactors = FALSE
  )
}


make_design_cfg <- function() {
  svySE_cfg(
    estimator = "prop",
    variance = "taylor",
    lonely_psu = "adjust",
    conf_level = 0.95,
    target = 1,
    valid_values = c(0, 1),
    truncate_lower_ci = TRUE,
    pct_mult = 100,
    deff = TRUE,
    cv = TRUE,
    na_rm = TRUE
  )
}


expected_error_columns <- c(
  "dept",
  "est_abs",
  "est_pct",
  "se_abs",
  "se_pct",
  "ci_l_abs",
  "ci_l_pct",
  "ci_u_abs",
  "ci_u_pct",
  "cv",
  "deff",
  "n_unw"
)


check_design_result <- function(result, expected_strata, expected_cluster) {

  expect_s3_class(result, "svySE_result")
  expect_true("ind_1" %in% names(result$results))
  expect_true("error" %in% names(result$results$ind_1))
  expect_false("simple" %in% names(result$results$ind_1))
  expect_true("TOTAL" %in% names(result$results$ind_1$error))

  tab <- result$results$ind_1$error$TOTAL

  expect_true(all(expected_error_columns %in% names(tab)))
  expect_equal(nrow(tab), 4)
  expect_equal(tab$dept[1], "NACIONAL")
  expect_setequal(tab$dept[-1], c("A", "B", "C"))
  expect_true(all(is.finite(tab$est_abs)))
  expect_true(all(is.finite(tab$est_pct)))
  expect_true(all(tab$est_pct >= 0 & tab$est_pct <= 100))
  expect_true(all(tab$n_unw >= 0))

  expect_identical(result$meta$strata, expected_strata)
  expect_identical(result$meta$cluster, expected_cluster)
  expect_identical(result$meta$weight, "weight")
}


test_that("svySE_calc supports a weight-only design", {

  skip_if_not_installed("survey")

  result <- svySE_calc(
    data = make_design_data(),
    indicators = "ind_1",
    group_vars = "dept",
    group_labels = "Department",
    strata = NULL,
    cluster = NULL,
    weight = "weight",
    cfg = make_design_cfg(),
    verbose = FALSE
  )

  check_design_result(
    result = result,
    expected_strata = NULL,
    expected_cluster = NULL
  )
})


test_that("svySE_calc supports a stratified design without clusters", {

  skip_if_not_installed("survey")

  result <- svySE_calc(
    data = make_design_data(),
    indicators = "ind_1",
    group_vars = "dept",
    group_labels = "Department",
    strata = "strata",
    cluster = NULL,
    weight = "weight",
    cfg = make_design_cfg(),
    verbose = FALSE
  )

  check_design_result(
    result = result,
    expected_strata = "strata",
    expected_cluster = NULL
  )
})


test_that("svySE_calc supports a clustered design without strata", {

  skip_if_not_installed("survey")

  result <- svySE_calc(
    data = make_design_data(),
    indicators = "ind_1",
    group_vars = "dept",
    group_labels = "Department",
    strata = NULL,
    cluster = "cluster",
    weight = "weight",
    cfg = make_design_cfg(),
    verbose = FALSE
  )

  check_design_result(
    result = result,
    expected_strata = NULL,
    expected_cluster = "cluster"
  )
})


test_that("svySE_calc supports a stratified and clustered design", {

  skip_if_not_installed("survey")

  result <- svySE_calc(
    data = make_design_data(),
    indicators = "ind_1",
    group_vars = "dept",
    group_labels = "Department",
    strata = "strata",
    cluster = "cluster",
    weight = "weight",
    cfg = make_design_cfg(),
    verbose = FALSE
  )

  check_design_result(
    result = result,
    expected_strata = "strata",
    expected_cluster = "cluster"
  )
})


test_that("svySE_calc preserves all grouping variables in the output", {

  skip_if_not_installed("survey")

  df <- make_design_data()
  df$area <- rep(c("Urban", "Rural"), length.out = nrow(df))

  result <- svySE_calc(
    data = df,
    indicators = "ind_1",
    group_vars = c("dept", "area"),
    group_labels = c("Department", "Area"),
    strata = "strata",
    cluster = "cluster",
    weight = "weight",
    cfg = make_design_cfg(),
    verbose = FALSE
  )

  tab <- result$results$ind_1$error$TOTAL

  expect_true(all(c("dept", "area") %in% names(tab)))
  expect_equal(tab$dept[1], "NACIONAL")
  expect_equal(tab$area[1], "NACIONAL")
  expect_equal(result$meta$group_vars, c("dept", "area"))
  expect_equal(result$meta$group_labels, c("Department", "Area"))
})


test_that("svySE_calc stores the configuration and result metadata", {

  skip_if_not_installed("survey")

  cfg <- make_design_cfg()

  result <- svySE_calc(
    data = make_design_data(),
    indicators = "ind_1",
    group_vars = "dept",
    group_labels = "Department",
    strata = "strata",
    cluster = "cluster",
    weight = "weight",
    cfg = cfg,
    strict = FALSE,
    verbose = FALSE
  )

  expect_identical(result$meta$indicators, "ind_1")
  expect_identical(result$meta$group_vars, "dept")
  expect_identical(result$meta$group_labels, "Department")
  expect_identical(result$meta$strata, "strata")
  expect_identical(result$meta$cluster, "cluster")
  expect_identical(result$meta$weight, "weight")
  expect_null(result$meta$division)
  expect_null(result$meta$div_weight)
  expect_identical(result$meta$cfg, cfg)
  expect_false(result$meta$strict)
})


test_that("print.svySE_result returns the result invisibly", {

  skip_if_not_installed("survey")

  result <- svySE_calc(
    data = make_design_data(),
    indicators = "ind_1",
    group_vars = "dept",
    group_labels = "Department",
    strata = "strata",
    cluster = "cluster",
    weight = "weight",
    cfg = make_design_cfg(),
    verbose = FALSE
  )

  output <- capture.output(
    returned <- print(result)
  )

  expect_identical(returned, result)
  expect_true(any(grepl("svySE sampling error result", output, fixed = TRUE)))
  expect_true(any(grepl("Simple tab : No", output, fixed = TRUE)))
})

# ==============================================================================
# Tests: divisiones en svySE_calc
# Tests: divisions in svySE_calc
# Archivo / File: tests/testthat/test-calculo-division.R
# ==============================================================================


make_division_data <- function() {

  set.seed(321)

  n <- 144

  data.frame(
    dept = rep(c("A", "B", "C"), each = 48),
    strata = rep(c("S1", "S2", "S3"), each = 48),
    cluster = rep(seq_len(36), each = 4),
    area = rep(c("Urban", "Rural"), length.out = n),
    weight = runif(n, 10, 40),
    div_weight = runif(n, 20, 60),
    ind_1 = rep(c(0, 1, 1, 0, 1, 0), length.out = n),
    ind_2 = rep(c(1, 0, 1, 1, 0, 0), length.out = n),
    stringsAsFactors = FALSE
  )
}


make_division_cfg <- function() {
  svySE_cfg(
    estimator = "prop",
    variance = "taylor",
    lonely_psu = "adjust",
    conf_level = 0.95,
    target = 1,
    valid_values = c(0, 1),
    truncate_lower_ci = TRUE,
    pct_mult = 100,
    deff = TRUE,
    cv = TRUE,
    na_rm = TRUE
  )
}


test_that("svySE_calc creates TOTAL and category results for division", {

  skip_if_not_installed("survey")

  result <- svySE_calc(
    data = make_division_data(),
    indicators = "ind_1",
    group_vars = "dept",
    group_labels = "Department",
    strata = "strata",
    cluster = "cluster",
    weight = "weight",
    division = "area",
    div_weight = NULL,
    cfg = make_division_cfg(),
    verbose = FALSE
  )

  divisions <- names(result$results$ind_1$error)

  expect_identical(divisions, c("TOTAL", "Rural", "Urban"))
  expect_true(all(vapply(
    result$results$ind_1$error,
    is.data.frame,
    logical(1)
  )))

  expect_identical(result$meta$division, "area")
  expect_null(result$meta$div_weight)
})


test_that("division results preserve national and grouped rows", {

  skip_if_not_installed("survey")

  result <- svySE_calc(
    data = make_division_data(),
    indicators = "ind_1",
    group_vars = "dept",
    group_labels = "Department",
    strata = "strata",
    cluster = "cluster",
    weight = "weight",
    division = "area",
    cfg = make_division_cfg(),
    verbose = FALSE
  )

  for (division_name in names(result$results$ind_1$error)) {

    tab <- result$results$ind_1$error[[division_name]]

    expect_equal(nrow(tab), 4)
    expect_equal(tab$dept[1], "NACIONAL")
    expect_setequal(tab$dept[-1], c("A", "B", "C"))
    expect_true(all(tab$est_pct >= 0 & tab$est_pct <= 100))
    expect_true(all(tab$n_unw >= 0))
  }
})


test_that("svySE_calc uses division weight when supplied", {

  skip_if_not_installed("survey")

  result <- svySE_calc(
    data = make_division_data(),
    indicators = "ind_1",
    group_vars = "dept",
    group_labels = "Department",
    strata = "strata",
    cluster = "cluster",
    weight = "weight",
    division = "area",
    div_weight = "div_weight",
    cfg = make_division_cfg(),
    verbose = FALSE
  )

  expect_identical(result$meta$weight, "weight")
  expect_identical(result$meta$division, "area")
  expect_identical(result$meta$div_weight, "div_weight")

  expect_true("TOTAL" %in% names(result$results$ind_1$error))
  expect_true("Urban" %in% names(result$results$ind_1$error))
  expect_true("Rural" %in% names(result$results$ind_1$error))
})


test_that("division weight changes category estimates but not TOTAL estimate", {

  skip_if_not_installed("survey")

  df <- make_division_data()
  df$div_weight <- ifelse(df$ind_1 == 1, df$weight * 5, df$weight)

  result_primary <- svySE_calc(
    data = df,
    indicators = "ind_1",
    group_vars = "dept",
    group_labels = "Department",
    strata = "strata",
    cluster = "cluster",
    weight = "weight",
    division = "area",
    div_weight = NULL,
    cfg = make_division_cfg(),
    verbose = FALSE
  )

  result_div_weight <- svySE_calc(
    data = df,
    indicators = "ind_1",
    group_vars = "dept",
    group_labels = "Department",
    strata = "strata",
    cluster = "cluster",
    weight = "weight",
    division = "area",
    div_weight = "div_weight",
    cfg = make_division_cfg(),
    verbose = FALSE
  )

  total_primary <- result_primary$results$ind_1$error$TOTAL$est_pct
  total_div <- result_div_weight$results$ind_1$error$TOTAL$est_pct

  expect_equal(total_primary, total_div)

  urban_primary <- result_primary$results$ind_1$error$Urban$est_pct
  urban_div <- result_div_weight$results$ind_1$error$Urban$est_pct

  expect_false(isTRUE(all.equal(urban_primary, urban_div)))
})


test_that("svySE_calc supports multiple indicators with division", {

  skip_if_not_installed("survey")

  result <- svySE_calc(
    data = make_division_data(),
    indicators = c("ind_1", "ind_2"),
    group_vars = "dept",
    group_labels = "Department",
    strata = "strata",
    cluster = "cluster",
    weight = "weight",
    division = "area",
    cfg = make_division_cfg(),
    verbose = FALSE
  )

  expect_setequal(names(result$results), c("ind_1", "ind_2"))

  for (indicator in c("ind_1", "ind_2")) {
    expect_identical(
      names(result$results[[indicator]]$error),
      c("TOTAL", "Rural", "Urban")
    )
    expect_false("simple" %in% names(result$results[[indicator]]))
  }
})


test_that("missing division values remain in TOTAL but not as a category", {

  skip_if_not_installed("survey")

  df <- make_division_data()
  df$area[seq(1, nrow(df), by = 12)] <- NA_character_

  expect_warning(
    result <- svySE_calc(
      data = df,
      indicators = "ind_1",
      group_vars = "dept",
      group_labels = "Department",
      strata = "strata",
      cluster = "cluster",
      weight = "weight",
      division = "area",
      cfg = make_division_cfg(),
      verbose = FALSE
    ),
    "valores perdidos"
  )

  expect_identical(
    names(result$results$ind_1$error),
    c("TOTAL", "Rural", "Urban")
  )

  expected_total_n <- sum(df$ind_1 == 1, na.rm = TRUE)
  observed_total_n <- result$results$ind_1$error$TOTAL$n_unw[1]

  expect_equal(observed_total_n, expected_total_n)
})


test_that("division results contain only error output", {

  skip_if_not_installed("survey")

  result <- svySE_calc(
    data = make_division_data(),
    indicators = "ind_1",
    group_vars = "dept",
    strata = "strata",
    cluster = "cluster",
    weight = "weight",
    division = "area",
    cfg = make_division_cfg(),
    verbose = FALSE
  )

  expect_identical(names(result$results$ind_1), "error")
  expect_false("simple" %in% names(result$results$ind_1))
})

# ==============================================================================
# Tests: validaciones de svySE_calc
# Tests: svySE_calc validations
# Archivo / File: tests/testthat/test-calculo-validaciones.R
# ==============================================================================


make_validation_data <- function() {

  data.frame(
    dept = rep(c("A", "B"), each = 12),
    strata = rep(c("S1", "S2"), each = 12),
    cluster = rep(seq_len(8), each = 3),
    weight = rep(10, 24),
    div_weight = rep(12, 24),
    area = rep(c("Urban", "Rural"), length.out = 24),
    ind_1 = rep(c(0, 1), length.out = 24),
    stringsAsFactors = FALSE
  )
}


make_validation_cfg <- function() {
  svySE_cfg(
    estimator = "prop",
    target = 1,
    valid_values = c(0, 1),
    lonely_psu = "adjust"
  )
}


test_that("svySE_calc rejects invalid configuration objects", {

  skip_if_not_installed("survey")

  df <- make_validation_data()

  expect_error(
    svySE_calc(
      data = df,
      indicators = "ind_1",
      group_vars = "dept",
      weight = "weight",
      cfg = list(estimator = "prop"),
      verbose = FALSE
    ),
    class = "svySE_error"
  )
})


test_that("svySE_calc rejects the ratio estimator", {

  skip_if_not_installed("survey")

  df <- make_validation_data()
  cfg_ratio <- svySE_cfg(estimator = "ratio")

  expect_error(
    svySE_calc(
      data = df,
      indicators = "ind_1",
      group_vars = "dept",
      weight = "weight",
      cfg = cfg_ratio,
      verbose = FALSE
    ),
    class = "svySE_error"
  )
})


test_that("svySE_calc validates data and character arguments", {

  skip_if_not_installed("survey")

  df <- make_validation_data()
  cfg <- make_validation_cfg()

  expect_error(
    svySE_calc(
      data = matrix(1:4, nrow = 2),
      indicators = "ind_1",
      group_vars = "dept",
      weight = "weight",
      cfg = cfg,
      verbose = FALSE
    ),
    class = "svySE_error"
  )

  expect_error(
    svySE_calc(
      data = df,
      indicators = 1,
      group_vars = "dept",
      weight = "weight",
      cfg = cfg,
      verbose = FALSE
    ),
    class = "svySE_error"
  )

  expect_error(
    svySE_calc(
      data = df,
      indicators = "ind_1",
      group_vars = 1,
      weight = "weight",
      cfg = cfg,
      verbose = FALSE
    ),
    class = "svySE_error"
  )

  expect_error(
    svySE_calc(
      data = df,
      indicators = "ind_1",
      group_vars = "dept",
      weight = NULL,
      cfg = cfg,
      verbose = FALSE
    ),
    class = "svySE_error"
  )
})


test_that("svySE_calc rejects missing variables", {

  skip_if_not_installed("survey")

  df <- make_validation_data()
  cfg <- make_validation_cfg()

  expect_error(
    svySE_calc(
      data = df,
      indicators = "missing_indicator",
      group_vars = "dept",
      weight = "weight",
      cfg = cfg,
      verbose = FALSE
    ),
    class = "svySE_error"
  )

  expect_error(
    svySE_calc(
      data = df,
      indicators = "ind_1",
      group_vars = "missing_group",
      weight = "weight",
      cfg = cfg,
      verbose = FALSE
    ),
    class = "svySE_error"
  )

  expect_error(
    svySE_calc(
      data = df,
      indicators = "ind_1",
      group_vars = "dept",
      strata = "missing_strata",
      weight = "weight",
      cfg = cfg,
      verbose = FALSE
    ),
    class = "svySE_error"
  )

  expect_error(
    svySE_calc(
      data = df,
      indicators = "ind_1",
      group_vars = "dept",
      cluster = "missing_cluster",
      weight = "weight",
      cfg = cfg,
      verbose = FALSE
    ),
    class = "svySE_error"
  )

  expect_error(
    svySE_calc(
      data = df,
      indicators = "ind_1",
      group_vars = "dept",
      weight = "missing_weight",
      cfg = cfg,
      verbose = FALSE
    ),
    class = "svySE_error"
  )
})


test_that("svySE_calc validates scalar weight, division, and division weight", {

  skip_if_not_installed("survey")

  df <- make_validation_data()
  cfg <- make_validation_cfg()

  expect_error(
    svySE_calc(
      data = df,
      indicators = "ind_1",
      group_vars = "dept",
      weight = c("weight", "div_weight"),
      cfg = cfg,
      verbose = FALSE
    ),
    class = "svySE_error"
  )

  expect_error(
    svySE_calc(
      data = df,
      indicators = "ind_1",
      group_vars = "dept",
      weight = "weight",
      division = c("area", "dept"),
      cfg = cfg,
      verbose = FALSE
    ),
    class = "svySE_error"
  )

  expect_error(
    svySE_calc(
      data = df,
      indicators = "ind_1",
      group_vars = "dept",
      weight = "weight",
      division = "area",
      div_weight = c("weight", "div_weight"),
      cfg = cfg,
      verbose = FALSE
    ),
    class = "svySE_error"
  )
})


test_that("svySE_calc validates group label length", {

  skip_if_not_installed("survey")

  df <- make_validation_data()
  cfg <- make_validation_cfg()

  expect_error(
    svySE_calc(
      data = df,
      indicators = "ind_1",
      group_vars = c("dept", "area"),
      group_labels = "Department",
      weight = "weight",
      cfg = cfg,
      verbose = FALSE
    ),
    class = "svySE_error"
  )
})


test_that("svySE_calc rejects missing values in grouping and design variables", {

  skip_if_not_installed("survey")

  cfg <- make_validation_cfg()

  df_group <- make_validation_data()
  df_group$dept[1] <- NA_character_

  expect_error(
    svySE_calc(
      data = df_group,
      indicators = "ind_1",
      group_vars = "dept",
      weight = "weight",
      cfg = cfg,
      verbose = FALSE
    ),
    class = "svySE_error"
  )

  df_strata <- make_validation_data()
  df_strata$strata[1] <- NA_character_

  expect_error(
    svySE_calc(
      data = df_strata,
      indicators = "ind_1",
      group_vars = "dept",
      strata = "strata",
      weight = "weight",
      cfg = cfg,
      verbose = FALSE
    ),
    class = "svySE_error"
  )

  df_cluster <- make_validation_data()
  df_cluster$cluster[1] <- NA_integer_

  expect_error(
    svySE_calc(
      data = df_cluster,
      indicators = "ind_1",
      group_vars = "dept",
      cluster = "cluster",
      weight = "weight",
      cfg = cfg,
      verbose = FALSE
    ),
    class = "svySE_error"
  )
})


test_that("svySE_calc validates sampling weight values", {

  skip_if_not_installed("survey")

  cfg <- make_validation_cfg()

  df_negative <- make_validation_data()
  df_negative$weight[1] <- -1

  expect_error(
    svySE_calc(
      data = df_negative,
      indicators = "ind_1",
      group_vars = "dept",
      weight = "weight",
      cfg = cfg,
      verbose = FALSE
    ),
    class = "svySE_error"
  )

  df_missing <- make_validation_data()
  df_missing$weight <- NA_real_

  expect_error(
    svySE_calc(
      data = df_missing,
      indicators = "ind_1",
      group_vars = "dept",
      weight = "weight",
      cfg = cfg,
      verbose = FALSE
    ),
    class = "svySE_error"
  )

  df_zero <- make_validation_data()
  df_zero$weight <- 0

  expect_error(
    svySE_calc(
      data = df_zero,
      indicators = "ind_1",
      group_vars = "dept",
      weight = "weight",
      cfg = cfg,
      verbose = FALSE
    ),
    class = "svySE_error"
  )
})


test_that("svySE_calc warns about zero and non-numeric weights", {

  skip_if_not_installed("survey")

  cfg <- make_validation_cfg()

  df_zero <- make_validation_data()
  df_zero$weight[1] <- 0

  expect_warning(
    svySE_calc(
      data = df_zero,
      indicators = "ind_1",
      group_vars = "dept",
      weight = "weight",
      cfg = cfg,
      verbose = FALSE
    ),
    "valores cero"
  )

  df_text <- make_validation_data()
  df_text$weight <- as.character(df_text$weight)
  df_text$weight[1] <- "invalid"

  expect_warning(
    result <- svySE_calc(
      data = df_text,
      indicators = "ind_1",
      group_vars = "dept",
      weight = "weight",
      cfg = cfg,
      verbose = FALSE
    ),
    "no numericos"
  )

  expect_s3_class(result, "svySE_result")
})


test_that("svySE_calc handles invalid indicator values according to strict", {

  skip_if_not_installed("survey")

  cfg <- make_validation_cfg()
  df <- make_validation_data()
  df$ind_1[1] <- 2

  expect_warning(
    result <- svySE_calc(
      data = df,
      indicators = "ind_1",
      group_vars = "dept",
      weight = "weight",
      cfg = cfg,
      strict = FALSE,
      verbose = FALSE
    ),
    "fuera de `valid_values`"
  )

  expect_s3_class(result, "svySE_result")

  expect_error(
    svySE_calc(
      data = df,
      indicators = "ind_1",
      group_vars = "dept",
      weight = "weight",
      cfg = cfg,
      strict = TRUE,
      verbose = FALSE
    ),
    class = "svySE_error"
  )
})


test_that("svySE_calc warns about missing division values", {

  skip_if_not_installed("survey")

  cfg <- make_validation_cfg()
  df <- make_validation_data()
  df$area[1] <- NA_character_

  expect_warning(
    result <- svySE_calc(
      data = df,
      indicators = "ind_1",
      group_vars = "dept",
      weight = "weight",
      division = "area",
      cfg = cfg,
      verbose = FALSE
    ),
    "valores perdidos"
  )

  expect_s3_class(result, "svySE_result")
})

# ==============================================================================
# svySE: calculo de errores muestrales
# svySE: sampling error calculation
# Archivo / File: R/calculo.R
# ==============================================================================


#' Calculate sampling errors
#'
#' Calcula errores muestrales para uno o mas indicadores usando un data.frame
#' previamente cargado por el usuario.
#'
#' Calculates sampling errors for one or more indicators using a data.frame
#' previously loaded by the user.
#'
#' @param data Base de datos ya cargada en R / Data frame already loaded in R.
#' @param indicators Vector con los nombres de los indicadores.
#' @param group_vars Variables de agrupacion.
#' @param group_labels Etiquetas de las variables de agrupacion.
#' @param strata Variable o variables de estrato.
#' @param weight Variable de peso principal.
#' @param division Variable de division opcional.
#' @param div_weight Variable de peso opcional para las divisiones.
#' @param cfg Configuracion creada con `svySE_cfg()`.
#' @param verbose Si es `TRUE`, imprime avance del calculo.
#' @param strict Si es `TRUE`, detiene el calculo cuando encuentra valores
#'   fuera de `cfg$valid_values`; si es `FALSE`, los excluye con advertencia.
#'
#' @return Objeto de clase `"svySE_result"`.
#'
#' @importFrom stats update reformulate confint coef
#' @importFrom utils head
#' @export
svySE_calc <- function(
    data,
    indicators,
    group_vars,
    group_labels = group_vars,
    strata,
    weight,
    division = NULL,
    div_weight = NULL,
    cfg = svySE_cfg(),
    verbose = TRUE,
    strict = FALSE
) {
  
  # ---------------------------------------------------------------------------
  # Validaciones iniciales de configuracion
  # Initial configuration validations
  # ---------------------------------------------------------------------------
  
  svySE_chk_pkg("survey")
  
  if (!svySE_is_cfg(cfg)) {
    svySE_abort(
      title = "Configuracion invalida / Invalid configuration.",
      details = "`cfg` debe ser creado con `svySE_cfg()` / `cfg` must be created with `svySE_cfg()`.",
      hint = "Ejemplo / Example: cfg <- svySE_cfg(estimator = \"prop\", target = 1)"
    )
  }
  
  svySE_chk_bool(verbose, "verbose")
  svySE_chk_bool(strict, "strict")
  
  svySE_chk_estimator(
    cfg = cfg,
    indicators = indicators
  )
  
  # ---------------------------------------------------------------------------
  # Validaciones generales de datos y variables
  # General data and variable validations
  # ---------------------------------------------------------------------------
  
  svySE_chk_df(data)
  svySE_chk_chr(indicators, "indicators")
  svySE_chk_chr(group_vars, "group_vars")
  svySE_chk_chr(group_labels, "group_labels")
  svySE_chk_chr(strata, "strata")
  svySE_chk_chr(weight, "weight")
  
  if (length(weight) != 1) {
    svySE_abort(
      title = "Peso invalido / Invalid weight.",
      details = "`weight` debe contener una sola variable / `weight` must contain one variable.",
      vars = list(weight = weight)
    )
  }
  
  if (!is.null(division)) {
    svySE_chk_chr(division, "division")
    
    if (length(division) != 1) {
      svySE_abort(
        title = "Variable de division invalida / Invalid division variable.",
        details = "`division` debe contener una sola variable / `division` must contain one variable.",
        vars = list(division = division)
      )
    }
  }
  
  if (!is.null(div_weight)) {
    svySE_chk_chr(div_weight, "div_weight")
    
    if (length(div_weight) != 1) {
      svySE_abort(
        title = "Peso de division invalido / Invalid division weight.",
        details = "`div_weight` debe contener una sola variable / `div_weight` must contain one variable.",
        vars = list(div_weight = div_weight)
      )
    }
  }
  
  svySE_chk_vars(data, indicators, "indicators")
  svySE_chk_vars(data, group_vars, "group_vars")
  svySE_chk_vars(data, strata, "strata")
  svySE_chk_vars(data, weight, "weight")
  
  if (!is.null(division)) {
    svySE_chk_vars(data, division, "division")
  }
  
  if (!is.null(div_weight)) {
    svySE_chk_vars(data, div_weight, "div_weight")
  }
  
  if (length(group_labels) != length(group_vars)) {
    svySE_abort(
      title = "Etiquetas de grupo incompatibles / Incompatible group labels.",
      details = "`group_labels` debe tener la misma longitud que `group_vars` / `group_labels` must have the same length as `group_vars`.",
      vars = list(
        group_vars = group_vars,
        group_labels = group_labels
      ),
      hint = "Si solo agrupas por departamento, usa por ejemplo: group_vars = \"NOMBRECCDD\", group_labels = \"DEPARTAMENTO\"."
    )
  }
  
  # ---------------------------------------------------------------------------
  # Copia interna para no modificar la base original
  # Internal copy to avoid modifying original data
  # ---------------------------------------------------------------------------
  
  data <- as.data.frame(data)
  
  data <- svySE_prepare_weight(
    data = data,
    weight = weight,
    arg = "weight"
  )
  
  if (!is.null(div_weight)) {
    data <- svySE_prepare_weight(
      data = data,
      weight = div_weight,
      arg = "div_weight"
    )
  }
  
  svySE_chk_required_no_na(
    data = data,
    vars = c(group_vars, strata),
    context = "variables de grupo y estrato / group and strata variables"
  )
  
  if (!is.null(division)) {
    svySE_warn_na_division(data, division)
  }
  
  # ---------------------------------------------------------------------------
  # Calculo por indicador
  # Calculation by indicator
  # ---------------------------------------------------------------------------
  
  out <- list()
  
  for (ind in indicators) {
    
    if (isTRUE(verbose)) {
      message("Procesando / Processing: ", ind)
    }
    
    res_ind <- tryCatch({
      
      svySE_chk_indicator_values(
        data = data,
        indicator = ind,
        cfg = cfg,
        strict = strict
      )
      
      data_ind <- svySE_prep_ind(
        data = data,
        indicator = ind,
        group_vars = group_vars,
        weight = weight,
        cfg = cfg
      )
      
      groups_master <- sort(unique(data_ind$.__svySE_group_id__))
      
      if (length(groups_master) == 0) {
        svySE_abort(
          title = "No se encontraron grupos validos / No valid groups found.",
          details = "Despues de filtrar valores validos del indicador y peso, no quedaron grupos para calcular.",
          vars = list(
            indicator = ind,
            group_vars = group_vars,
            weight = weight
          )
        )
      }
      
      filters <- svySE_filters(
        data = data_ind,
        division = division,
        weight = weight,
        div_weight = div_weight
      )
      
      err_list <- list()
      tab_list <- list()
      
      for (i in seq_along(filters)) {
        
        f <- filters[[i]]
        
        if (isTRUE(verbose)) {
          message("  Division / Division: ", f$name, " | Peso / Weight: ", f$weight)
        }
        
        res_div <- tryCatch({
          
          err_list[[f$name]] <- svySE_err_one(
            data = f$data,
            group_vars = group_vars,
            strata = strata,
            weight = f$weight,
            indicator = ind,
            groups_master = groups_master,
            cfg = cfg
          )
          
          tab_list[[f$name]] <- svySE_tab_one(
            data = f$data,
            group_vars = group_vars,
            indicator = ind,
            groups_master = groups_master,
            cfg = cfg
          )
          
          TRUE
          
        }, error = function(e) {
          
          svySE_abort(
            title = "Error al calcular una division / Error while calculating a division.",
            details = paste0(
              "El calculo fallo dentro del indicador `", ind,
              "` y la division `", f$name, "`."
            ),
            vars = list(
              indicator = ind,
              division = f$name,
              group_vars = group_vars,
              strata = strata,
              weight_used = f$weight,
              estimator = cfg$estimator,
              target = cfg$target,
              valid_values = cfg$valid_values
            ),
            hint = "Revisa que el peso no tenga valores negativos, que el estrato tenga registros validos y que el indicador tenga valores compatibles con `valid_values`.",
            parent = e
          )
        })
        
        invisible(res_div)
      }
      
      list(
        error = err_list,
        simple = tab_list
      )
      
    }, error = function(e) {
      
      svySE_abort(
        title = "Error al calcular un indicador / Error while calculating an indicator.",
        details = paste0("El calculo fallo para el indicador `", ind, "`."),
        vars = list(
          indicator = ind,
          group_vars = group_vars,
          strata = strata,
          weight = weight,
          division = if (is.null(division)) "NULL" else division,
          div_weight = if (is.null(div_weight)) "NULL" else div_weight,
          estimator = cfg$estimator,
          target = cfg$target,
          valid_values = cfg$valid_values
        ),
        hint = "Revisa el mensaje tecnico original al final. Usualmente el problema esta en valores invalidos, pesos no numericos, estratos sin variacion o uso incorrecto del estimador.",
        parent = e
      )
    })
    
    out[[ind]] <- res_ind
  }
  
  result <- list(
    results = out,
    meta = list(
      indicators = indicators,
      group_vars = group_vars,
      group_labels = group_labels,
      strata = strata,
      weight = weight,
      division = division,
      div_weight = div_weight,
      cfg = cfg,
      strict = strict
    )
  )
  
  class(result) <- c("svySE_result", "list")
  
  result
}


# ==============================================================================
# Calculo de tabla de error por indicador y division
# Error table calculation by indicator and division
# ==============================================================================

#' @keywords internal
svySE_err_one <- function(
    data,
    group_vars,
    strata,
    weight,
    indicator,
    groups_master,
    cfg
) {
  
  metric_names <- svySE_cols_err_all()
  
  out <- data.frame()
  
  groups_in_data <- sort(unique(data$.__svySE_group_id__))
  
  if (nrow(data) == 0) {
    return(
      svySE_empty_by_groups(
        groups_master = groups_master,
        group_vars = group_vars,
        metric_names = metric_names
      )
    )
  }
  
  design <- tryCatch(
    svySE_design(
      data = data,
      strata = strata,
      weight = weight
    ),
    error = function(e) {
      svySE_abort(
        title = "No se pudo construir el diseno muestral / Could not build survey design.",
        details = "El error ocurrio al ejecutar `survey::svydesign()`.",
        vars = list(
          strata = strata,
          weight = weight,
          n_rows = nrow(data)
        ),
        hint = "Verifica que el peso sea numerico, no negativo, no todo NA, y que la variable de estrato exista y no tenga valores perdidos.",
        parent = e
      )
    }
  )
  
  for (g in groups_master) {
    
    if (!(g %in% groups_in_data)) {
      
      row_na <- svySE_na_row(
        group_id = g,
        group_vars = group_vars,
        metric_names = metric_names
      )
      
      out <- rbind(out, row_na)
      next
    }
    
    group_data <- data[data$.__svySE_group_id__ == g, , drop = FALSE]
    group_row <- group_data[1, group_vars, drop = FALSE]
    
    design_dom <- tryCatch(
      update(
        design,
        .__svySE_dom__ = ifelse(.__svySE_group_id__ == g, 1, 0)
      ),
      error = function(e) {
        svySE_abort(
          title = "No se pudo crear el dominio / Could not create domain.",
          details = paste0("Fallo la creacion del dominio para el grupo `", g, "`."),
          vars = list(
            group_id = g,
            group_vars = group_vars
          ),
          parent = e
        )
      }
    )
    
    metrics <- tryCatch(
      svySE_est_dom(
        design = design_dom,
        indicator = indicator,
        domain_var = ".__svySE_dom__",
        cfg = cfg,
        data_raw = data,
        group_id = g
      ),
      error = function(e) {
        svySE_abort(
          title = "No se pudo estimar el dominio / Could not estimate domain.",
          details = paste0("Fallo la estimacion para el grupo `", g, "`."),
          vars = list(
            indicator = indicator,
            group_id = g,
            estimator = cfg$estimator,
            target = cfg$target
          ),
          hint = "Si el indicador es binario o categorico, usa `estimator = \"prop\"`. Si el dominio tiene pocos casos, revisa el tamano muestral no ponderado.",
          parent = e
        )
      }
    )
    
    out <- rbind(out, cbind(group_row, metrics))
  }
  
  total_metrics <- tryCatch(
    svySE_est_total(
      design = design,
      indicator = indicator,
      cfg = cfg,
      data_raw = data
    ),
    error = function(e) {
      svySE_abort(
        title = "No se pudo calcular el total nacional / Could not calculate national total.",
        details = "Fallo la estimacion total del indicador.",
        vars = list(
          indicator = indicator,
          estimator = cfg$estimator,
          target = cfg$target
        ),
        parent = e
      )
    }
  )
  
  total_group <- as.data.frame(
    as.list(rep("NACIONAL", length(group_vars))),
    stringsAsFactors = FALSE
  )
  
  names(total_group) <- group_vars
  
  out <- rbind(cbind(total_group, total_metrics), out)
  
  rownames(out) <- NULL
  
  out
}


# ==============================================================================
# Estimacion por dominio
# Domain estimation
# ==============================================================================

#' @keywords internal
svySE_est_dom <- function(
    design,
    indicator,
    domain_var,
    cfg,
    data_raw,
    group_id
) {
  
  if (cfg$estimator == "prop") {
    
    total_obj <- survey::svytotal(
      stats::as.formula(
        paste0("~ I(.__svySE_cat__ * ", domain_var, ")")
      ),
      design
    )
    
    prop_obj <- survey::svyratio(
      numerator = stats::as.formula(
        paste0("~ I(.__svySE_cat__ * ", domain_var, ")")
      ),
      denominator = stats::as.formula(
        paste0("~ ", domain_var)
      ),
      design = design,
      deff = cfg$deff
    )
    
    ci_total <- stats::confint(total_obj, level = cfg$conf_level)
    ci_prop  <- stats::confint(prop_obj, level = cfg$conf_level)
    
    return(
      svySE_metrics(
        est_abs = as.numeric(stats::coef(total_obj)[1]),
        est_pct = as.numeric(stats::coef(prop_obj)[1]) * cfg$pct_mult,
        se_abs = as.numeric(survey::SE(total_obj)[1]),
        se_pct = as.numeric(survey::SE(prop_obj)[1]) * cfg$pct_mult,
        ci_l_abs = as.numeric(ci_total[1]),
        ci_l_pct = as.numeric(ci_prop[1]) * cfg$pct_mult,
        ci_u_abs = as.numeric(ci_total[2]),
        ci_u_pct = as.numeric(ci_prop[2]) * cfg$pct_mult,
        cv = svySE_safe_cv(prop_obj, cfg$pct_mult),
        deff = svySE_safe_deff(prop_obj),
        n_unw = sum(
          data_raw$.__svySE_group_id__ == group_id &
            data_raw$.__svySE_cat__ == 1,
          na.rm = TRUE
        ),
        cfg = cfg
      )
    )
  }
  
  if (cfg$estimator == "total") {
    
    total_obj <- survey::svytotal(
      stats::as.formula(
        paste0("~ I(.__svySE_cat__ * ", domain_var, ")")
      ),
      design,
      deff = cfg$deff
    )
    
    ci_total <- stats::confint(total_obj, level = cfg$conf_level)
    
    return(
      svySE_metrics(
        est_abs = as.numeric(stats::coef(total_obj)[1]),
        est_pct = NA_real_,
        se_abs = as.numeric(survey::SE(total_obj)[1]),
        se_pct = NA_real_,
        ci_l_abs = as.numeric(ci_total[1]),
        ci_l_pct = NA_real_,
        ci_u_abs = as.numeric(ci_total[2]),
        ci_u_pct = NA_real_,
        cv = svySE_safe_cv(total_obj, cfg$pct_mult),
        deff = svySE_safe_deff(total_obj),
        n_unw = sum(
          data_raw$.__svySE_group_id__ == group_id &
            data_raw$.__svySE_cat__ == 1,
          na.rm = TRUE
        ),
        cfg = cfg
      )
    )
  }
  
  if (cfg$estimator == "mean") {
    
    mean_obj <- survey::svyratio(
      numerator = stats::as.formula(
        paste0("~ I(", indicator, " * ", domain_var, ")")
      ),
      denominator = stats::as.formula(
        paste0("~ ", domain_var)
      ),
      design = design,
      deff = cfg$deff
    )
    
    ci_mean <- stats::confint(mean_obj, level = cfg$conf_level)
    
    return(
      svySE_metrics(
        est_abs = as.numeric(stats::coef(mean_obj)[1]),
        est_pct = NA_real_,
        se_abs = as.numeric(survey::SE(mean_obj)[1]),
        se_pct = NA_real_,
        ci_l_abs = as.numeric(ci_mean[1]),
        ci_l_pct = NA_real_,
        ci_u_abs = as.numeric(ci_mean[2]),
        ci_u_pct = NA_real_,
        cv = svySE_safe_cv(mean_obj, cfg$pct_mult),
        deff = svySE_safe_deff(mean_obj),
        n_unw = sum(data_raw$.__svySE_group_id__ == group_id, na.rm = TRUE),
        cfg = cfg
      )
    )
  }
  
  if (cfg$estimator == "ratio") {
    svySE_abort(
      title = "Estimador `ratio` no disponible en `svySE_calc()` / `ratio` estimator is not available in `svySE_calc()`.",
      details = "El estimador `ratio` requiere numerador y denominador explicitos.",
      hint = paste0(
        "Para indicadores binarios usa: cfg <- svySE_cfg(estimator = \"prop\", target = 1, valid_values = c(0, 1)). ",
        "La proporcion ya se calcula internamente como una razon entre el total ponderado del valor objetivo y el total ponderado del dominio."
      )
    )
  }
  
  svySE_abort(
    title = "Estimador no reconocido / Unknown estimator.",
    vars = list(estimator = cfg$estimator)
  )
}


# ==============================================================================
# Estimacion total nacional
# National total estimation
# ==============================================================================

#' @keywords internal
svySE_est_total <- function(
    design,
    indicator,
    cfg,
    data_raw
) {
  
  if (cfg$estimator == "prop") {
    
    total_obj <- survey::svytotal(
      ~ .__svySE_cat__,
      design
    )
    
    prop_obj <- survey::svymean(
      ~ .__svySE_cat__,
      design,
      deff = cfg$deff
    )
    
    ci_total <- stats::confint(total_obj, level = cfg$conf_level)
    ci_prop  <- stats::confint(prop_obj, level = cfg$conf_level)
    
    return(
      svySE_metrics(
        est_abs = as.numeric(stats::coef(total_obj)[1]),
        est_pct = as.numeric(stats::coef(prop_obj)[1]) * cfg$pct_mult,
        se_abs = as.numeric(survey::SE(total_obj)[1]),
        se_pct = as.numeric(survey::SE(prop_obj)[1]) * cfg$pct_mult,
        ci_l_abs = as.numeric(ci_total[1]),
        ci_l_pct = as.numeric(ci_prop[1]) * cfg$pct_mult,
        ci_u_abs = as.numeric(ci_total[2]),
        ci_u_pct = as.numeric(ci_prop[2]) * cfg$pct_mult,
        cv = svySE_safe_cv(prop_obj, cfg$pct_mult),
        deff = svySE_safe_deff(prop_obj),
        n_unw = sum(data_raw$.__svySE_cat__ == 1, na.rm = TRUE),
        cfg = cfg
      )
    )
  }
  
  if (cfg$estimator == "total") {
    
    total_obj <- survey::svytotal(
      ~ .__svySE_cat__,
      design,
      deff = cfg$deff
    )
    
    ci_total <- stats::confint(total_obj, level = cfg$conf_level)
    
    return(
      svySE_metrics(
        est_abs = as.numeric(stats::coef(total_obj)[1]),
        est_pct = NA_real_,
        se_abs = as.numeric(survey::SE(total_obj)[1]),
        se_pct = NA_real_,
        ci_l_abs = as.numeric(ci_total[1]),
        ci_l_pct = NA_real_,
        ci_u_abs = as.numeric(ci_total[2]),
        ci_u_pct = NA_real_,
        cv = svySE_safe_cv(total_obj, cfg$pct_mult),
        deff = svySE_safe_deff(total_obj),
        n_unw = sum(data_raw$.__svySE_cat__ == 1, na.rm = TRUE),
        cfg = cfg
      )
    )
  }
  
  if (cfg$estimator == "mean") {
    
    mean_obj <- survey::svymean(
      stats::as.formula(paste0("~", indicator)),
      design,
      deff = cfg$deff
    )
    
    ci_mean <- stats::confint(mean_obj, level = cfg$conf_level)
    
    return(
      svySE_metrics(
        est_abs = as.numeric(stats::coef(mean_obj)[1]),
        est_pct = NA_real_,
        se_abs = as.numeric(survey::SE(mean_obj)[1]),
        se_pct = NA_real_,
        ci_l_abs = as.numeric(ci_mean[1]),
        ci_l_pct = NA_real_,
        ci_u_abs = as.numeric(ci_mean[2]),
        ci_u_pct = NA_real_,
        cv = svySE_safe_cv(mean_obj, cfg$pct_mult),
        deff = svySE_safe_deff(mean_obj),
        n_unw = nrow(data_raw),
        cfg = cfg
      )
    )
  }
  
  svySE_abort(
    title = "Estimador no reconocido / Unknown estimator.",
    vars = list(estimator = cfg$estimator)
  )
}


# ==============================================================================
# Tabla simple
# Simple table
# ==============================================================================

#' @keywords internal
svySE_tab_one <- function(
    data,
    group_vars,
    indicator,
    groups_master,
    cfg
) {
  
  metric_names <- svySE_cols_tab_all()
  
  out <- data.frame()
  
  for (g in groups_master) {
    
    group_data <- data[data$.__svySE_group_id__ == g, , drop = FALSE]
    
    if (nrow(group_data) == 0) {
      
      row_na <- svySE_na_row(
        group_id = g,
        group_vars = group_vars,
        metric_names = metric_names
      )
      
      out <- rbind(out, row_na)
      next
    }
    
    group_row <- group_data[1, group_vars, drop = FALSE]
    
    m <- data.frame(
      freq_0 = sum(group_data[[indicator]] == 0, na.rm = TRUE),
      pct_0 = mean(group_data[[indicator]] == 0, na.rm = TRUE) * cfg$pct_mult,
      freq_1 = sum(group_data[[indicator]] == cfg$target, na.rm = TRUE),
      pct_1 = mean(group_data[[indicator]] == cfg$target, na.rm = TRUE) * cfg$pct_mult,
      freq_total = nrow(group_data),
      pct_total = cfg$pct_mult
    )
    
    out <- rbind(out, cbind(group_row, m))
  }
  
  total_group <- as.data.frame(
    as.list(rep("NACIONAL", length(group_vars))),
    stringsAsFactors = FALSE
  )
  
  names(total_group) <- group_vars
  
  total_m <- data.frame(
    freq_0 = sum(data[[indicator]] == 0, na.rm = TRUE),
    pct_0 = mean(data[[indicator]] == 0, na.rm = TRUE) * cfg$pct_mult,
    freq_1 = sum(data[[indicator]] == cfg$target, na.rm = TRUE),
    pct_1 = mean(data[[indicator]] == cfg$target, na.rm = TRUE) * cfg$pct_mult,
    freq_total = nrow(data),
    pct_total = cfg$pct_mult
  )
  
  out <- rbind(cbind(total_group, total_m), out)
  
  rownames(out) <- NULL
  
  out
}


# ==============================================================================
# Preparacion de datos
# Data preparation
# ==============================================================================

#' @keywords internal
svySE_prep_ind <- function(
    data,
    indicator,
    group_vars,
    weight,
    cfg
) {
  
  data <- data[
    data[[indicator]] %in% cfg$valid_values &
      !is.na(data[[weight]]),
    ,
    drop = FALSE
  ]
  
  if (nrow(data) == 0) {
    svySE_abort(
      title = "Indicador sin registros validos / Indicator without valid records.",
      details = paste0(
        "El indicador `", indicator,
        "` no tiene registros despues de filtrar `valid_values` y peso no perdido."
      ),
      vars = list(
        indicator = indicator,
        valid_values = cfg$valid_values,
        weight = weight
      ),
      hint = "Revisa si el indicador esta codificado como 0/1, TRUE/FALSE, texto, o si el peso tiene valores perdidos."
    )
  }
  
  data$.__svySE_cat__ <- ifelse(data[[indicator]] == cfg$target, 1, 0)
  
  data$.__svySE_group_id__ <- do.call(
    paste,
    c(data[group_vars], sep = " | ")
  )
  
  data
}


#' @keywords internal
svySE_filters <- function(
    data,
    division = NULL,
    weight,
    div_weight = NULL
) {
  
  if (is.null(division)) {
    return(
      list(
        TOTAL = list(
          name = "TOTAL",
          data = data,
          weight = weight
        )
      )
    )
  }
  
  cats <- sort(unique(data[[division]]))
  cats <- cats[!is.na(cats)]
  
  if (length(cats) == 0) {
    svySE_abort(
      title = "Variable de division sin categorias validas / Division variable without valid categories.",
      details = paste0("La variable `", division, "` no tiene categorias no perdidas."),
      vars = list(division = division)
    )
  }
  
  filters <- list(
    TOTAL = list(
      name = "TOTAL",
      data = data,
      weight = weight
    )
  )
  
  for (cat in cats) {
    
    data_cat <- data[data[[division]] == cat, , drop = FALSE]
    
    this_weight <- if (!is.null(div_weight)) div_weight else weight
    
    if (!is.null(div_weight)) {
      data_cat <- data_cat[!is.na(data_cat[[div_weight]]), , drop = FALSE]
    }
    
    filters[[as.character(cat)]] <- list(
      name = as.character(cat),
      data = data_cat,
      weight = this_weight
    )
  }
  
  filters
}


#' @keywords internal
svySE_design <- function(
    data,
    strata,
    weight
) {
  
  data <- data[!is.na(data[[weight]]), , drop = FALSE]
  
  if (nrow(data) == 0) {
    svySE_abort(
      title = "No hay registros con peso valido / No records with valid weight.",
      vars = list(weight = weight)
    )
  }
  
  svySE_chk_weight_values(
    data = data,
    weight = weight
  )
  
  survey::svydesign(
    ids = ~1,
    strata = stats::reformulate(strata),
    weights = stats::reformulate(weight),
    data = data
  )
}


# ==============================================================================
# Metricas
# Metrics
# ==============================================================================

#' @keywords internal
svySE_metrics <- function(
    est_abs,
    est_pct,
    se_abs,
    se_pct,
    ci_l_abs,
    ci_l_pct,
    ci_u_abs,
    ci_u_pct,
    cv,
    deff,
    n_unw,
    cfg
) {
  
  ci_l_abs <- as.numeric(ci_l_abs)
  ci_l_pct <- as.numeric(ci_l_pct)
  
  if (cfg$truncate_lower_ci) {
    
    if (is.finite(ci_l_abs)) {
      ci_l_abs <- max(ci_l_abs, 0)
    }
    
    if (is.finite(ci_l_pct)) {
      ci_l_pct <- max(ci_l_pct, 0)
    }
  }
  
  if (!cfg$cv) {
    cv <- NA_real_
  }
  
  if (!cfg$deff) {
    deff <- NA_real_
  }
  
  data.frame(
    est_abs = as.numeric(est_abs),
    est_pct = as.numeric(est_pct),
    se_abs = as.numeric(se_abs),
    se_pct = as.numeric(se_pct),
    ci_l_abs = as.numeric(ci_l_abs),
    ci_l_pct = as.numeric(ci_l_pct),
    ci_u_abs = as.numeric(ci_u_abs),
    ci_u_pct = as.numeric(ci_u_pct),
    cv = as.numeric(cv),
    deff = as.numeric(deff),
    n_unw = as.numeric(n_unw)
  )
}


#' @keywords internal
svySE_safe_cv <- function(
    obj,
    pct_mult = 100
) {
  
  out <- tryCatch(
    as.numeric(survey::cv(obj)[1]) * pct_mult,
    error = function(e) NA_real_
  )
  
  if (!is.finite(out)) {
    out <- NA_real_
  }
  
  out
}


#' @keywords internal
svySE_safe_deff <- function(obj) {
  
  out <- tryCatch(
    as.numeric(survey::deff(obj)),
    error = function(e) NA_real_
  )
  
  if (!is.finite(out)) {
    out <- NA_real_
  }
  
  out
}


# ==============================================================================
# Validaciones de usuario
# User validations
# ==============================================================================

#' @keywords internal
svySE_chk_pkg <- function(pkg) {
  
  if (!requireNamespace(pkg, quietly = TRUE)) {
    svySE_abort(
      title = "Paquete requerido no instalado / Required package not installed.",
      details = paste0("El paquete `", pkg, "` es requerido para ejecutar esta funcion."),
      hint = paste0("Instala el paquete con: install.packages(\"", pkg, "\")")
    )
  }
  
  invisible(TRUE)
}


#' @keywords internal
svySE_chk_df <- function(data) {
  
  if (!is.data.frame(data)) {
    svySE_abort(
      title = "`data` no es un data.frame / `data` is not a data.frame.",
      details = paste0("Clase recibida / received class: ", paste(class(data), collapse = ", ")),
      hint = "Carga o convierte tu base antes de usar svySE: data <- as.data.frame(data)."
    )
  }
  
  if (nrow(data) == 0) {
    svySE_abort(
      title = "`data` esta vacio / `data` is empty.",
      details = "La base no contiene filas."
    )
  }
  
  invisible(TRUE)
}


#' @keywords internal
svySE_chk_chr <- function(
    x,
    arg
) {
  
  if (is.null(x) || length(x) == 0) {
    svySE_abort(
      title = "Argumento vacio / Empty argument.",
      details = paste0("`", arg, "` debe contener al menos un valor."),
      vars = list(argument = arg)
    )
  }
  
  if (!is.character(x)) {
    svySE_abort(
      title = "Tipo de argumento invalido / Invalid argument type.",
      details = paste0("`", arg, "` debe ser un vector de texto / must be a character vector."),
      vars = list(
        argument = arg,
        received_class = class(x)
      )
    )
  }
  
  if (any(is.na(x)) || any(trimws(x) == "")) {
    svySE_abort(
      title = "Nombres de variables invalidos / Invalid variable names.",
      details = paste0("`", arg, "` contiene NA o textos vacios."),
      vars = list(argument = arg, values = x)
    )
  }
  
  invisible(TRUE)
}


#' @keywords internal
svySE_chk_vars <- function(
    data,
    vars,
    arg
) {
  
  missing_vars <- setdiff(vars, names(data))
  
  if (length(missing_vars) > 0) {
    svySE_abort(
      title = "Variables no encontradas / Variables not found.",
      details = paste0("Algunas variables indicadas en `", arg, "` no existen en `data`."),
      vars = list(
        argument = arg,
        missing_vars = missing_vars,
        available_example = head(names(data), 30)
      ),
      hint = "Usa `names(data)` para revisar los nombres exactos. Recuerda que R distingue mayusculas y minusculas."
    )
  }
  
  invisible(TRUE)
}


#' @keywords internal
svySE_chk_estimator <- function(
    cfg,
    indicators
) {
  
  if (cfg$estimator == "ratio") {
    svySE_abort(
      title = "Configuracion no compatible / Incompatible configuration.",
      details = paste0(
        "Has usado `estimator = \"ratio\"` junto con `indicators`. ",
        "Esta funcion esta disenada para indicadores directos, especialmente binarios o categoricos."
      ),
      vars = list(
        estimator = cfg$estimator,
        indicators = indicators,
        target = cfg$target,
        valid_values = cfg$valid_values
      ),
      hint = paste0(
        "Para indicadores binarios como Indicador_1, Indicador_2, etc., usa: ",
        "cfg <- svySE_cfg(estimator = \"prop\", target = 1, valid_values = c(0, 1)). ",
        "Tecnicamente, `prop` ya calcula internamente una razon: total ponderado del valor objetivo dividido entre total ponderado del dominio."
      )
    )
  }
  
  invisible(TRUE)
}


#' @keywords internal
svySE_chk_indicator_values <- function(
    data,
    indicator,
    cfg,
    strict = FALSE
) {
  
  x <- data[[indicator]]
  
  n_total <- length(x)
  n_na <- sum(is.na(x))
  n_valid <- sum(x %in% cfg$valid_values, na.rm = TRUE)
  n_target <- sum(x == cfg$target, na.rm = TRUE)
  
  invalid_values <- setdiff(unique(x[!is.na(x)]), cfg$valid_values)
  
  if (n_valid == 0) {
    svySE_abort(
      title = "Indicador sin valores validos / Indicator without valid values.",
      details = paste0(
        "El indicador `", indicator,
        "` no contiene ningun valor dentro de `valid_values`."
      ),
      vars = list(
        indicator = indicator,
        n_total = n_total,
        n_na = n_na,
        valid_values = cfg$valid_values,
        observed_values = svySE_observed_values(x)
      ),
      hint = "Revisa si el indicador esta codificado como 1/2, TRUE/FALSE, texto, o si necesitas cambiar `valid_values`."
    )
  }
  
  if (n_target == 0 && cfg$estimator %in% c("prop", "total")) {
    warning(
      paste0(
        "El indicador `", indicator, "` no tiene casos con target = ",
        cfg$target, ". Las estimaciones pueden ser cero o no estimables."
      ),
      call. = FALSE
    )
  }
  
  if (length(invalid_values) > 0) {
    
    msg <- paste0(
      "El indicador `", indicator,
      "` contiene valores fuera de `valid_values`: ",
      paste(invalid_values, collapse = ", "),
      ". Estos registros seran excluidos del calculo."
    )
    
    if (isTRUE(strict)) {
      svySE_abort(
        title = "Indicador con valores no permitidos / Indicator with invalid values.",
        details = msg,
        vars = list(
          indicator = indicator,
          valid_values = cfg$valid_values,
          invalid_values = invalid_values,
          observed_values = svySE_observed_values(x)
        ),
        hint = "Corrige la codificacion del indicador o usa `strict = FALSE` para excluir esos registros."
      )
    } else {
      warning(msg, call. = FALSE)
    }
  }
  
  invisible(TRUE)
}


#' @keywords internal
svySE_prepare_weight <- function(
    data,
    weight,
    arg = "weight"
) {
  
  original <- data[[weight]]
  
  converted <- suppressWarnings(as.numeric(original))
  
  new_na <- sum(is.na(converted) & !is.na(original))
  
  if (new_na > 0) {
    warning(
      paste0(
        "La variable `", weight, "` indicada en `", arg,
        "` tiene ", new_na,
        " valores no numericos que fueron convertidos a NA."
      ),
      call. = FALSE
    )
  }
  
  data[[weight]] <- converted
  
  svySE_chk_weight_values(data, weight)
  
  data
}


#' @keywords internal
svySE_chk_weight_values <- function(
    data,
    weight
) {
  
  w <- data[[weight]]
  
  if (all(is.na(w))) {
    svySE_abort(
      title = "Peso completamente perdido / Weight is completely missing.",
      details = paste0("La variable de peso `", weight, "` solo contiene NA."),
      vars = list(weight = weight),
      hint = "Revisa si el nombre del peso es correcto o si fue leido como texto no convertible a numero."
    )
  }
  
  if (any(w < 0, na.rm = TRUE)) {
    svySE_abort(
      title = "Peso con valores negativos / Weight has negative values.",
      details = paste0("La variable `", weight, "` contiene pesos negativos."),
      vars = list(
        weight = weight,
        min_weight = min(w, na.rm = TRUE)
      ),
      hint = "Los pesos muestrales deben ser no negativos. Revisa la variable de factor de expansion."
    )
  }
  
  if (all(w == 0 | is.na(w))) {
    svySE_abort(
      title = "Peso sin valores positivos / Weight has no positive values.",
      details = paste0("La variable `", weight, "` no tiene valores positivos."),
      vars = list(weight = weight),
      hint = "Verifica que estes usando el factor de expansion correcto."
    )
  }
  
  if (any(w == 0, na.rm = TRUE)) {
    warning(
      paste0(
        "La variable de peso `", weight,
        "` contiene valores cero. Estos registros no aportaran al estimador ponderado."
      ),
      call. = FALSE
    )
  }
  
  invisible(TRUE)
}


#' @keywords internal
svySE_chk_required_no_na <- function(
    data,
    vars,
    context
) {
  
  for (v in vars) {
    
    n_na <- sum(is.na(data[[v]]))
    
    if (n_na > 0) {
      svySE_abort(
        title = "Variable requerida con valores perdidos / Required variable has missing values.",
        details = paste0(
          "La variable `", v, "` tiene ", n_na,
          " valores perdidos dentro de ", context, "."
        ),
        vars = list(
          variable = v,
          context = context,
          n_missing = n_na
        ),
        hint = "Depura la base antes del calculo o filtra registros con estrato/grupo perdido."
      )
    }
  }
  
  invisible(TRUE)
}


#' @keywords internal
svySE_warn_na_division <- function(
    data,
    division
) {
  
  n_na <- sum(is.na(data[[division]]))
  
  if (n_na > 0) {
    warning(
      paste0(
        "La variable de division `", division,
        "` tiene ", n_na,
        " valores perdidos. Esos registros se mantienen en TOTAL, ",
        "pero no forman una categoria de division."
      ),
      call. = FALSE
    )
  }
  
  invisible(TRUE)
}


# ==============================================================================
# Filas auxiliares
# Auxiliary rows
# ==============================================================================

#' @keywords internal
svySE_na_row <- function(
    group_id,
    group_vars,
    metric_names
) {
  
  group_values <- strsplit(group_id, " \\| ")[[1]]
  
  if (length(group_values) < length(group_vars)) {
    group_values <- c(
      group_values,
      rep("", length(group_vars) - length(group_values))
    )
  }
  
  group_values <- group_values[seq_along(group_vars)]
  
  group_df <- as.data.frame(
    as.list(group_values),
    stringsAsFactors = FALSE
  )
  
  names(group_df) <- group_vars
  
  metric_df <- as.data.frame(
    as.list(rep(NA_real_, length(metric_names))),
    stringsAsFactors = FALSE
  )
  
  names(metric_df) <- metric_names
  
  cbind(group_df, metric_df)
}


#' @keywords internal
svySE_empty_by_groups <- function(
    groups_master,
    group_vars,
    metric_names
) {
  
  out <- data.frame()
  
  for (g in groups_master) {
    out <- rbind(
      out,
      svySE_na_row(
        group_id = g,
        group_vars = group_vars,
        metric_names = metric_names
      )
    )
  }
  
  rownames(out) <- NULL
  
  out
}


# ==============================================================================
# Mensajes de error y diagnostico
# Error messages and diagnostics
# ==============================================================================

#' @keywords internal
svySE_abort <- function(
    title,
    details = NULL,
    vars = NULL,
    hint = NULL,
    parent = NULL
) {
  
  msg <- c(
    "",
    title
  )
  
  if (!is.null(details)) {
    msg <- c(
      msg,
      "",
      "Detalle / Detail:",
      details
    )
  }
  
  if (!is.null(vars)) {
    msg <- c(
      msg,
      "",
      "Variables involucradas / Involved variables:",
      svySE_format_vars(vars)
    )
  }
  
  if (!is.null(hint)) {
    msg <- c(
      msg,
      "",
      "Sugerencia / Hint:",
      hint
    )
  }
  
  if (!is.null(parent)) {
    msg <- c(
      msg,
      "",
      "Error tecnico original / Original technical error:",
      conditionMessage(parent)
    )
  }
  
  stop(
    structure(
      list(
        message = paste(msg, collapse = "\n"),
        call = NULL
      ),
      class = c("svySE_error", "error", "condition")
    )
  )
}


#' @keywords internal
svySE_format_vars <- function(vars) {
  
  out <- character(0)
  
  nms <- names(vars)
  
  if (is.null(nms)) {
    nms <- paste0("var_", seq_along(vars))
  }
  
  for (i in seq_along(vars)) {
    
    value <- vars[[i]]
    
    if (length(value) > 20) {
      value <- c(value[1:20], "...")
    }
    
    value <- paste(value, collapse = ", ")
    
    out <- c(
      out,
      paste0("- ", nms[i], ": ", value)
    )
  }
  
  out
}


#' @keywords internal
svySE_observed_values <- function(x) {
  
  vals <- unique(x[!is.na(x)])
  
  if (length(vals) > 20) {
    vals <- c(vals[1:20], "...")
  }
  
  vals
}


# ==============================================================================
# Impresion del resultado
# Print result
# ==============================================================================

#' Print svySE result
#'
#' Imprime un resumen del resultado de `svySE_calc()`.
#'
#' Prints a summary of the result from `svySE_calc()`.
#'
#' @param x Objeto de clase `"svySE_result"`.
#' @param ... Argumentos adicionales.
#'
#' @return Invisiblemente `x`.
#'
#' @export
print.svySE_result <- function(x, ...) {
  
  cat("svySE result\n")
  cat("--------------------------------------------------\n")
  cat("Indicators :", paste(x$meta$indicators, collapse = ", "), "\n")
  cat("Groups     :", paste(x$meta$group_vars, collapse = ", "), "\n")
  cat("Strata     :", paste(x$meta$strata, collapse = ", "), "\n")
  cat("Weight     :", x$meta$weight, "\n")
  cat("Division   :", ifelse(is.null(x$meta$division), "NULL", x$meta$division), "\n")
  cat("Estimator  :", x$meta$cfg$estimator, "\n")
  cat("Target     :", x$meta$cfg$target, "\n")
  cat("Strict     :", x$meta$strict, "\n")
  
  invisible(x)
}

# ==============================================================================
# Tests: calculo general
# Tests: general calculation
# Archivo / File: tests/testthat/test-calculo.R
# ==============================================================================


make_calculation_data <- function() {
  
  set.seed(456)
  
  data.frame(
    dept = rep(c("A", "B", "C"), each = 40),
    strata = rep(c("S1", "S2", "S3"), each = 40),
    cluster = rep(seq_len(30), each = 4),
    weight = runif(120, 10, 50),
    ind_1 = rep(c(0, 1, 1, 0, 1), length.out = 120),
    ind_2 = rep(c(1, 0, 0, 1, 1, 0), length.out = 120),
    stringsAsFactors = FALSE
  )
}


make_calculation_cfg <- function(
    estimator = "prop",
    target = 1,
    valid_values = c(0, 1),
    deff = TRUE,
    cv = TRUE
) {
  svySE_cfg(
    estimator = estimator,
    variance = "taylor",
    lonely_psu = "adjust",
    conf_level = 0.95,
    target = target,
    valid_values = valid_values,
    truncate_lower_ci = TRUE,
    pct_mult = 100,
    deff = deff,
    cv = cv,
    na_rm = TRUE
  )
}


test_that("svySE_calc returns a valid result with basic workflow", {
  
  skip_if_not_installed("survey")
  
  result <- svySE_calc(
    data = make_calculation_data(),
    indicators = "ind_1",
    group_vars = "dept",
    group_labels = "Department",
    strata = "strata",
    cluster = "cluster",
    weight = "weight",
    cfg = make_calculation_cfg(),
    verbose = FALSE
  )
  
  expect_s3_class(result, "svySE_result")
  expect_identical(names(result$results), "ind_1")
  expect_identical(names(result$results$ind_1), "error")
  expect_identical(names(result$results$ind_1$error), "TOTAL")
  expect_false("simple" %in% names(result$results$ind_1))
  
  expect_identical(result$meta$indicators, "ind_1")
  expect_identical(result$meta$group_vars, "dept")
  expect_identical(result$meta$group_labels, "Department")
  expect_identical(result$meta$strata, "strata")
  expect_identical(result$meta$cluster, "cluster")
  expect_identical(result$meta$weight, "weight")
  expect_null(result$meta$division)
  expect_null(result$meta$div_weight)
})


test_that("svySE_calc works with multiple indicators", {
  
  skip_if_not_installed("survey")
  
  result <- svySE_calc(
    data = make_calculation_data(),
    indicators = c("ind_1", "ind_2"),
    group_vars = "dept",
    group_labels = "Department",
    strata = "strata",
    cluster = "cluster",
    weight = "weight",
    cfg = make_calculation_cfg(),
    verbose = FALSE
  )
  
  expect_setequal(names(result$results), c("ind_1", "ind_2"))
  
  for (indicator in c("ind_1", "ind_2")) {
    expect_identical(names(result$results[[indicator]]), "error")
    expect_true("TOTAL" %in% names(result$results[[indicator]]$error))
    expect_false("simple" %in% names(result$results[[indicator]]))
  }
})


test_that("sampling error table contains the expected columns", {
  
  skip_if_not_installed("survey")
  
  result <- svySE_calc(
    data = make_calculation_data(),
    indicators = "ind_1",
    group_vars = "dept",
    group_labels = "Department",
    strata = "strata",
    cluster = "cluster",
    weight = "weight",
    cfg = make_calculation_cfg(),
    verbose = FALSE
  )
  
  tab <- result$results$ind_1$error$TOTAL
  
  expected <- c(
    "dept",
    "est_abs",
    "est_pct",
    "se_abs",
    "se_pct",
    "ci_l_abs",
    "ci_l_pct",
    "ci_u_abs",
    "ci_u_pct",
    "cv",
    "deff",
    "n_unw"
  )
  
  expect_identical(names(tab), expected)
  expect_equal(tab$dept[1], "NACIONAL")
  expect_setequal(tab$dept[-1], c("A", "B", "C"))
})


test_that("national unweighted target count is correct", {
  
  skip_if_not_installed("survey")
  
  df <- make_calculation_data()
  
  result <- svySE_calc(
    data = df,
    indicators = "ind_1",
    group_vars = "dept",
    strata = "strata",
    cluster = "cluster",
    weight = "weight",
    cfg = make_calculation_cfg(),
    verbose = FALSE
  )
  
  tab <- result$results$ind_1$error$TOTAL
  expected_n <- sum(df$ind_1 == 1)
  
  expect_equal(tab$n_unw[1], expected_n)
  
  for (dept_name in c("A", "B", "C")) {
    expected_group_n <- sum(df$dept == dept_name & df$ind_1 == 1)
    observed_group_n <- tab$n_unw[tab$dept == dept_name]
    
    expect_equal(observed_group_n, expected_group_n)
  }
})


test_that("percentage estimates are expressed using pct_mult", {
  
  skip_if_not_installed("survey")
  
  df <- make_calculation_data()
  df$weight <- 1
  
  cfg_100 <- make_calculation_cfg()
  cfg_1 <- svySE_cfg(
    estimator = "prop",
    lonely_psu = "adjust",
    target = 1,
    valid_values = c(0, 1),
    pct_mult = 1,
    deff = TRUE,
    cv = TRUE
  )
  
  result_100 <- svySE_calc(
    data = df,
    indicators = "ind_1",
    group_vars = "dept",
    weight = "weight",
    cfg = cfg_100,
    verbose = FALSE
  )
  
  result_1 <- svySE_calc(
    data = df,
    indicators = "ind_1",
    group_vars = "dept",
    weight = "weight",
    cfg = cfg_1,
    verbose = FALSE
  )
  
  pct_100 <- result_100$results$ind_1$error$TOTAL$est_pct
  pct_1 <- result_1$results$ind_1$error$TOTAL$est_pct
  
  expect_equal(pct_100, pct_1 * 100, tolerance = 1e-8)
})


test_that("cv and deff can be disabled", {
  
  skip_if_not_installed("survey")
  
  result <- svySE_calc(
    data = make_calculation_data(),
    indicators = "ind_1",
    group_vars = "dept",
    strata = "strata",
    cluster = "cluster",
    weight = "weight",
    cfg = make_calculation_cfg(deff = FALSE, cv = FALSE),
    verbose = FALSE
  )
  
  tab <- result$results$ind_1$error$TOTAL
  
  expect_true(all(is.na(tab$cv)))
  expect_true(all(is.na(tab$deff)))
})


test_that("lower confidence limits are truncated at zero when requested", {
  
  cfg <- make_calculation_cfg()
  
  metrics <- svySE_metrics(
    est_abs = 1,
    est_pct = 5,
    se_abs = 2,
    se_pct = 10,
    ci_l_abs = -3,
    ci_l_pct = -15,
    ci_u_abs = 5,
    ci_u_pct = 25,
    cv = 10,
    deff = 1,
    n_unw = 2,
    cfg = cfg
  )
  
  expect_equal(metrics$ci_l_abs, 0)
  expect_equal(metrics$ci_l_pct, 0)
})


test_that("lower confidence limits are preserved when truncation is disabled", {
  
  cfg <- svySE_cfg(
    estimator = "prop",
    truncate_lower_ci = FALSE
  )
  
  metrics <- svySE_metrics(
    est_abs = 1,
    est_pct = 5,
    se_abs = 2,
    se_pct = 10,
    ci_l_abs = -3,
    ci_l_pct = -15,
    ci_u_abs = 5,
    ci_u_pct = 25,
    cv = 10,
    deff = 1,
    n_unw = 2,
    cfg = cfg
  )
  
  expect_equal(metrics$ci_l_abs, -3)
  expect_equal(metrics$ci_l_pct, -15)
})


test_that("svySE_calc does not modify the original data frame", {
  
  skip_if_not_installed("survey")
  
  df <- make_calculation_data()
  original <- df
  
  invisible(
    svySE_calc(
      data = df,
      indicators = "ind_1",
      group_vars = "dept",
      strata = "strata",
      cluster = "cluster",
      weight = "weight",
      cfg = make_calculation_cfg(),
      verbose = FALSE
    )
  )
  
  expect_identical(df, original)
  expect_false(".__svySE_cat__" %in% names(df))
  expect_false(".__svySE_group_id__" %in% names(df))
})


test_that("print.svySE_result returns the object invisibly", {
  
  skip_if_not_installed("survey")
  
  result <- svySE_calc(
    data = make_calculation_data(),
    indicators = "ind_1",
    group_vars = "dept",
    strata = "strata",
    cluster = "cluster",
    weight = "weight",
    cfg = make_calculation_cfg(),
    verbose = FALSE
  )
  
  output <- capture.output(
    returned <- print(result)
  )
  
  expect_identical(returned, result)
  
  # Validate relevant printed content without depending on one exact header.
  expect_true(any(grepl("svySE", output, fixed = TRUE)))
  expect_true(any(grepl("Indicators", output, fixed = TRUE)))
  expect_true(any(grepl("Groups", output, fixed = TRUE)))
  expect_true(any(grepl("Strata", output, fixed = TRUE)))
  expect_true(any(grepl("Cluster", output, fixed = TRUE)))
  expect_true(any(grepl("Weight", output, fixed = TRUE)))
})



test_that("verbose mode reports indicator and division progress", {
  
  skip_if_not_installed("survey")
  
  expect_message(
    svySE_calc(
      data = make_calculation_data(),
      indicators = "ind_1",
      group_vars = "dept",
      weight = "weight",
      cfg = make_calculation_cfg(),
      verbose = TRUE
    ),
    "Procesando"
  )
})

# ==============================================================================
# svySE: exportacion de resultados
# svySE: result export
# Archivo / File: R/exportar.R
# ==============================================================================


#' Export svySE results to '.xlsx' files
#'
#' Exporta uno o varios resultados generados por `svySE_calc()` y
#' `svySE_simple()` a archivos `.xlsx`.
#'
#' Exports one or more results generated by `svySE_calc()` and
#' `svySE_simple()` to `.xlsx` files.
#'
#' @param x Objeto generado por `svySE_calc()` o `svySE_simple()`, o una lista
#'   nombrada que contenga varios objetos de estas clases.
#'   Object generated by `svySE_calc()` or `svySE_simple()`, or a named list
#'   containing multiple objects of these classes.
#' @param select Vector de texto opcional con los nombres de los elementos de
#'   `x` que se desean exportar. Solo se utiliza cuando `x` es una lista de
#'   resultados. Si es `NULL`, se exportan todos.
#'   Optional character vector containing the names of the elements in `x` to
#'   export. It is only used when `x` is a list of results. If `NULL`, all
#'   results are exported.
#' @param file_err Ruta del archivo `.xlsx` para exportar las tablas de errores
#'   muestrales. Si es `NULL`, no se exportan.
#'   Path to the `.xlsx` file used to export sampling error tables. If `NULL`,
#'   they are not exported.
#' @param file_tab Ruta del archivo `.xlsx` para exportar las tablas simples.
#'   Si es `NULL`, no se exportan.
#'   Path to the `.xlsx` file used to export simple tables. If `NULL`, they are
#'   not exported.
#' @param cols_err Columnas de error a exportar.
#'   Sampling error columns to export.
#' @param cols_tab Columnas de tabla simple a exportar.
#'   Simple table columns to export.
#' @param start_row Fila inicial de escritura.
#'   Starting row for writing.
#' @param overwrite Sobrescribir archivos existentes.
#'   Whether existing files should be overwritten.
#' @param keep_na Mantener `NA` en el archivo exportado.
#'   Whether `NA` values should be preserved in the exported file.
#'
#' @details
#' Debe especificarse al menos uno de `file_err` o `file_tab`.
#'
#' Cuando `x` es una lista, sus elementos deben tener nombres unicos. Los
#' objetos de clase `"svySE_result"` se exportan a `file_err`, mientras que los
#' objetos de clase `"svySE_simple_result"` se exportan a `file_tab`.
#'
#' At least one of `file_err` or `file_tab` must be provided.
#'
#' When `x` is a list, its elements must have unique names. Objects of class
#' `"svySE_result"` are exported to `file_err`, while objects of class
#' `"svySE_simple_result"` are exported to `file_tab`.
#'
#' @return Invisiblemente una lista con las rutas de los archivos exportados y
#'   los resultados seleccionados.
#'   Invisibly returns a list containing the exported file paths and selected
#'   result names.
#'
#' @export
svySE_xlsx <- function(
    x,
    select = NULL,
    file_err = NULL,
    file_tab = NULL,
    cols_err = svySE_cols_err("full"),
    cols_tab = svySE_cols_tab("full"),
    start_row = 10,
    overwrite = TRUE,
    keep_na = FALSE
) {
  
  # ---------------------------------------------------------------------------
  # Dependencia
  # Dependency
  # ---------------------------------------------------------------------------
  
  if (!requireNamespace("openxlsx", quietly = TRUE)) {
    svySE_abort(
      title = "Paquete requerido no instalado / Required package not installed.",
      details = "El paquete `openxlsx` es requerido para exportar archivos `.xlsx`.",
      hint = "Instala el paquete con: install.packages(\"openxlsx\")"
    )
  }
  
  # ---------------------------------------------------------------------------
  # Normalizar entrada
  # Normalize input
  # ---------------------------------------------------------------------------
  
  valid_result <- function(obj) {
    inherits(obj, "svySE_result") ||
      inherits(obj, "svySE_simple_result")
  }
  
  is_single_result <- valid_result(x)
  
  if (is_single_result) {
    results <- list(Result = x)
    use_prefix <- FALSE
  } else {
    
    if (!is.list(x) || length(x) == 0) {
      svySE_abort(
        title = "Objeto invalido / Invalid object.",
        details = paste(
          "`x` debe ser generado por `svySE_calc()` o `svySE_simple()`,",
          "o ser una lista nombrada de estos resultados."
        )
      )
    }
    
    if (is.null(names(x)) || any(is.na(names(x))) || any(trimws(names(x)) == "")) {
      svySE_abort(
        title = "Lista sin nombres validos / List without valid names.",
        details = paste(
          "Cuando `x` contiene varios resultados, todos sus elementos deben",
          "tener nombres no vacios."
        ),
        hint = paste0(
          "Ejemplo: x = list(Vivienda = res_viv, Salud = res_salud, ",
          "Educacion = res_simple)."
        )
      )
    }
    
    if (anyDuplicated(names(x)) > 0) {
      svySE_abort(
        title = "Nombres duplicados / Duplicate names.",
        details = "Los nombres de los resultados incluidos en `x` deben ser unicos.",
        vars = list(duplicated_names = unique(names(x)[duplicated(names(x))]))
      )
    }
    
    invalid <- !vapply(x, valid_result, logical(1))
    
    if (any(invalid)) {
      svySE_abort(
        title = "Resultados incompatibles / Incompatible results.",
        details = paste(
          "Todos los elementos de `x` deben ser objetos generados por",
          "`svySE_calc()` o `svySE_simple()`."
        ),
        vars = list(invalid_elements = names(x)[invalid])
      )
    }
    
    results <- x
    use_prefix <- TRUE
  }
  
  # ---------------------------------------------------------------------------
  # Seleccion
  # Selection
  # ---------------------------------------------------------------------------
  
  if (!is.null(select)) {
    svySE_chk_chr(select, "select")
    
    missing_select <- setdiff(select, names(results))
    
    if (length(missing_select) > 0) {
      svySE_abort(
        title = "Resultados no encontrados / Results not found.",
        details = "Algunos nombres indicados en `select` no existen dentro de `x`.",
        vars = list(
          missing = missing_select,
          available = names(results)
        )
      )
    }
    
    results <- results[select]
  }
  
  if (length(results) == 0) {
    svySE_abort(
      title = "No hay resultados para exportar / No results to export."
    )
  }
  
  error_mask <- vapply(
    results,
    inherits,
    logical(1),
    what = "svySE_result"
  )
  
  simple_mask <- vapply(
    results,
    inherits,
    logical(1),
    what = "svySE_simple_result"
  )
  
  error_results <- results[error_mask]
  simple_results <- results[simple_mask]
  
  # ---------------------------------------------------------------------------
  # Validaciones de archivos
  # Output validations
  # ---------------------------------------------------------------------------
  
  if (is.null(file_err) && is.null(file_tab)) {
    svySE_abort(
      title = "No se especifico archivo de salida / No output file specified.",
      details = paste(
        "Debe proporcionarse al menos uno de `file_err` o `file_tab`.",
        "At least one of `file_err` or `file_tab` must be provided."
      ),
      hint = paste(
        "Ejemplo / Example:",
        "svySE_xlsx(x = res, file_err = \"errors.xlsx\", file_tab = NULL)"
      )
    )
  }
  
  if (!is.null(file_err)) {
    svySE_chk_chr(file_err, "file_err")
    
    if (length(file_err) != 1) {
      svySE_abort(
        title = "Archivo de errores invalido / Invalid error output file.",
        details = "`file_err` debe contener una sola ruta / `file_err` must contain one path.",
        vars = list(file_err = file_err)
      )
    }
    
    if (length(error_results) == 0) {
      svySE_abort(
        title = "No hay errores muestrales para exportar / No sampling errors to export.",
        details = paste(
          "`file_err` fue especificado, pero los resultados seleccionados no",
          "contienen objetos generados por `svySE_calc()`."
        ),
        hint = "Incluye al menos un objeto de clase `svySE_result` o usa `file_err = NULL`."
      )
    }
    
    svySE_chk_cols(cols_err, svySE_cols_err_all(), "cols_err")
  }
  
  if (!is.null(file_tab)) {
    svySE_chk_chr(file_tab, "file_tab")
    
    if (length(file_tab) != 1) {
      svySE_abort(
        title = "Archivo de tabla simple invalido / Invalid simple table output file.",
        details = "`file_tab` debe contener una sola ruta / `file_tab` must contain one path.",
        vars = list(file_tab = file_tab)
      )
    }
    
    if (length(simple_results) == 0) {
      svySE_abort(
        title = "No hay tablas simples para exportar / No simple tables to export.",
        details = paste(
          "`file_tab` fue especificado, pero los resultados seleccionados no",
          "contienen objetos generados por `svySE_simple()`."
        ),
        hint = "Incluye al menos un objeto de clase `svySE_simple_result` o usa `file_tab = NULL`."
      )
    }
    
    svySE_chk_cols(cols_tab, svySE_cols_tab_all(), "cols_tab")
  }
  
  if (!is.null(file_err) && !is.null(file_tab)) {
    same_path <- identical(
      normalizePath(file_err, winslash = "/", mustWork = FALSE),
      normalizePath(file_tab, winslash = "/", mustWork = FALSE)
    )
    
    if (same_path) {
      svySE_abort(
        title = "Rutas de salida duplicadas / Duplicate output paths.",
        details = "`file_err` y `file_tab` deben apuntar a archivos diferentes.",
        vars = list(file_err = file_err, file_tab = file_tab),
        hint = "Usa nombres distintos para ambos archivos de salida."
      )
    }
  }
  
  if (
    !is.numeric(start_row) ||
    length(start_row) != 1 ||
    is.na(start_row) ||
    !is.finite(start_row) ||
    start_row < 4 ||
    start_row %% 1 != 0
  ) {
    svySE_abort(
      title = "Fila inicial invalida / Invalid starting row.",
      details = "`start_row` debe ser un numero entero mayor o igual a 4.",
      vars = list(start_row = start_row)
    )
  }
  
  svySE_chk_bool(overwrite, "overwrite")
  svySE_chk_bool(keep_na, "keep_na")
  
  # ---------------------------------------------------------------------------
  # Exportar errores muestrales
  # Export sampling errors
  # ---------------------------------------------------------------------------
  
  if (!is.null(file_err)) {
    
    wb_err <- openxlsx::createWorkbook()
    used_sheets <- character(0)
    
    for (result_name in names(error_results)) {
      
      obj <- error_results[[result_name]]
      
      for (ind in obj$meta$indicators) {
        
        base_name <- if (use_prefix) {
          paste(result_name, ind, "Error", sep = "_")
        } else {
          paste(ind, "Error", sep = "_")
        }
        
        sheet <- svySE_unique_sheet_name(
          x = base_name,
          used = used_sheets
        )
        
        used_sheets <- c(used_sheets, sheet)
        
        openxlsx::addWorksheet(wb_err, sheet)
        
        svySE_write_err_sheet(
          wb = wb_err,
          sheet = sheet,
          result_ind = obj$results[[ind]]$error,
          group_vars = obj$meta$group_vars,
          group_labels = obj$meta$group_labels,
          cols = cols_err,
          start_row = start_row,
          keep_na = keep_na
        )
      }
    }
    
    openxlsx::saveWorkbook(
      wb = wb_err,
      file = file_err,
      overwrite = overwrite
    )
  }
  
  # ---------------------------------------------------------------------------
  # Exportar tablas simples
  # Export simple tables
  # ---------------------------------------------------------------------------
  
  if (!is.null(file_tab)) {
    
    wb_tab <- openxlsx::createWorkbook()
    used_sheets <- character(0)
    
    for (result_name in names(simple_results)) {
      
      obj <- simple_results[[result_name]]
      
      for (ind in obj$meta$indicators) {
        
        base_name <- if (use_prefix) {
          paste(result_name, ind, "Simple", sep = "_")
        } else {
          paste(ind, "Simple", sep = "_")
        }
        
        sheet <- svySE_unique_sheet_name(
          x = base_name,
          used = used_sheets
        )
        
        used_sheets <- c(used_sheets, sheet)
        
        openxlsx::addWorksheet(wb_tab, sheet)
        
        svySE_write_tab_sheet(
          wb = wb_tab,
          sheet = sheet,
          result_ind = obj$results[[ind]]$simple,
          group_vars = obj$meta$group_vars,
          group_labels = obj$meta$group_labels,
          cols = cols_tab,
          start_row = start_row,
          keep_na = keep_na
        )
      }
    }
    
    openxlsx::saveWorkbook(
      wb = wb_tab,
      file = file_tab,
      overwrite = overwrite
    )
  }
  
  invisible(
    list(
      file_err = file_err,
      file_tab = file_tab,
      selected = names(results),
      error_results = names(error_results),
      simple_results = names(simple_results)
    )
  )
}


# ==============================================================================
# Escritura de hoja de errores
# Error sheet writing
# ==============================================================================

#' @keywords internal
svySE_write_err_sheet <- function(
    wb,
    sheet,
    result_ind,
    group_vars,
    group_labels,
    cols,
    start_row,
    keep_na = FALSE
) {
  
  col_now <- 1
  divisions <- names(result_ind)
  
  for (i in seq_along(divisions)) {
    
    div <- divisions[i]
    is_total <- i == 1
    
    tab <- result_ind[[div]]
    
    if (is_total) {
      tab_out <- tab[, c(group_vars, cols), drop = FALSE]
    } else {
      tab_out <- tab[, cols, drop = FALSE]
    }
    
    svySE_head_err(
      wb = wb,
      sheet = sheet,
      col_start = col_now,
      row_data = start_row,
      group_labels = group_labels,
      only_values = !is_total,
      division_name = div,
      cols = cols
    )
    
    openxlsx::writeData(
      wb = wb,
      sheet = sheet,
      x = tab_out,
      startRow = start_row,
      startCol = col_now,
      colNames = FALSE,
      keepNA = keep_na
    )
    
    width <- if (is_total) {
      length(group_vars) + length(cols)
    } else {
      length(cols)
    }
    
    col_now <- col_now + width
  }
  
  svySE_style_sheet(
    wb = wb,
    sheet = sheet,
    max_col = col_now - 1,
    start_row = start_row
  )
  
  invisible(TRUE)
}


# ==============================================================================
# Escritura de hoja simple
# Simple sheet writing
# ==============================================================================

#' @keywords internal
svySE_write_tab_sheet <- function(
    wb,
    sheet,
    result_ind,
    group_vars,
    group_labels,
    cols,
    start_row,
    keep_na = FALSE
) {
  
  col_now <- 1
  divisions <- names(result_ind)
  
  for (i in seq_along(divisions)) {
    
    div <- divisions[i]
    is_total <- i == 1
    
    tab <- result_ind[[div]]
    
    if (is_total) {
      tab_out <- tab[, c(group_vars, cols), drop = FALSE]
    } else {
      tab_out <- tab[, cols, drop = FALSE]
    }
    
    svySE_head_tab(
      wb = wb,
      sheet = sheet,
      col_start = col_now,
      row_data = start_row,
      group_labels = group_labels,
      only_values = !is_total,
      division_name = div,
      cols = cols
    )
    
    openxlsx::writeData(
      wb = wb,
      sheet = sheet,
      x = tab_out,
      startRow = start_row,
      startCol = col_now,
      colNames = FALSE,
      keepNA = keep_na
    )
    
    width <- if (is_total) {
      length(group_vars) + length(cols)
    } else {
      length(cols)
    }
    
    col_now <- col_now + width
  }
  
  svySE_style_sheet(
    wb = wb,
    sheet = sheet,
    max_col = col_now - 1,
    start_row = start_row
  )
  
  invisible(TRUE)
}


# ==============================================================================
# Encabezados de tabla de errores
# Error table headers
# ==============================================================================

#' @keywords internal
svySE_head_err <- function(
    wb,
    sheet,
    col_start,
    row_data,
    group_labels,
    only_values = FALSE,
    division_name = NULL,
    cols
) {
  
  row_div <- row_data - 3
  row_top <- row_data - 2
  row_low <- row_data - 1
  
  col_met <- if (only_values) {
    col_start
  } else {
    col_start + length(group_labels)
  }
  
  if (!only_values) {
    for (j in seq_along(group_labels)) {
      openxlsx::writeData(
        wb,
        sheet,
        x = data.frame(V1 = group_labels[j]),
        startRow = row_div,
        startCol = col_start + j - 1,
        colNames = FALSE
      )
    }
  }
  
  if (!is.null(division_name)) {
    openxlsx::writeData(
      wb,
      sheet,
      x = data.frame(V1 = division_name),
      startRow = row_div,
      startCol = col_met,
      colNames = FALSE
    )
  }
  
  labels <- svySE_lbl_err(cols)
  sub_labels <- svySE_sublbl_err(cols)
  
  for (j in seq_along(cols)) {
    
    openxlsx::writeData(
      wb,
      sheet,
      x = data.frame(V1 = labels[j]),
      startRow = row_top,
      startCol = col_met + j - 1,
      colNames = FALSE
    )
    
    if (!is.na(sub_labels[j])) {
      openxlsx::writeData(
        wb,
        sheet,
        x = data.frame(V1 = sub_labels[j]),
        startRow = row_low,
        startCol = col_met + j - 1,
        colNames = FALSE
      )
    }
  }
  
  invisible(TRUE)
}


# ==============================================================================
# Encabezados de tabla simple
# Simple table headers
# ==============================================================================

#' @keywords internal
svySE_head_tab <- function(
    wb,
    sheet,
    col_start,
    row_data,
    group_labels,
    only_values = FALSE,
    division_name = NULL,
    cols
) {
  
  row_div <- row_data - 3
  row_top <- row_data - 2
  row_low <- row_data - 1
  
  col_met <- if (only_values) {
    col_start
  } else {
    col_start + length(group_labels)
  }
  
  if (!only_values) {
    for (j in seq_along(group_labels)) {
      openxlsx::writeData(
        wb,
        sheet,
        x = data.frame(V1 = group_labels[j]),
        startRow = row_div,
        startCol = col_start + j - 1,
        colNames = FALSE
      )
    }
  }
  
  if (!is.null(division_name)) {
    openxlsx::writeData(
      wb,
      sheet,
      x = data.frame(V1 = division_name),
      startRow = row_div,
      startCol = col_met,
      colNames = FALSE
    )
  }
  
  labels <- svySE_lbl_tab(cols)
  
  for (j in seq_along(cols)) {
    
    openxlsx::writeData(
      wb,
      sheet,
      x = data.frame(V1 = labels$top[j]),
      startRow = row_top,
      startCol = col_met + j - 1,
      colNames = FALSE
    )
    
    openxlsx::writeData(
      wb,
      sheet,
      x = data.frame(V1 = labels$low[j]),
      startRow = row_low,
      startCol = col_met + j - 1,
      colNames = FALSE
    )
  }
  
  invisible(TRUE)
}


# ==============================================================================
# Etiquetas de columnas
# Column labels
# ==============================================================================

#' @keywords internal
svySE_lbl_err <- function(cols) {
  
  dict <- c(
    est_abs = "Estimacion",
    est_pct = "Estimacion",
    se_abs = "Error estandar",
    se_pct = "Error estandar",
    ci_l_abs = "95% de intervalo de confianza",
    ci_l_pct = "95% de intervalo de confianza",
    ci_u_abs = "95% de intervalo de confianza",
    ci_u_pct = "95% de intervalo de confianza",
    cv = "Coeficiente de variacion",
    deff = "Efecto de diseno",
    n_unw = "Recuento no ponderado"
  )
  
  unname(dict[cols])
}


#' @keywords internal
svySE_sublbl_err <- function(cols) {
  
  dict <- c(
    est_abs = "Abs.",
    est_pct = "%",
    se_abs = "Abs.",
    se_pct = "%",
    ci_l_abs = "Inf. Abs.",
    ci_l_pct = "Inf. %",
    ci_u_abs = "Sup. Abs.",
    ci_u_pct = "Sup. %",
    cv = NA,
    deff = NA,
    n_unw = NA
  )
  
  unname(dict[cols])
}


#' @keywords internal
svySE_lbl_tab <- function(cols) {
  
  top <- c(
    freq_0 = "0",
    pct_0 = "0",
    freq_1 = "1",
    pct_1 = "1",
    freq_total = "TOTAL",
    pct_total = "TOTAL"
  )
  
  low <- c(
    freq_0 = "Abs.",
    pct_0 = "%",
    freq_1 = "Abs.",
    pct_1 = "%",
    freq_total = "Abs.",
    pct_total = "%"
  )
  
  data.frame(
    top = unname(top[cols]),
    low = unname(low[cols]),
    stringsAsFactors = FALSE
  )
}


# ==============================================================================
# Estilos de .xlsx
# .xlsx styles
# ==============================================================================

#' @keywords internal
svySE_style_sheet <- function(
    wb,
    sheet,
    max_col,
    start_row
) {
  
  header_style <- openxlsx::createStyle(
    textDecoration = "bold",
    halign = "center",
    valign = "center",
    border = "TopBottomLeftRight",
    wrapText = TRUE
  )
  
  openxlsx::addStyle(
    wb = wb,
    sheet = sheet,
    style = header_style,
    rows = (start_row - 3):(start_row - 1),
    cols = 1:max_col,
    gridExpand = TRUE,
    stack = TRUE
  )
  
  openxlsx::setColWidths(
    wb = wb,
    sheet = sheet,
    cols = 1:max_col,
    widths = "auto"
  )
  
  openxlsx::freezePane(
    wb = wb,
    sheet = sheet,
    firstActiveRow = start_row,
    firstActiveCol = 1
  )
  
  invisible(TRUE)
}


# ==============================================================================
# Utilidades
# Utilities
# ==============================================================================

#' @keywords internal
svySE_sheet_name <- function(x) {
  
  x <- gsub("[\\[\\]\\:\\*\\?\\/\\\\]", "_", x)
  
  if (nchar(x) > 31) {
    x <- substr(x, 1, 31)
  }
  
  x
}
#' @keywords internal
svySE_unique_sheet_name <- function(
    x,
    used = character(0)
) {
  
  base <- svySE_sheet_name(x)
  
  if (!(base %in% used)) {
    return(base)
  }
  
  i <- 2L
  
  repeat {
    
    suffix <- paste0("_", i)
    max_base <- 31L - nchar(suffix)
    
    candidate <- paste0(
      substr(base, 1L, max_base),
      suffix
    )
    
    if (!(candidate %in% used)) {
      return(candidate)
    }
    
    i <- i + 1L
  }
}

# ==============================================================================
# Tests: exportacion de resultados
# Tests: result export
# Archivo / File: tests/testthat/test-exportar.R
# ==============================================================================


make_export_data <- function() {

  set.seed(789)

  data.frame(
    dept = rep(c("A", "B"), each = 30),
    weight = runif(60, 10, 50),
    ind_1 = rep(c(0, 1), length.out = 60),
    ind_2 = rep(c(1, 0, 1, 0, 0), length.out = 60),
    stringsAsFactors = FALSE
  )
}


make_export_error_result <- function(indicators = "ind_1") {

  cfg <- svySE_cfg(
    estimator = "prop",
    target = 1,
    valid_values = c(0, 1),
    lonely_psu = "adjust"
  )

  svySE_calc(
    data = make_export_data(),
    indicators = indicators,
    group_vars = "dept",
    group_labels = "Department",
    strata = NULL,
    cluster = NULL,
    weight = "weight",
    cfg = cfg,
    verbose = FALSE
  )
}


make_export_simple_result <- function(indicators = "ind_1") {

  svySE_simple(
    data = make_export_data(),
    indicators = indicators,
    group_vars = "dept",
    group_labels = "Department",
    verbose = FALSE
  )
}


test_that("svySE_xlsx exports one sampling error result", {

  skip_if_not_installed("survey")
  skip_if_not_installed("openxlsx")

  file_err <- tempfile(fileext = ".xlsx")

  output <- svySE_xlsx(
    x = make_export_error_result(),
    file_err = file_err,
    file_tab = NULL,
    overwrite = TRUE
  )

  expect_true(file.exists(file_err))
  expect_identical(output$file_err, file_err)
  expect_null(output$file_tab)
  expect_identical(output$selected, "Result")
  expect_identical(output$error_results, "Result")
  expect_length(output$simple_results, 0)

  expect_identical(
    openxlsx::getSheetNames(file_err),
    "ind_1_Error"
  )
})


test_that("svySE_xlsx exports one simple result", {

  skip_if_not_installed("openxlsx")

  file_tab <- tempfile(fileext = ".xlsx")

  output <- svySE_xlsx(
    x = make_export_simple_result(),
    file_err = NULL,
    file_tab = file_tab,
    overwrite = TRUE
  )

  expect_true(file.exists(file_tab))
  expect_null(output$file_err)
  expect_identical(output$file_tab, file_tab)
  expect_identical(output$selected, "Result")
  expect_length(output$error_results, 0)
  expect_identical(output$simple_results, "Result")

  expect_identical(
    openxlsx::getSheetNames(file_tab),
    "ind_1_Simple"
  )
})


test_that("svySE_xlsx exports multiple indicators from individual results", {

  skip_if_not_installed("survey")
  skip_if_not_installed("openxlsx")

  file_err <- tempfile(fileext = ".xlsx")
  file_tab <- tempfile(fileext = ".xlsx")

  svySE_xlsx(
    x = make_export_error_result(c("ind_1", "ind_2")),
    file_err = file_err,
    file_tab = NULL,
    overwrite = TRUE
  )

  svySE_xlsx(
    x = make_export_simple_result(c("ind_1", "ind_2")),
    file_err = NULL,
    file_tab = file_tab,
    overwrite = TRUE
  )

  expect_setequal(
    openxlsx::getSheetNames(file_err),
    c("ind_1_Error", "ind_2_Error")
  )

  expect_setequal(
    openxlsx::getSheetNames(file_tab),
    c("ind_1_Simple", "ind_2_Simple")
  )
})


test_that("svySE_xlsx consolidates named error and simple results", {

  skip_if_not_installed("survey")
  skip_if_not_installed("openxlsx")

  results <- list(
    Housing = make_export_error_result(c("ind_1", "ind_2")),
    Health = make_export_error_result("ind_1"),
    Education = make_export_simple_result(c("ind_1", "ind_2"))
  )

  file_err <- tempfile(fileext = ".xlsx")
  file_tab <- tempfile(fileext = ".xlsx")

  output <- svySE_xlsx(
    x = results,
    file_err = file_err,
    file_tab = file_tab,
    overwrite = TRUE
  )

  expect_true(file.exists(file_err))
  expect_true(file.exists(file_tab))

  expect_identical(
    output$selected,
    c("Housing", "Health", "Education")
  )
  expect_identical(output$error_results, c("Housing", "Health"))
  expect_identical(output$simple_results, "Education")

  error_sheets <- openxlsx::getSheetNames(file_err)
  simple_sheets <- openxlsx::getSheetNames(file_tab)

  expect_length(error_sheets, 3)
  expect_length(simple_sheets, 2)

  expect_true("Housing_ind_1_Error" %in% error_sheets)
  expect_true("Housing_ind_2_Error" %in% error_sheets)
  expect_true("Health_ind_1_Error" %in% error_sheets)
  expect_true("Education_ind_1_Simple" %in% simple_sheets)
  expect_true("Education_ind_2_Simple" %in% simple_sheets)
})


test_that("svySE_xlsx exports only selected named results", {

  skip_if_not_installed("survey")
  skip_if_not_installed("openxlsx")

  results <- list(
    Housing = make_export_error_result("ind_1"),
    Health = make_export_error_result("ind_2"),
    Education = make_export_simple_result("ind_1")
  )

  file_err <- tempfile(fileext = ".xlsx")

  output <- svySE_xlsx(
    x = results,
    select = "Health",
    file_err = file_err,
    file_tab = NULL,
    overwrite = TRUE
  )

  expect_identical(output$selected, "Health")
  expect_identical(output$error_results, "Health")
  expect_length(output$simple_results, 0)

  expect_identical(
    openxlsx::getSheetNames(file_err),
    "Health_ind_2_Error"
  )
})


test_that("svySE_xlsx supports custom column selections", {

  skip_if_not_installed("survey")
  skip_if_not_installed("openxlsx")

  file_err <- tempfile(fileext = ".xlsx")
  file_tab <- tempfile(fileext = ".xlsx")

  svySE_xlsx(
    x = make_export_error_result(),
    file_err = file_err,
    file_tab = NULL,
    cols_err = c("est_pct", "cv", "n_unw"),
    overwrite = TRUE
  )

  svySE_xlsx(
    x = make_export_simple_result(),
    file_err = NULL,
    file_tab = file_tab,
    cols_tab = c("freq_1", "pct_1", "freq_total"),
    overwrite = TRUE
  )

  error_data <- openxlsx::read.xlsx(
    file_err,
    sheet = 1,
    colNames = FALSE,
    rows = 10:12
  )

  simple_data <- openxlsx::read.xlsx(
    file_tab,
    sheet = 1,
    colNames = FALSE,
    rows = 10:12
  )

  expect_equal(ncol(error_data), 4)
  expect_equal(ncol(simple_data), 4)
})


test_that("svySE_xlsx requires at least one output file", {

  skip_if_not_installed("survey")
  skip_if_not_installed("openxlsx")

  expect_error(
    svySE_xlsx(
      x = make_export_error_result(),
      file_err = NULL,
      file_tab = NULL
    ),
    class = "svySE_error"
  )
})


test_that("svySE_xlsx rejects incompatible output type requests", {

  skip_if_not_installed("survey")
  skip_if_not_installed("openxlsx")

  expect_error(
    svySE_xlsx(
      x = make_export_error_result(),
      file_err = NULL,
      file_tab = tempfile(fileext = ".xlsx")
    ),
    class = "svySE_error"
  )

  expect_error(
    svySE_xlsx(
      x = make_export_simple_result(),
      file_err = tempfile(fileext = ".xlsx"),
      file_tab = NULL
    ),
    class = "svySE_error"
  )
})


test_that("svySE_xlsx validates named result lists", {

  skip_if_not_installed("survey")
  skip_if_not_installed("openxlsx")

  unnamed <- list(
    make_export_error_result(),
    make_export_simple_result()
  )

  expect_error(
    svySE_xlsx(
      x = unnamed,
      file_err = tempfile(fileext = ".xlsx"),
      file_tab = tempfile(fileext = ".xlsx")
    ),
    class = "svySE_error"
  )

  duplicated <- list(
    Result = make_export_error_result(),
    Result = make_export_simple_result()
  )

  expect_error(
    svySE_xlsx(
      x = duplicated,
      file_err = tempfile(fileext = ".xlsx"),
      file_tab = tempfile(fileext = ".xlsx")
    ),
    class = "svySE_error"
  )

  incompatible <- list(
    Valid = make_export_error_result(),
    Invalid = data.frame(x = 1:3)
  )

  expect_error(
    svySE_xlsx(
      x = incompatible,
      file_err = tempfile(fileext = ".xlsx"),
      file_tab = NULL
    ),
    class = "svySE_error"
  )
})


test_that("svySE_xlsx validates select names", {

  skip_if_not_installed("survey")
  skip_if_not_installed("openxlsx")

  results <- list(
    Housing = make_export_error_result(),
    Education = make_export_simple_result()
  )

  expect_error(
    svySE_xlsx(
      x = results,
      select = "Missing",
      file_err = tempfile(fileext = ".xlsx"),
      file_tab = NULL
    ),
    class = "svySE_error"
  )
})


test_that("svySE_xlsx rejects identical output paths", {

  skip_if_not_installed("survey")
  skip_if_not_installed("openxlsx")

  results <- list(
    Errors = make_export_error_result(),
    Simple = make_export_simple_result()
  )

  same_file <- tempfile(fileext = ".xlsx")

  expect_error(
    svySE_xlsx(
      x = results,
      file_err = same_file,
      file_tab = same_file
    ),
    class = "svySE_error"
  )
})


test_that("svySE_xlsx validates start_row and logical arguments", {

  skip_if_not_installed("survey")
  skip_if_not_installed("openxlsx")

  result <- make_export_error_result()
  file_err <- tempfile(fileext = ".xlsx")

  expect_error(
    svySE_xlsx(
      x = result,
      file_err = file_err,
      start_row = 3
    ),
    class = "svySE_error"
  )

  expect_error(
    svySE_xlsx(
      x = result,
      file_err = file_err,
      start_row = 4.5
    ),
    class = "svySE_error"
  )

  expect_error(
    svySE_xlsx(
      x = result,
      file_err = file_err,
      overwrite = 1
    )
  )

  expect_error(
    svySE_xlsx(
      x = result,
      file_err = file_err,
      keep_na = NA
    )
  )
})


test_that("sheet-name helper creates valid and unique names", {

  long_name <- paste(rep("VeryLongResultName", 4), collapse = "_")

  first <- svySE_unique_sheet_name(long_name)
  second <- svySE_unique_sheet_name(long_name, used = first)

  expect_lte(nchar(first), 31)
  expect_lte(nchar(second), 31)
  expect_false(identical(first, second))
  expect_match(second, "_2$")
})


test_that("sheet-name helper removes invalid XLSX characters", {

  cleaned <- svySE_sheet_name("A/B:C*D?E[F]")

  expect_false(grepl("[/\\:*?\\[\\]]", cleaned))
  expect_lte(nchar(cleaned), 31)
})

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.align = "center"
)

## ----example-data-------------------------------------------------------------
library(svySE)

set.seed(123)

df <- data.frame(
  dept = rep(c("A", "B", "C"), each = 50),
  strata = rep(c("S1", "S2", "S3"), each = 50),
  cluster = rep(seq_len(30), each = 5),
  service = rep(c("S1", "S2"), length.out = 150),
  weight = runif(150, 10, 50),
  ind_1 = sample(c(0, 1), 150, replace = TRUE),
  ind_2 = sample(c(0, 1), 150, replace = TRUE),
  stringsAsFactors = FALSE
)

head(df)

## ----configuration------------------------------------------------------------
cfg <- svySE_cfg(
  estimator = "prop",
  variance = "taylor",
  lonely_psu = "adjust",
  conf_level = 0.95,
  target = 1,
  valid_values = c(0, 1),
  truncate_lower_ci = TRUE,
  pct_mult = 100,
  deff = TRUE,
  cv = TRUE,
  na_rm = TRUE
)

cfg

## ----sampling-errors----------------------------------------------------------
res_error <- svySE_calc(
  data = df,
  indicators = c("ind_1", "ind_2"),
  group_vars = "dept",
  group_labels = "Department",
  strata = "strata",
  cluster = "cluster",
  weight = "weight",
  division = NULL,
  div_weight = NULL,
  cfg = cfg,
  verbose = FALSE
)

res_error

## ----sampling-error-class-----------------------------------------------------
class(res_error)

## ----inspect-error-object-----------------------------------------------------
names(res_error$results)
names(res_error$results$ind_1$error)

## ----inspect-error-table------------------------------------------------------
res_error$results$ind_1$error$TOTAL

## ----design-weight-only-------------------------------------------------------
res_weight <- svySE_calc(
  data = df,
  indicators = "ind_1",
  group_vars = "dept",
  group_labels = "Department",
  strata = NULL,
  cluster = NULL,
  weight = "weight",
  cfg = cfg,
  verbose = FALSE
)

res_weight

## ----design-stratified--------------------------------------------------------
res_strata <- svySE_calc(
  data = df,
  indicators = "ind_1",
  group_vars = "dept",
  group_labels = "Department",
  strata = "strata",
  cluster = NULL,
  weight = "weight",
  cfg = cfg,
  verbose = FALSE
)

res_strata

## ----design-clustered---------------------------------------------------------
res_cluster <- svySE_calc(
  data = df,
  indicators = "ind_1",
  group_vars = "dept",
  group_labels = "Department",
  strata = NULL,
  cluster = "cluster",
  weight = "weight",
  cfg = cfg,
  verbose = FALSE
)

res_cluster

## ----design-complex-----------------------------------------------------------
res_complex <- svySE_calc(
  data = df,
  indicators = "ind_1",
  group_vars = "dept",
  group_labels = "Department",
  strata = "strata",
  cluster = "cluster",
  weight = "weight",
  cfg = cfg,
  verbose = FALSE
)

res_complex

## ----domain-estimation--------------------------------------------------------
res_domain <- svySE_calc(
  data = df,
  indicators = "ind_1",
  group_vars = "dept",
  group_labels = "Department",
  strata = "strata",
  cluster = "cluster",
  weight = "weight",
  division = "service",
  div_weight = NULL,
  cfg = cfg,
  verbose = FALSE
)

names(res_domain$results$ind_1$error)

## ----inspect-domain-----------------------------------------------------------
res_domain$results$ind_1$error$S1

## ----simple-tables------------------------------------------------------------
res_simple <- svySE_simple(
  data = df,
  indicators = c("ind_1", "ind_2"),
  group_vars = "dept",
  group_labels = "Department",
  division = NULL,
  target = 1,
  valid_values = c(0, 1),
  pct_mult = 100,
  verbose = FALSE
)

res_simple

## ----simple-class-------------------------------------------------------------
class(res_simple)

## ----inspect-simple-----------------------------------------------------------
res_simple$results$ind_1$simple$TOTAL

## ----simple-domain------------------------------------------------------------
res_simple_domain <- svySE_simple(
  data = df,
  indicators = "ind_1",
  group_vars = "dept",
  group_labels = "Department",
  division = "service",
  target = 1,
  valid_values = c(0, 1),
  pct_mult = 100,
  verbose = FALSE
)

names(res_simple_domain$results$ind_1$simple)

## ----export-error-------------------------------------------------------------
file_err <- tempfile(fileext = ".xlsx")

export_error <- svySE_xlsx(
  x = res_error,
  file_err = file_err,
  file_tab = NULL,
  cols_err = svySE_cols_err("full"),
  overwrite = TRUE
)

file.exists(file_err)

## ----export-simple------------------------------------------------------------
file_tab <- tempfile(fileext = ".xlsx")

export_simple <- svySE_xlsx(
  x = res_simple,
  file_err = NULL,
  file_tab = file_tab,
  cols_tab = svySE_cols_tab("full"),
  overwrite = TRUE
)

file.exists(file_tab)

## ----multiple-results---------------------------------------------------------
results <- list(
  Main_errors = res_error,
  Domain_errors = res_domain,
  Main_simple = res_simple,
  Domain_simple = res_simple_domain
)

## ----export-multiple----------------------------------------------------------
multiple_err <- tempfile(fileext = ".xlsx")
multiple_tab <- tempfile(fileext = ".xlsx")

export_multiple <- svySE_xlsx(
  x = results,
  file_err = multiple_err,
  file_tab = multiple_tab,
  cols_err = svySE_cols_err("full"),
  cols_tab = svySE_cols_tab("full"),
  overwrite = TRUE
)

file.exists(multiple_err)
file.exists(multiple_tab)

## ----select-errors------------------------------------------------------------
selected_err <- tempfile(fileext = ".xlsx")

svySE_xlsx(
  x = results,
  select = c("Main_errors", "Domain_errors"),
  file_err = selected_err,
  file_tab = NULL,
  overwrite = TRUE
)

file.exists(selected_err)

## ----select-simple------------------------------------------------------------
selected_tab <- tempfile(fileext = ".xlsx")

svySE_xlsx(
  x = results,
  select = c("Main_simple", "Domain_simple"),
  file_err = NULL,
  file_tab = selected_tab,
  overwrite = TRUE
)

file.exists(selected_tab)

## ----error-columns-full-------------------------------------------------------
svySE_cols_err("full")

## ----error-columns-custom-----------------------------------------------------
error_columns <- svySE_cols_err(
  type = "custom",
  cols = c(
    "est_pct",
    "se_pct",
    "ci_l_pct",
    "ci_u_pct",
    "cv",
    "deff",
    "n_unw"
  )
)

error_columns

## ----export-custom-errors-----------------------------------------------------
custom_err <- tempfile(fileext = ".xlsx")

svySE_xlsx(
  x = res_error,
  file_err = custom_err,
  file_tab = NULL,
  cols_err = error_columns,
  overwrite = TRUE
)

file.exists(custom_err)

## ----simple-columns-full------------------------------------------------------
svySE_cols_tab("full")

## ----simple-columns-custom----------------------------------------------------
simple_columns <- svySE_cols_tab(
  type = "custom",
  cols = c(
    "freq_1",
    "pct_1",
    "freq_total"
  )
)

simple_columns

## ----export-custom-simple-----------------------------------------------------
custom_tab <- tempfile(fileext = ".xlsx")

svySE_xlsx(
  x = res_simple,
  file_err = NULL,
  file_tab = custom_tab,
  cols_tab = simple_columns,
  overwrite = TRUE
)

file.exists(custom_tab)

---
title: "Basic Workflow with svySE"
author: "Luis Burgos"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 2
vignette: >
  %\VignetteIndexEntry{Basic Workflow with svySE}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.align = "center"
)
```

# Introduction

`svySE` provides a structured workflow for producing indicator tables from
complex survey data.

The package separates two complementary tasks:

1. `svySE_calc()` calculates weighted estimates and sampling errors using
   sampling weights and, when available, strata and cluster variables.
2. `svySE_simple()` calculates unweighted frequencies and percentages from the
   observed sample without using the survey design.

Results generated by either function can be exported to `.xlsx` files with
`svySE_xlsx()`. The export function can process one result or consolidate
several analyses generated from different datasets, indicators, weights, or
function calls.

The package is built on top of `survey` for design-based estimation and uses
`openxlsx` for workbook generation.

# Package workflow

| Step | Function | Purpose |
|------|----------|---------|
| 1 | `svySE_cfg()` | Configure estimation options |
| 2A | `svySE_calc()` | Calculate weighted estimates and sampling errors |
| 2B | `svySE_simple()` | Calculate unweighted frequencies and percentages |
| 3 | `svySE_xlsx()` | Export one or multiple results to `.xlsx` files |

The two calculation functions are intentionally separated. This avoids
performing unnecessary calculations when only sampling errors or only
descriptive tables are required.

# Example data

The following simulated dataset contains:

- one grouping variable;
- one stratification variable;
- one cluster variable;
- one sampling weight;
- one division variable;
- two binary indicators.

```{r example-data}
library(svySE)

set.seed(123)

df <- data.frame(
  dept = rep(c("A", "B", "C"), each = 50),
  strata = rep(c("S1", "S2", "S3"), each = 50),
  cluster = rep(seq_len(30), each = 5),
  service = rep(c("S1", "S2"), length.out = 150),
  weight = runif(150, 10, 50),
  ind_1 = sample(c(0, 1), 150, replace = TRUE),
  ind_2 = sample(c(0, 1), 150, replace = TRUE),
  stringsAsFactors = FALSE
)

head(df)
```

# Configure sampling error estimation

`svySE_cfg()` defines the settings used by `svySE_calc()`.

```{r configuration}
cfg <- svySE_cfg(
  estimator = "prop",
  variance = "taylor",
  lonely_psu = "adjust",
  conf_level = 0.95,
  target = 1,
  valid_values = c(0, 1),
  truncate_lower_ci = TRUE,
  pct_mult = 100,
  deff = TRUE,
  cv = TRUE,
  na_rm = TRUE
)

cfg
```

The principal options are:

| Argument | Purpose |
|----------|---------|
| `estimator` | Selects the estimator used in the analysis |
| `variance` | Defines the variance estimation method |
| `lonely_psu` | Specifies how strata containing one PSU are handled |
| `conf_level` | Defines the confidence level |
| `target` | Identifies the indicator category of interest |
| `valid_values` | Defines the valid indicator values |
| `truncate_lower_ci` | Truncates negative lower confidence limits at zero |
| `pct_mult` | Converts proportions to the selected percentage scale |
| `deff` | Enables or disables design-effect calculation |
| `cv` | Enables or disables coefficient-of-variation calculation |
| `na_rm` | Controls the treatment of missing values |

# Calculate sampling errors

`svySE_calc()` calculates weighted estimates and measures of sampling
precision.

```{r sampling-errors}
res_error <- svySE_calc(
  data = df,
  indicators = c("ind_1", "ind_2"),
  group_vars = "dept",
  group_labels = "Department",
  strata = "strata",
  cluster = "cluster",
  weight = "weight",
  division = NULL,
  div_weight = NULL,
  cfg = cfg,
  verbose = FALSE
)

res_error
```

The returned object has class `"svySE_result"`.

```{r sampling-error-class}
class(res_error)
```

Results are organized by indicator and division.

```{r inspect-error-object}
names(res_error$results)
names(res_error$results$ind_1$error)
```

The national and grouped results for the first indicator can be inspected with:

```{r inspect-error-table}
res_error$results$ind_1$error$TOTAL
```

The sampling error table contains:

| Column | Description |
|--------|-------------|
| `est_abs` | Weighted absolute estimate |
| `est_pct` | Weighted percentage estimate |
| `se_abs` | Standard error of the absolute estimate |
| `se_pct` | Standard error of the percentage |
| `ci_l_abs` | Lower confidence limit for the absolute estimate |
| `ci_l_pct` | Lower confidence limit for the percentage |
| `ci_u_abs` | Upper confidence limit for the absolute estimate |
| `ci_u_pct` | Upper confidence limit for the percentage |
| `cv` | Coefficient of variation |
| `deff` | Design effect |
| `n_unw` | Unweighted number of target cases |

# Supported survey designs

`svySE_calc()` can be used with different combinations of design variables.

| Design | `strata` | `cluster` |
|--------|----------|-----------|
| Weight only | `NULL` | `NULL` |
| Stratified | Variable name | `NULL` |
| Clustered | `NULL` | Variable name |
| Stratified and clustered | Variable name | Variable name |

## Weight-only design

```{r design-weight-only}
res_weight <- svySE_calc(
  data = df,
  indicators = "ind_1",
  group_vars = "dept",
  group_labels = "Department",
  strata = NULL,
  cluster = NULL,
  weight = "weight",
  cfg = cfg,
  verbose = FALSE
)

res_weight
```

## Stratified design

```{r design-stratified}
res_strata <- svySE_calc(
  data = df,
  indicators = "ind_1",
  group_vars = "dept",
  group_labels = "Department",
  strata = "strata",
  cluster = NULL,
  weight = "weight",
  cfg = cfg,
  verbose = FALSE
)

res_strata
```

## Clustered design

```{r design-clustered}
res_cluster <- svySE_calc(
  data = df,
  indicators = "ind_1",
  group_vars = "dept",
  group_labels = "Department",
  strata = NULL,
  cluster = "cluster",
  weight = "weight",
  cfg = cfg,
  verbose = FALSE
)

res_cluster
```

## Stratified and clustered design

```{r design-complex}
res_complex <- svySE_calc(
  data = df,
  indicators = "ind_1",
  group_vars = "dept",
  group_labels = "Department",
  strata = "strata",
  cluster = "cluster",
  weight = "weight",
  cfg = cfg,
  verbose = FALSE
)

res_complex
```

# Domain estimation

The `division` argument generates separate results for each observed category
of a division variable.

```{r domain-estimation}
res_domain <- svySE_calc(
  data = df,
  indicators = "ind_1",
  group_vars = "dept",
  group_labels = "Department",
  strata = "strata",
  cluster = "cluster",
  weight = "weight",
  division = "service",
  div_weight = NULL,
  cfg = cfg,
  verbose = FALSE
)

names(res_domain$results$ind_1$error)
```

The `TOTAL` element contains the complete estimate, while the remaining
elements contain the corresponding division results.

```{r inspect-domain}
res_domain$results$ind_1$error$S1
```

When a division requires a different expansion factor, it can be provided
through `div_weight`.

# Calculate simple indicator tables

`svySE_simple()` calculates unweighted frequencies and percentages without
requiring sampling weights, strata, or clusters.

```{r simple-tables}
res_simple <- svySE_simple(
  data = df,
  indicators = c("ind_1", "ind_2"),
  group_vars = "dept",
  group_labels = "Department",
  division = NULL,
  target = 1,
  valid_values = c(0, 1),
  pct_mult = 100,
  verbose = FALSE
)

res_simple
```

The returned object has class `"svySE_simple_result"`.

```{r simple-class}
class(res_simple)
```

A simple table can be inspected with:

```{r inspect-simple}
res_simple$results$ind_1$simple$TOTAL
```

The table contains:

| Column | Description |
|--------|-------------|
| `freq_0` | Frequency of non-target cases |
| `pct_0` | Percentage of non-target cases |
| `freq_1` | Frequency of target cases |
| `pct_1` | Percentage of target cases |
| `freq_total` | Total number of valid observations |
| `pct_total` | Total percentage |

The results generated by `svySE_simple()` describe the observed sample only.
Because no sampling weights or design variables are used, these percentages
should not be interpreted as population estimates.

# Simple tables by division

A division variable can also be used with `svySE_simple()`.

```{r simple-domain}
res_simple_domain <- svySE_simple(
  data = df,
  indicators = "ind_1",
  group_vars = "dept",
  group_labels = "Department",
  division = "service",
  target = 1,
  valid_values = c(0, 1),
  pct_mult = 100,
  verbose = FALSE
)

names(res_simple_domain$results$ind_1$simple)
```

# Export sampling errors to XLSX

The following example writes a sampling error workbook to a temporary file.

```{r export-error}
file_err <- tempfile(fileext = ".xlsx")

export_error <- svySE_xlsx(
  x = res_error,
  file_err = file_err,
  file_tab = NULL,
  cols_err = svySE_cols_err("full"),
  overwrite = TRUE
)

file.exists(file_err)
```

# Export simple tables to XLSX

```{r export-simple}
file_tab <- tempfile(fileext = ".xlsx")

export_simple <- svySE_xlsx(
  x = res_simple,
  file_err = NULL,
  file_tab = file_tab,
  cols_tab = svySE_cols_tab("full"),
  overwrite = TRUE
)

file.exists(file_tab)
```

# Export multiple analyses

`svySE_xlsx()` can receive a named list containing several result objects.

```{r multiple-results}
results <- list(
  Main_errors = res_error,
  Domain_errors = res_domain,
  Main_simple = res_simple,
  Domain_simple = res_simple_domain
)
```

The function automatically identifies the class of each object:

- `"svySE_result"` objects are exported to `file_err`;
- `"svySE_simple_result"` objects are exported to `file_tab`.

```{r export-multiple}
multiple_err <- tempfile(fileext = ".xlsx")
multiple_tab <- tempfile(fileext = ".xlsx")

export_multiple <- svySE_xlsx(
  x = results,
  file_err = multiple_err,
  file_tab = multiple_tab,
  cols_err = svySE_cols_err("full"),
  cols_tab = svySE_cols_tab("full"),
  overwrite = TRUE
)

file.exists(multiple_err)
file.exists(multiple_tab)
```

This is useful when several indicators are calculated from different datasets,
survey weights, domains, or analytical processes.

# Export selected results

The `select` argument restricts the export to chosen elements of a named list.

## Select sampling error results

```{r select-errors}
selected_err <- tempfile(fileext = ".xlsx")

svySE_xlsx(
  x = results,
  select = c("Main_errors", "Domain_errors"),
  file_err = selected_err,
  file_tab = NULL,
  overwrite = TRUE
)

file.exists(selected_err)
```

## Select simple results

```{r select-simple}
selected_tab <- tempfile(fileext = ".xlsx")

svySE_xlsx(
  x = results,
  select = c("Main_simple", "Domain_simple"),
  file_err = NULL,
  file_tab = selected_tab,
  overwrite = TRUE
)

file.exists(selected_tab)
```

# Customize exported columns

## Sampling error columns

The complete set of sampling error columns can be requested with:

```{r error-columns-full}
svySE_cols_err("full")
```

A custom selection can be defined with:

```{r error-columns-custom}
error_columns <- svySE_cols_err(
  type = "custom",
  cols = c(
    "est_pct",
    "se_pct",
    "ci_l_pct",
    "ci_u_pct",
    "cv",
    "deff",
    "n_unw"
  )
)

error_columns
```

Use the custom selection during export:

```{r export-custom-errors}
custom_err <- tempfile(fileext = ".xlsx")

svySE_xlsx(
  x = res_error,
  file_err = custom_err,
  file_tab = NULL,
  cols_err = error_columns,
  overwrite = TRUE
)

file.exists(custom_err)
```

## Simple table columns

```{r simple-columns-full}
svySE_cols_tab("full")
```

A custom selection can be defined with:

```{r simple-columns-custom}
simple_columns <- svySE_cols_tab(
  type = "custom",
  cols = c(
    "freq_1",
    "pct_1",
    "freq_total"
  )
)

simple_columns
```

Use the custom selection during export:

```{r export-custom-simple}
custom_tab <- tempfile(fileext = ".xlsx")

svySE_xlsx(
  x = res_simple,
  file_err = NULL,
  file_tab = custom_tab,
  cols_tab = simple_columns,
  overwrite = TRUE
)

file.exists(custom_tab)
```

# Main functions

| Function | Description |
|----------|-------------|
| `svySE_cfg()` | Configure sampling error estimation |
| `svySE_calc()` | Calculate weighted estimates and sampling errors |
| `svySE_simple()` | Calculate unweighted frequencies and percentages |
| `svySE_xlsx()` | Export one or multiple results to `.xlsx` files |
| `svySE_cols_err()` | Select sampling error columns |
| `svySE_cols_tab()` | Select simple table columns |

# Technical foundation

`svySE` uses established R packages for its internal workflow.

| Package | Role |
|---------|------|
| `survey` | Design-based estimation and variance calculation |
| `openxlsx` | Creation and formatting of `.xlsx` workbooks |
| `stats` | Statistical formulas, coefficients, and confidence intervals |
| `svySE` | High-level workflow for indicators, sampling errors, tables, and export |

`svySE` does not replace `survey`. It provides a structured interface for
repeated indicator production, quality measurement, and export.

# Summary

This vignette demonstrated how to:

1. configure sampling error estimation with `svySE_cfg()`;
2. calculate weighted estimates and sampling errors with `svySE_calc()`;
3. work with weight-only, stratified, clustered, and combined designs;
4. calculate domain estimates;
5. generate unweighted descriptive tables with `svySE_simple()`;
6. export sampling errors and simple tables separately;
7. consolidate several analyses in one export process;
8. select specific results before exporting;
9. customize the columns included in `.xlsx` files.

The same workflow can be applied to official survey datasets by specifying the
corresponding indicators, grouping variables, weights, strata, and clusters.

---
title: "Basic Workflow with svySE"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Basic Workflow with svySE}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction

`svySE` provides a workflow for estimating sampling errors for complex survey indicators.

It is designed for binary or categorical indicators and supports weighted estimates, percentages, standard errors, confidence intervals, coefficients of variation, design effects, unweighted counts, grouped estimates, optional division variables, and Excel exports.

# Example data

```{r}
library(svySE)

set.seed(123)

df <- data.frame(
  dept = rep(c("A", "B", "C"), each = 50),
  strata = rep(c("A", "B", "C"), each = 50),
  service = rep(c("S1", "S2"), length.out = 150),
  weight = runif(150, 10, 50),
  ind_1 = sample(c(0, 1), 150, replace = TRUE)
)

head(df)
```

# Configuration

```{r}
cfg <- svySE_cfg(
  estimator = "prop",
  target = 1,
  valid_values = c(0, 1),
  lonely_psu = "adjust",
  truncate_lower_ci = TRUE
)

cfg
```

# Sampling error calculation

```{r}
res <- svySE_calc(
  data = df,
  indicators = "ind_1",
  group_vars = "dept",
  group_labels = "Department",
  strata = "strata",
  weight = "weight",
  cfg = cfg,
  verbose = FALSE
)

res
```

# Inspect results

```{r}
res$results$ind_1$error$TOTAL
```

# Calculation with division variable

```{r}
res_div <- svySE_calc(
  data = df,
  indicators = "ind_1",
  group_vars = "dept",
  group_labels = "Department",
  strata = "strata",
  weight = "weight",
  division = "service",
  div_weight = "weight",
  cfg = cfg,
  verbose = FALSE
)

names(res_div$results$ind_1$error)
```

# Export to Excel

The following example writes files to a temporary directory.

```{r}
file_err <- tempfile(fileext = ".xlsx")
file_tab <- tempfile(fileext = ".xlsx")

svySE_xlsx(
  x = res,
  file_err = file_err,
  file_tab = file_tab,
  cols_err = svySE_cols_err("full"),
  cols_tab = svySE_cols_tab("full")
)

file.exists(file_err)
file.exists(file_tab)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exportar.R
\name{svySE_xlsx}
\alias{svySE_xlsx}
\title{Export svySE results to '.xlsx' files}
\usage{
svySE_xlsx(
  x,
  select = NULL,
  file_err = NULL,
  file_tab = NULL,
  cols_err = svySE_cols_err("full"),
  cols_tab = svySE_cols_tab("full"),
  start_row = 10,
  overwrite = TRUE,
  keep_na = FALSE
)
}
\arguments{
\item{x}{Objeto generado por \code{svySE_calc()} o \code{svySE_simple()}, o una lista
nombrada que contenga varios objetos de estas clases.
Object generated by \code{svySE_calc()} or \code{svySE_simple()}, or a named list
containing multiple objects of these classes.}

\item{select}{Vector de texto opcional con los nombres de los elementos de
\code{x} que se desean exportar. Solo se utiliza cuando \code{x} es una lista de
resultados. Si es \code{NULL}, se exportan todos.
Optional character vector containing the names of the elements in \code{x} to
export. It is only used when \code{x} is a list of results. If \code{NULL}, all
results are exported.}

\item{file_err}{Ruta del archivo \code{.xlsx} para exportar las tablas de errores
muestrales. Si es \code{NULL}, no se exportan.
Path to the \code{.xlsx} file used to export sampling error tables. If \code{NULL},
they are not exported.}

\item{file_tab}{Ruta del archivo \code{.xlsx} para exportar las tablas simples.
Si es \code{NULL}, no se exportan.
Path to the \code{.xlsx} file used to export simple tables. If \code{NULL}, they are
not exported.}

\item{cols_err}{Columnas de error a exportar.
Sampling error columns to export.}

\item{cols_tab}{Columnas de tabla simple a exportar.
Simple table columns to export.}

\item{start_row}{Fila inicial de escritura.
Starting row for writing.}

\item{overwrite}{Sobrescribir archivos existentes.
Whether existing files should be overwritten.}

\item{keep_na}{Mantener \code{NA} en el archivo exportado.
Whether \code{NA} values should be preserved in the exported file.}
}
\value{
Invisiblemente una lista con las rutas de los archivos exportados y
los resultados seleccionados.
Invisibly returns a list containing the exported file paths and selected
result names.
}
\description{
Exporta uno o varios resultados generados por \code{svySE_calc()} y
\code{svySE_simple()} a archivos \code{.xlsx}.
}
\details{
Exports one or more results generated by \code{svySE_calc()} and
\code{svySE_simple()} to \code{.xlsx} files.

Debe especificarse al menos uno de \code{file_err} o \code{file_tab}.

Cuando \code{x} es una lista, sus elementos deben tener nombres unicos. Los
objetos de clase \code{"svySE_result"} se exportan a \code{file_err}, mientras que los
objetos de clase \code{"svySE_simple_result"} se exportan a \code{file_tab}.

At least one of \code{file_err} or \code{file_tab} must be provided.

When \code{x} is a list, its elements must have unique names. Objects of class
\code{"svySE_result"} are exported to \code{file_err}, while objects of class
\code{"svySE_simple_result"} are exported to \code{file_tab}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/config.R
\name{print.svySE_cfg}
\alias{print.svySE_cfg}
\title{Print svySE configuration}
\usage{
\method{print}{svySE_cfg}(x, ...)
}
\arguments{
\item{x}{Objeto de clase \code{"svySE_cfg"}.}

\item{...}{Argumentos adicionales.}
}
\value{
Invisiblemente el objeto \code{x}.
}
\description{
Imprime la configuracion de \code{svySE}.
Prints the \code{svySE} configuration.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/columnas.R
\name{svySE_cols_err_all}
\alias{svySE_cols_err_all}
\title{Available sampling error columns}
\usage{
svySE_cols_err_all()
}
\value{
Vector de nombres de columnas / Character vector of column names.
}
\description{
Devuelve todas las columnas metricas disponibles para la tabla de errores
muestrales.
Returns all available metric columns for the sampling error table.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/columnas.R
\name{svySE_cols_tab_all}
\alias{svySE_cols_tab_all}
\title{Available simple table columns}
\usage{
svySE_cols_tab_all()
}
\value{
Vector de nombres de columnas / Character vector of column names.
}
\description{
Devuelve todas las columnas disponibles para la tabla simple.
Returns all available columns for the simple table.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/config.R
\name{svySE_chk_bool}
\alias{svySE_chk_bool}
\title{Check logical scalar}
\usage{
svySE_chk_bool(x, arg)
}
\arguments{
\item{x}{Objeto a validar / Object to validate.}

\item{arg}{Nombre del argumento / Argument name.}
}
\value{
Invisiblemente \code{TRUE}.
}
\description{
Valida que un argumento sea logico de longitud uno.
Checks whether an argument is a single logical value.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculo.R
\name{print.svySE_result}
\alias{print.svySE_result}
\title{Print svySE result}
\usage{
\method{print}{svySE_result}(x, ...)
}
\arguments{
\item{x}{Objeto de clase \code{"svySE_result"}.}

\item{...}{Argumentos adicionales.}
}
\value{
Invisiblemente \code{x}.
}
\description{
Imprime un resumen del resultado de \code{svySE_calc()}.
}
\details{
Prints a summary of the result from \code{svySE_calc()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculo.R
\name{svySE_calc}
\alias{svySE_calc}
\title{Calculate sampling errors}
\usage{
svySE_calc(
  data,
  indicators,
  group_vars,
  group_labels = group_vars,
  strata,
  weight,
  division = NULL,
  div_weight = NULL,
  cfg = svySE_cfg(),
  verbose = TRUE,
  strict = FALSE
)
}
\arguments{
\item{data}{Base de datos ya cargada en R / Data frame already loaded in R.}

\item{indicators}{Vector con los nombres de los indicadores.}

\item{group_vars}{Variables de agrupacion.}

\item{group_labels}{Etiquetas de las variables de agrupacion.}

\item{strata}{Variable o variables de estrato.}

\item{weight}{Variable de peso principal.}

\item{division}{Variable de division opcional.}

\item{div_weight}{Variable de peso opcional para las divisiones.}

\item{cfg}{Configuracion creada con \code{svySE_cfg()}.}

\item{verbose}{Si es \code{TRUE}, imprime avance del calculo.}

\item{strict}{Si es \code{TRUE}, detiene el calculo cuando encuentra valores
fuera de \code{cfg$valid_values}; si es \code{FALSE}, los excluye con advertencia.}
}
\value{
Objeto de clase \code{"svySE_result"}.
}
\description{
Calcula errores muestrales para uno o mas indicadores usando un data.frame
previamente cargado por el usuario.
}
\details{
Calculates sampling errors for one or more indicators using a data.frame
previously loaded by the user.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/columnas.R
\name{svySE_chk_cols}
\alias{svySE_chk_cols}
\title{Check selected columns}
\usage{
svySE_chk_cols(cols, all_cols, arg = "cols")
}
\arguments{
\item{cols}{Columnas seleccionadas / Selected columns.}

\item{all_cols}{Columnas disponibles / Available columns.}

\item{arg}{Nombre del argumento / Argument name.}
}
\value{
Invisiblemente \code{TRUE}.
}
\description{
Valida que las columnas seleccionadas existan dentro del conjunto permitido.
Checks whether selected columns exist in the allowed column set.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/config.R
\name{svySE_is_cfg}
\alias{svySE_is_cfg}
\title{Check svySE configuration object}
\usage{
svySE_is_cfg(x)
}
\arguments{
\item{x}{Objeto a evaluar / Object to check.}
}
\value{
\code{TRUE} o \code{FALSE}.
}
\description{
Verifica si un objeto pertenece a la clase \code{svySE_cfg}.
Checks whether an object belongs to class \code{svySE_cfg}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/columnas.R
\name{svySE_cols_tab}
\alias{svySE_cols_tab}
\title{Select simple table columns}
\usage{
svySE_cols_tab(type = "full", cols = NULL)
}
\arguments{
\item{type}{Tipo de salida / Output type. Opciones:
\code{"full"}, \code{"target"}, \code{"freq"}, \code{"pct"} o \code{"custom"}.}

\item{cols}{Vector de columnas cuando \code{type = "custom"}.}
}
\value{
Vector de columnas seleccionadas / Selected column vector.
}
\description{
Selecciona columnas para exportar la tabla simple.
Selects columns to export the simple table.
}
\examples{
svySE_cols_tab("full")
svySE_cols_tab("target")
svySE_cols_tab("custom", cols = c("freq_1", "pct_1"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simple.R
\name{svySE_simple}
\alias{svySE_simple}
\title{Calculate unweighted simple indicator tables}
\usage{
svySE_simple(
  data,
  indicators,
  group_vars,
  group_labels = group_vars,
  division = NULL,
  target = 1,
  valid_values = c(0, 1),
  pct_mult = 100,
  verbose = TRUE,
  strict = FALSE
)
}
\arguments{
\item{data}{Base de datos previamente cargada en R.
Data frame previously loaded in R.}

\item{indicators}{Vector de texto con los nombres de los indicadores.
Character vector containing the indicator names.}

\item{group_vars}{Variables utilizadas para agrupar los resultados.
Variables used to group the results.}

\item{group_labels}{Etiquetas de las variables de agrupacion.
Labels for the grouping variables.}

\item{division}{Variable opcional para generar resultados separados por
categoria. Si es \code{NULL}, solo se calcula el total.
Optional variable used to generate results by category. If \code{NULL}, only
the total is calculated.}

\item{target}{Valor que identifica la categoria de interes.
Value identifying the target category.}

\item{valid_values}{Valores permitidos en los indicadores.
Values allowed in the indicators.}

\item{pct_mult}{Multiplicador utilizado para expresar los porcentajes.
Multiplier used to express percentages.}

\item{verbose}{Si es \code{TRUE}, muestra el avance del calculo.
If \code{TRUE}, displays calculation progress.}

\item{strict}{Si es \code{TRUE}, detiene el calculo cuando encuentra valores fuera
de \code{valid_values}. Si es \code{FALSE}, los excluye con una advertencia.
If \code{TRUE}, stops when values outside \code{valid_values} are found. If \code{FALSE},
they are excluded with a warning.}
}
\value{
Objeto de clase \code{"svySE_simple_result"}.
Object of class \code{"svySE_simple_result"}.
}
\description{
Calcula frecuencias y porcentajes simples para uno o mas indicadores sin
utilizar factores de expansion ni variables del diseno muestral.
}
\details{
Calculates unweighted frequencies and percentages for one or more indicators
without using sampling weights or survey design variables.

Los resultados generados por esta funcion describen solamente los registros
observados en la muestra. Como no se utilizan factores de expansion ni
componentes del diseno muestral, los porcentajes no deben interpretarse como
estimaciones representativas de la poblacion.

The results generated by this function describe only the records observed in
the sample. Because sampling weights and survey design components are not
used, percentages should not be interpreted as population estimates.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simple.R
\name{print.svySE_simple_result}
\alias{print.svySE_simple_result}
\title{Print an svySE simple result}
\usage{
\method{print}{svySE_simple_result}(x, ...)
}
\arguments{
\item{x}{Objeto de clase \code{"svySE_simple_result"}.}

\item{...}{Argumentos adicionales.}
}
\value{
Invisiblemente \code{x}.
}
\description{
Imprime un resumen del resultado generado por \code{svySE_simple()}.
}
\details{
Prints a summary of the result generated by \code{svySE_simple()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/config.R
\name{svySE_cfg}
\alias{svySE_cfg}
\title{Configure sampling error parameters}
\usage{
svySE_cfg(
  estimator = "prop",
  variance = "taylor",
  lonely_psu = "adjust",
  conf_level = 0.95,
  target = 1,
  valid_values = c(0, 1),
  truncate_lower_ci = TRUE,
  pct_mult = 100,
  deff = TRUE,
  cv = TRUE,
  na_rm = TRUE
)
}
\arguments{
\item{estimator}{Tipo de estimador. Opciones: \code{"prop"}, \code{"total"},
\code{"mean"} o \code{"ratio"}.}

\item{variance}{Metodo de estimacion de varianza. Actualmente solo \code{"taylor"}.}

\item{lonely_psu}{Tratamiento de estratos con una sola UPM. Opciones:
\code{"adjust"}, \code{"average"}, \code{"certainty"}, \code{"remove"} o \code{"fail"}.}

\item{conf_level}{Nivel de confianza. Por defecto \code{0.95}.}

\item{target}{Valor objetivo del indicador. Por defecto \code{1}.}

\item{valid_values}{Valores validos del indicador. Por defecto \code{c(0, 1)}.}

\item{truncate_lower_ci}{Si es \code{TRUE}, trunca el limite inferior del IC en cero.}

\item{pct_mult}{Multiplicador para expresar proporciones como porcentajes.
Por defecto \code{100}.}

\item{deff}{Si es \code{TRUE}, calcula efecto de diseno cuando sea posible.}

\item{cv}{Si es \code{TRUE}, calcula coeficiente de variacion.}

\item{na_rm}{Si es \code{TRUE}, remueve valores perdidos en operaciones auxiliares.}
}
\value{
Lista de clase \code{"svySE_cfg"}.
}
\description{
Crea una configuracion general para el calculo de errores muestrales.
Creates a general configuration object for sampling error estimation.
}
\examples{
cfg <- svySE_cfg(
  estimator = "prop",
  variance = "taylor",
  target = 1
)

}
